@@ -9,10 +9,10 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
 #include "analyser/analyser.h"
-#include "lib/fastbuf.h"
-#include "lib/chartype.h"
-#include "lib/binsearch.h"
-#include "lib/clists.h"
+#include "ucw/fastbuf.h"
+#include "ucw/chartype.h"
+#include "ucw/binsearch.h"
+#include "ucw/clists.h"
 #include "sherlock/conf.h"
 
 #include <stdio.h>
@@ -26,7 +26,7 @@
 
 struct ip_alias {
   cnode n;
-  byte *name;
+  char *name;
   clist nodes;
 };
 
@@ -123,7 +123,7 @@ ip_nodes_cmp(struct ip_node *x, struct ip_node *y)
 #define ASORT_KEY_TYPE struct ip_node *
 #define ASORT_ELT(i) (ip_nodes[i])
 #define ASORT_LT(x,y) (ip_nodes_cmp((x),(y)))
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static void
 an_iprange_init(struct an_hook *h)
