@@ -12,3 +12,10 @@ AN_MODULE(an_substr)
 AN_MODULE(an_lang)
 #endif
 
+#if defined(CONFIG_IMAGES_DUP) || defined(CONFIG_IMAGES_SIM)
+AN_MODULE(an_imagesig)
+#endif
+
+#ifdef CUSTOM_ANALYSERS
+CUSTOM_ANALYSERS
+#endif
