@@ -9,9 +9,9 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
 #include "analyser/analyser.h"
-#include "lib/mempool.h"
-#include "lib/fastbuf.h"
-#include "lib/chartype.h"
+#include "ucw/mempool.h"
+#include "ucw/fastbuf.h"
+#include "ucw/chartype.h"
 #include "sherlock/tagged-text.h"
 #include "sherlock/conf.h"
 #include "charset/unicat.h"
@@ -63,7 +63,7 @@ static const KMP_CHAR kmp_sentence_break[] = { ':', '.' };
 #define KMPS_STEP(kmp,src,s) s->u.mask |= s->s->u.mask
 #define KMPS_ADD_CONTROLS
 #define KMPS_MERGE_CONTROLS
-#include "lib/kmp.h"
+#include "ucw/kmp.h"
 
 enum substr_parts {
   PART_TEXT = 1,
@@ -73,7 +73,7 @@ enum substr_parts {
 
 struct substr_context {
   cnode n;
-  byte *name;
+  char *name;
   uns attr;
   uns append;
   uns parts;
@@ -84,19 +84,19 @@ struct substr_context {
 
 struct substr_group {
   cnode n;
-  byte *name;
+  char *name;
   clist items;
 };
 
 struct substr_link {
   cnode n;
-  byte *group;
+  char *group;
   uns mask;
 };
 
 struct substr_item {
   cnode n;
-  byte *str;
+  char *str;
 };
 
 static clist contexts;
@@ -177,7 +177,7 @@ static struct cf_section context_config = {
     CF_STRING("Name", F(name)),
     CF_USER("Attr", F(attr), &cf_type_attr),
     CF_UNS("Append", F(append)),
-    CF_BITMAP_LOOKUP("Parts", F(parts), ((byte *[]) {"text", "metas", "urls", NULL })),
+    CF_BITMAP_LOOKUP("Parts", F(parts), ((const char * const []) {"text", "metas", "urls", NULL })),
     CF_LIST("Groups", F(links), &link_config),
     CF_END
   }
