@@ -7,7 +7,7 @@
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
 #include "analyser/analyser.h"
-#include "lib/fastbuf.h"
+#include "ucw/fastbuf.h"
 
 static void
 an_test_init(struct an_hook *h)
