@@ -9,9 +9,9 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
-#include "lib/conf.h"
-#include "lib/fastbuf.h"
-#include "lib/mempool.h"
+#include "ucw/conf.h"
+#include "ucw/fastbuf.h"
+#include "ucw/mempool.h"
 #include "analyser/analyser.h"
 
 #include <string.h>
@@ -30,15 +30,15 @@ static clist an_hooks[AN_HOOK_MAX];
 uns an_trace;
 static uns an_log_stats;
 
-static byte *
+static char *
 hook_init(struct an_hook *h)
 {
   h->a = NULL;
-  h->older_than = ~(sh_time_t)0;
+  h->older_than = ~(ucw_time_t)0;
   return NULL;
 }
 
-static byte *
+static char *
 hook_commit(struct an_hook *h)
 {
   if (!h->a)
@@ -46,8 +46,8 @@ hook_commit(struct an_hook *h)
   return NULL;
 }
 
-static byte *
-analyser_parser(uns number UNUSED, byte **pars, struct analyser **ptr)
+static char *
+analyser_parser(uns number UNUSED, char **pars, struct analyser **ptr)
 {
   CF_JOURNAL_VAR(*ptr);
   uns a;
@@ -58,13 +58,13 @@ analyser_parser(uns number UNUSED, byte **pars, struct analyser **ptr)
   return NULL;
 }
 
-static byte *
-condition_parser(uns number UNUSED, byte **pars, sh_time_t *ptr)
+static char *
+condition_parser(uns number UNUSED, char **pars, ucw_time_t *ptr)
 {
   byte *err;
   uns t;
   if (!strcmp(pars[0], "always"))
-    *ptr = ~(sh_time_t)0;
+    *ptr = ~(ucw_time_t)0;
   else if (!strcmp(pars[0], "needed"))
     *ptr = 0;
   else if (err = cf_parse_int(pars[0], &t))
@@ -164,7 +164,7 @@ static int
 analyser_needed_p(struct an_hook *h, struct an_iface *ai)
 {
   struct analyser *a = h->a;
-  if (h->older_than == ~(sh_time_t)0)
+  if (h->older_than == ~(ucw_time_t)0)
     {
       TRACE("Analyser %s: Always", a->name);
       return 1;
