@@ -7,8 +7,9 @@
 #ifndef _ANALYSER_ANALYSER_H
 #define _ANALYSER_ANALYSER_H
 
-#include "lib/clists.h"
-#include "lib/slists.h"
+#include "ucw/clists.h"
+#include "ucw/slists.h"
+#include "sherlock/index.h"
 
 struct an_iface {		// Information passed to an analyser
   struct odes *obj;		// Original object
@@ -41,12 +42,12 @@ struct an_hook {
   cnode n;
   struct analyser *a;
   struct an_context *c;
-  sh_time_t older_than;		// Older documents are re-analysed; 0=never, ~0=always
+  ucw_time_t older_than;		// Older documents are re-analysed; 0=never, ~0=always
   void *priv;			// Private to the analyser
   uns doc_count;
   uns triggered;		// Requested by previous analyser_need()
   uns need_mask;		// A mask of AN_NEED_xxx flags
-  byte *parameter;		// Parameter from config
+  char *parameter;		// Parameter from config
   uns initialized;
 };
 
@@ -59,7 +60,7 @@ struct an_context {
 
 struct analyser {
   snode n;			// We have a list of all analysers
-  byte *name;
+  char *name;
   void (*init)(struct an_hook *h);				// Called once per each hook (optional)
   void (*init_context)(struct an_hook *h);			// For each thread (optional)
   void (*cleanup_context)(struct an_hook *h);			// Cleanup context (optional)
