@@ -5,9 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/mempool.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/mempool.h"
 #include "sherlock/object.h"
 #include "analyser/analyser.h"
 
