@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lfs.h"
+#include "ucw/lfs.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -28,11 +28,11 @@ int main(int argc, char **argv)
   if (argc != 2)
     die("Usage: bmap [-v] <file>");
 
-  int fd = sh_open(argv[1], O_RDONLY);
+  int fd = ucw_open(argv[1], O_RDONLY);
   if (fd < 0)
     die("open(%s): %m", argv[1]);
-  sh_stat_t st;
-  if (sh_fstat(fd, &st) < 0)
+  ucw_stat_t st;
+  if (ucw_fstat(fd, &st) < 0)
     die("fstat(%s): %m", argv[1]);
 
   uns bsize;
