@@ -22,6 +22,14 @@ foreach $_ (@ARGV) {
 		$prolog .= "my (\$sum$i, \$cnt$i) = (0,0);\n";
 		$each .= "if (defined \$a[$f]) { \$cnt$i++; \$sum$i += \$a[$f]; }\n";
 		$epilog .= "print \$cnt$i ? (\$sum$i / \$cnt$i) : 0, \"\\n\";\n";
+	} elsif ($s eq ":min") {
+		$prolog .= "my \$min$i;\n";
+		$each .= "if (defined \$a[$f] && (!defined(\$min$i) || \$a[$f] < \$min$i)) { \$min$i = \$a[$f]; }\n";
+		$epilog .= "print \$min$i, \"\\n\";\n";
+	} elsif ($s eq ":max") {
+		$prolog .= "my \$max$i;\n";
+		$each .= "if (defined \$a[$f] && (!defined(\$max$i) || \$a[$f] > \$max$i)) { \$max$i = \$a[$f]; }\n";
+		$epilog .= "print \$max$i, \"\\n\";\n";
 	} else {
 		die "Invalid operator: $s";
 	}
