@@ -1,13 +1,20 @@
 #!/usr/bin/perl
 # Find the call tree of memory allocations (using cscope)
 # (c) 2004 Martin Mares <mj@ucw.cz>
+# (c) 2006 Robert Spalek <robert@ucw.cz>
 
 # The set of allocator calls we want to trace
 my @queue = (
-	"xmalloc", "xmalloc_zero", "sh_xmalloc", "xfree", "free",
+	"xmalloc", "xmalloc_zero", "ucw_xmalloc", "xfree", "free", "ucw_xfree",
 	"mmap_file", "munmap_file",
 	"mp_new", "mp_delete",
-	"resolve_fingerprints"
+	"alloc_read_ary", "write_free_ary",
+	"attrs_map", "attrs_unmap",
+	"resolve_fastbuf",
+	"bb_grow",
+	#"partmap_open", "partmap_close",
+	#"notes_part_map", "notes_part_unmap", "notes_skel_part_map", "notes_skel_part_unmap",
+	#"attrs_part_map", "attrs_part_unmap",
 );
 my %known = map { $_ => 2 } @queue;
 
