@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/fastbuf.h"
+#include "ucw/fastbuf.h"
 
 struct hist {
   uns cnt;
@@ -18,7 +18,7 @@ struct hist {
 #define HASH_WANT_LOOKUP
 #define HASH_AUTO_POOL 16384
 #define HASH_ZERO_FILL
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 int
 main(void)
