@@ -9,11 +9,11 @@
 #include "search/sherlockd.h"
 #include "search/refs.h"
 
-byte *custom_file_type_names[MAX_FILE_TYPES] = {
+char *custom_file_type_names[MAX_FILE_TYPES] = {
   "unknown", "html", "pdf", "text", "msword", "excel", "RFU6", "RFU7",
   "jpeg", "png", "gif",  "RFU11", "RFU12", "RFU13", "RFU14", "RFU15" };
 
-byte *lng_parse(struct query *q, enum custom_op op, byte *val, uns intval)
+char *lng_parse(struct query *q, enum custom_op op, char *val, uns intval)
 {
   if (op != CUSTOM_OP_EQ)
     return "LNG: Only equality supported";
