@@ -20,5 +20,5 @@ struct query;
 struct ref_context;
 struct card_attr;
 
-byte *lng_parse(struct query *q, enum custom_op op, byte *val, uns intval);
+char *lng_parse(struct query *q, enum custom_op op, char *val, uns intval);
 int custom_match(struct query *q, struct ref_context *c, struct card_attr *ca, int *QQ);
