@@ -1,6 +1,6 @@
-#include "lib/lib.h"
-#include "lib/fastbuf.h"
-#include "lib/bbuf.h"
+#include "ucw/lib.h"
+#include "ucw/fastbuf.h"
+#include "ucw/bbuf.h"
 #include "sherlock/sherlock.h"
 #include "indexer/graph.h"
 
@@ -13,7 +13,7 @@ main(int argc UNUSED, char **argv)
   uns start = strtoul(argv[1], NULL, 16);
   struct fastbuf *in = bopen(argv[2], O_RDONLY, 1<<16);
   u32 node, deg;
-  sh_off_t pos;
+  ucw_off_t pos;
   static bb_t positions;
   uns max = 0;
   while (pos = btell(in), bget_graph_hdr(in, &node, &deg))
@@ -29,7 +29,7 @@ main(int argc UNUSED, char **argv)
   for (uns i=0; i<max; i++)	// so that uninitialized is set to -1
   {
     byte *p = positions.ptr + i * BYTES_PER_O;
-    sh_off_t pos = GET_O(p);
+    ucw_off_t pos = GET_O(p);
     PUT_O(p, pos-1);
   }
   bclose(in);
