@@ -4,8 +4,8 @@
  *	(c) 2006 Pavel Charvat <pchar@ucw.cz>
  */
 
-#include "lib/lib.h"
-#include "lib/conf.h"
+#include "ucw/lib.h"
+#include "ucw/conf.h"
 #include <stdlib.h>
 #include <unistd.h>
 #include <pthread.h>
