@@ -5,10 +5,10 @@
  *	(c) 2006 Pavel Charvat <pchar@ucw.cz>
  */
 
-#include "lib/lib.h"
-#include "lib/fastbuf.h"
-#include "lib/bbuf.h"
-#include "lib/conf.h"
+#include "ucw/lib.h"
+#include "ucw/fastbuf.h"
+#include "ucw/bbuf.h"
+#include "ucw/conf.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -50,7 +50,7 @@ static uns keep_order;
 #define ASORT_KEY_TYPE u64
 #define ASORT_ELT(i) samples[i].sort_key
 #define ASORT_SWAP(i, j) SWAP(i, j)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static inline void
 new_sample(uns index)
