@@ -5,20 +5,22 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "ucw/conf.h"
 #include "filter/filter.h"
 
 uns filter_trace = 0;
 uns filter_hash_limit = 4;
+uns filter_kmp_limit = 4;
 uns filter_trie_limit = 4;
 uns filter_tree_limit = 4;
 uns filter_optimize = 1;
-byte *filter_dump_to = NULL;
+char *filter_dump_to = NULL;
 
 static struct cf_section fconfig = {
 	CF_ITEMS {
 		CF_UNS("Trace", &filter_trace),
 		CF_UNS("HashLimit", &filter_hash_limit),
+		CF_UNS("KMPLimit", &filter_kmp_limit),
 		CF_UNS("TrieLimit", &filter_trie_limit),
 		CF_UNS("TreeLimit", &filter_tree_limit),
 		CF_UNS("Optimize", &filter_optimize),
