@@ -6,11 +6,12 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/mempool.h"
-#include "lib/url.h"
-#include "lib/fastbuf.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/mempool.h"
+#include "ucw/url.h"
+#include "ucw/fastbuf.h"
+#include "ucw/string.h"
 #include "sherlock/object.h"
 #include "filter/filter.h"
 #include "filter/parse.tab.h"
@@ -22,9 +23,19 @@
 #include <fcntl.h>
 
 struct filter *filter_current;
+jmp_buf filter_jmp_buf;
+void (*filter_err_hook)(char *err);
 
-struct filter *
-filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind, struct filter_function *func)
+static int
+filter_do_parse(struct filter *f)
+{
+	if (setjmp(filter_jmp_buf))
+		return 1;
+	return yyparse(f);
+}
+
+static struct filter *
+filter_do_load(struct filter_variable *var, struct filter_binding *bind, struct filter_function *func)
 {
 	struct filter *f;
 
@@ -45,10 +56,14 @@ filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind
 	bzero(f->lookup, sizeof(*f->lookup));
 
 	filter_current = f;
-	filter_lex_init(name);
-	yyparse(f);
+	int err = filter_do_parse(f);
 	filter_lex_cleanup();
 	filter_current = NULL;
+	if (err)
+	{
+		filter_delete(f);
+		return NULL;
+	}
 
 	filter_prune(f);
 	if (filter_dump_to)
@@ -60,6 +75,20 @@ filter_load(byte *name, struct filter_variable *var, struct filter_binding *bind
 	return f;
 }
 
+struct filter *
+filter_load(char *name, struct filter_variable *var, struct filter_binding *bind, struct filter_function *func)
+{
+	filter_lex_init(name);
+	return filter_do_load(var, bind, func);
+}
+
+struct filter *
+filter_load_fb(struct fastbuf *fb, struct filter_variable *var, struct filter_binding *bind, struct filter_function *func)
+{
+	filter_lex_init_fb(fb);
+	return filter_do_load(var, bind, func);
+}
+
 struct filter *
 filter_clone(struct filter *filter)
 {
@@ -87,15 +116,15 @@ filter_clone(struct filter *filter)
 			{
 				l->trie = mp_memdup(f->pool, l->trie, sizeof(*l->trie));
 				l->trie->results = mp_memdup(f->pool, l->trie->results, l->trie->cmds * sizeof(struct filter_trie_result));
-				init_list(&l->trie->tests);
-				init_list(&l->trie->incremented_tests);
-				init_list(&l->trie->passed_tests);
+				clist_init(&l->trie->tests);
+				clist_init(&l->trie->incremented_tests);
+				clist_init(&l->trie->passed_tests);
 				for (uns i = 0; i < l->trie->cmds; i++)
 				{
-		  			if (l->trie->results[i].init_passed == 2)
-						add_tail(&l->trie->passed_tests, &l->trie->results[i].n);
+					if (l->trie->results[i].init_passed == 2)
+						clist_add_tail(&l->trie->passed_tests, &l->trie->results[i].n);
 					else
-		  				add_tail(&l->trie->tests, &l->trie->results[i].n);
+						clist_add_tail(&l->trie->tests, &l->trie->results[i].n);
 				}
 				break;
 			}
@@ -117,7 +146,7 @@ filter_lookup_new(struct filter *filter, enum filter_lookup_type type, void *val
 	struct filter_lookup *l = filter->lookup + ++filter->lookup_count;
 	l->type = type;
 	l->value = value;
-	return filter->lookup_count; 
+	return filter->lookup_count;
 }
 
 int
@@ -133,6 +162,7 @@ filter_delete(struct filter *f)
 	if (!f)
 		return;
 	mp_delete(f->pool);
+	xfree(f->lookup);
 	xfree(f);
 }
 
@@ -515,9 +545,9 @@ compare_string(struct filter_value *l, int icase, int op, struct filter_value *r
 	if (op == EPAT)
 	{
 		if (icase)
-			return match_pattern_nocase(r->v.s, l->v.s) ? TRUE : FALSE;
+			return str_match_pattern_nocase(r->v.s, l->v.s) ? TRUE : FALSE;
 		else
-			return match_pattern(r->v.s, l->v.s) ? TRUE : FALSE;
+			return str_match_pattern(r->v.s, l->v.s) ? TRUE : FALSE;
 	}
 
 	if (icase)
@@ -697,6 +727,11 @@ return_msg(struct filter_args *args, struct filter_expr *expr)
 	}
 }
 
+#define ASORT_PREFIX(x) cases_##x
+#define ASORT_KEY_TYPE struct filter_case *
+#define ASORT_LT(x, y) ((x)->case_id < (y)->case_id)
+#include "ucw/sorter/array-simple.h"
+
 static int
 filter_eval_cmd(struct filter_args *args, struct filter_cmd *cmd)
 {
@@ -757,53 +792,64 @@ filter_eval_cmd(struct filter_args *args, struct filter_cmd *cmd)
 			}
 			case SWITCH:
 			{
-				struct filter_cmd *run_cmd = NULL;
-				struct filter_case *cas;
-				int res;
+				struct filter_case *cas, *buf[16]; // rerely need to reallocate
+				struct filter_cases res = { .args = args, .list = buf, .size = ARRAY_SIZE(buf) };
+				int ires;
 				filter_eval_expr(&a, args, cmd->c.swit.expr);
 				if (a.undef)
 				{
-					run_cmd = cmd->c.swit.undefined;
-					goto eval_cmd;
+					if (ires = filter_eval_cmd(args, cmd->c.swit.undefined))
+						return ires;
+					break;
 				}
 				if (cmd->c.swit.cmp || cmd->c.swit.icmp || cmd->c.swit.pat || cmd->c.swit.ipat)
 					ASSERT(a.type == F_ET_STRING);
-				if (cmd->c.swit.cmp && filter_ht_find(cmd->c.swit.cmp, a.v.s, &run_cmd))
-					goto eval_cmd;
-				if (cmd->c.swit.icmp && filter_ht_find(cmd->c.swit.icmp, a.v.s, &run_cmd))
-					goto eval_cmd;
-				if (cmd->c.swit.pat && (run_cmd = filter_trie_search(args->filter->lookup[cmd->c.swit.pat].trie, a.v.s)) )
-					goto eval_cmd;
-				if (cmd->c.swit.ipat && (run_cmd = filter_trie_search(args->filter->lookup[cmd->c.swit.ipat].trie, a.v.s)) )
-					goto eval_cmd;
+				if (cmd->c.swit.cmp)
+					filter_ht_find(cmd->c.swit.cmp, a.v.s, &res);
+				if (cmd->c.swit.icmp)
+					filter_ht_find(cmd->c.swit.icmp, a.v.s, &res);
+				if (cmd->c.swit.kmp)
+					filter_kmp_find(cmd->c.swit.kmp, a.v.s, &res);
+				if (cmd->c.swit.ikmp)
+					filter_kmp_find(cmd->c.swit.ikmp, a.v.s, &res);
+				if (cmd->c.swit.pat)
+					filter_trie_search(args->filter->lookup[cmd->c.swit.pat].trie, a.v.s, &res);
+				if (cmd->c.swit.ipat)
+					filter_trie_search(args->filter->lookup[cmd->c.swit.ipat].trie, a.v.s, &res);
 				if (cmd->c.swit.expr->type == F_ET_STRING)
 				{
-					if (cmd->c.swit.bins && (run_cmd = filter_s_tree_search(cmd->c.swit.bins, a.v.s)) )
-						goto eval_cmd;
-					else if (cmd->c.swit.binis && (run_cmd = filter_is_tree_search(cmd->c.swit.binis, a.v.s)) )
-						goto eval_cmd;
+					if (cmd->c.swit.bins)
+						filter_s_tree_search(cmd->c.swit.bins, a.v.s, &res);
+					else if (cmd->c.swit.binis)
+						filter_is_tree_search(cmd->c.swit.binis, a.v.s, &res);
 				}
 				else
 				{
-					if (cmd->c.swit.binud && (run_cmd = filter_ud_tree_search(cmd->c.swit.binud, a.v.i)) )
-						goto eval_cmd;
-					else if (cmd->c.swit.bind && (run_cmd = filter_d_tree_search(cmd->c.swit.bind, a.v.i)) )
-						goto eval_cmd;
+					if (cmd->c.swit.binud)
+						filter_ud_tree_search(cmd->c.swit.binud, a.v.i, &res);
+					else if (cmd->c.swit.bind)
+						filter_d_tree_search(cmd->c.swit.bind, a.v.i, &res);
 				}
 				for (cas=cmd->c.swit.cases; cas; cas=cas->next)
 				{
-					res = filter_eval_cond(args, cas->cond, cmd->c.swit.expr);
-					if (res == TRUE)
-					{
-						run_cmd = cas->positive;
-						goto eval_cmd;
-					}
+					ires = filter_eval_cond(args, cas->cond, cmd->c.swit.expr);
+					if (ires == TRUE)
+						filter_cases_add(&res, cas);
 				}
-				run_cmd = cmd->c.swit.negative;
-eval_cmd:
-				res = filter_eval_cmd(args, run_cmd);
-				if (res)
-					return res;
+				if (!res.count)
+				{
+					if (ires = filter_eval_cmd(args, cmd->c.swit.negative))
+						return ires;
+					break;
+				}
+				cases_sort(res.list, res.count);
+				for (uns i = 0; i < res.count; i++)
+					if (!i || res.list[i]->case_id != res.list[i - 1]->case_id)
+					{
+						ires = filter_eval_cmd(args, res.list[i]->positive);
+						if (ires)
+							return ires;
+					}
 				break;
 			}
 			default:
@@ -878,3 +924,14 @@ filter_intr_run(struct filter_args *a)
 	}
 	return res == ACCEPT ? 1 : 0;
 }
+
+void
+filter_cases_grow(struct filter_cases *r)
+{
+	ASSERT(r->count == r->size);
+	r->size *= 2;
+	void *ptr = mp_alloc(r->args->pool, r->size * sizeof(void *));
+	memcpy(ptr, r->list, r->count * sizeof(void *));
+	r->list = ptr;
+}
+
