@@ -2,8 +2,8 @@
 #include <string.h>
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/mempool.h"
+#include "ucw/conf.h"
+#include "ucw/mempool.h"
 #include "sherlock/object.h"
 #include "filter/filter.h"
 
