@@ -5,8 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/hashfunc.h"
+#include "ucw/mempool.h"
+#include "ucw/hashfunc.h"
+#include "ucw/prime.h"
 #include "filter/filter.h"
 
 #include <string.h>
@@ -25,20 +26,20 @@ filter_ht_new(struct mempool *mp, uns count, uns icase)
 }
 
 void
-filter_ht_add(struct filter_hash_table *ht, byte *string, struct filter_cmd *cmd)
+filter_ht_add(struct filter_hash_table *ht, byte *string, struct filter_case *cas)
 {
 	struct filter_hash_record *rec = mp_alloc(ht->mp, sizeof(struct filter_hash_record));
 	uns hash = ht->icase ? hash_string_nocase(string) : hash_string(string);
 	hash %= ht->size;
 	rec->next = ht->h[hash];
 	rec->string = string;
-	rec->cmd = cmd;
+	rec->cas = cas;
 	ht->h[hash] = rec;
 	ht->count++;
 }
 
-int
-filter_ht_find(struct filter_hash_table *ht, byte *string, struct filter_cmd **Cmd)
+void
+filter_ht_find(struct filter_hash_table *ht, byte *string, struct filter_cases *res)
 {
 	uns hash = ht->icase ? hash_string_nocase(string) : hash_string(string);
 	struct filter_hash_record *rec;
@@ -47,11 +48,6 @@ filter_ht_find(struct filter_hash_table *ht, byte *string, struct filter_cmd **C
 	{
 		if (ht->icase && !strcasecmp(string, rec->string)
 		|| !ht->icase && !strcmp(string, rec->string))
-		{
-			if (Cmd)
-				*Cmd = rec->cmd;
-			return 1;
-		}
+			filter_cases_add(res, rec->cas);
 	}
-	return 0;
 }
