@@ -17,7 +17,7 @@ void
 custom_create_attrs(struct odes *odes UNUSED, struct card_attr *ca UNUSED)
 {
 #ifdef CONFIG_FILETYPE
-  byte *ctype = obj_find_aval(odes, 'T');
+  char *ctype = obj_find_aval(odes, 'T');
   uns tf;
   if (!ctype)
     tf = 0x00;
@@ -34,7 +34,7 @@ custom_create_attrs(struct odes *odes UNUSED, struct card_attr *ca UNUSED)
 }
 
 #ifdef CONFIG_FILETYPE
-byte *custom_file_type_names[MAX_FILE_TYPES] = {
+char *custom_file_type_names[MAX_FILE_TYPES] = {
   "unknown", "html", "pdf", "text", "RFU4", "RFU5", "RFU6", "RFU7",
   "RFU8", "RFU9", "RFU10",  "RFU11", "RFU12", "RFU13", "RFU14", "RFU15" };
 #endif
