@@ -3,15 +3,17 @@
  *
  *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
  *	(c) 2004 Robert Spalek <robert@ucw.cz>
+ *	(c) 2007 Pavel Charvat <pchar@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/clists.h"
-#include "lib/fastbuf.h"
-#include "lib/chartype.h"
-#include "lib/unicode.h"
-#include "lib/ff-utf8.h"
-#include "lib/stkstring.h"
+#include "ucw/clists.h"
+#include "ucw/fastbuf.h"
+#include "ucw/chartype.h"
+#include "ucw/unicode.h"
+#include "ucw/ff-unicode.h"
+#include "ucw/string.h"
+#include "ucw/stkstring.h"
 #include "lang/lang.h"
 #include "gather/gather.h"
 #include "charset/charconv.h"
@@ -46,7 +48,7 @@ static u32 *typical_chars, *forbidden_chars, *improbable_chars;
 struct lang_family {
   cnode n;
   uns id;
-  byte *name;
+  char *name;
   uns lang_set;
   int *auto_sets;			/* DARY of auto-detected charsets */
   struct unirange *chars_typical, *chars_forbidden, *chars_improbable;
@@ -54,19 +56,19 @@ struct lang_family {
 
 struct def_lang {
   cnode n;
-  byte *host_patt, *lang_list;
+  char *host_patt, *lang_list;
 };
 
 struct charset_names {
   cnode n;
   int charset_id;			/* As known to the conversion library */
-  byte **aliases;			/* DARY of charset aliases */
+  char **aliases;			/* DARY of charset aliases */
 };
 
 static clist def_lang_list, lfam_list, charset_list;
 static struct lang_family *lfam_by_id[MAX_LFAMS];
 static uns num_lfams;
-static byte **charset_blacklist;
+static char **charset_blacklist;
 
 #define CHN_UNKNOWN		-1
 #define CHN_BLACKLISTED		-2
@@ -84,7 +86,7 @@ struct charset_name {
 #define HASH_NOCASE
 #define HASH_USE_POOL		cf_pool
 #define HASH_ALLOC_TABLE
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static int
 lookup_charset(byte *name)
@@ -330,7 +332,7 @@ guess_lfams(int *lfams)
   if (n <= 0)
     {
       CLIST_FOR_EACH(struct def_lang *, d, def_lang_list)
-	if (match_pattern_nocase(d->host_patt, gthis->url_s.host))
+	if (str_match_pattern_nocase(d->host_patt, gthis->url_s.host))
 	  {
 	    guessed_lang = d->lang_list;
 	    n = lang_parse_list(stk_strdup(guessed_lang), NULL, langs, 1);
@@ -658,6 +660,8 @@ start_pass:
 	{
 	  if (log_charset_errors)
 	    log(L_ERROR_R, "Unrecognized server charset %s", cs);
+          if (believe_server_charset >= 2)
+	    goto unknown;
 	  s_rem = " [unknown]";
 	}
       else if (believe_server_charset < 0)
@@ -691,8 +695,6 @@ start_pass:
 	}
       }
     }
-  if (believe_server_charset > 0)	/* Fallback */
-    goto okay;
 
   /* Charset supplied by META tags */
   if (cs = meta_charset)
@@ -702,6 +704,8 @@ start_pass:
 	{
 	  if (log_charset_errors)
 	    log(L_ERROR_R, "Unrecognized META charset %s", cs);
+          if (believe_meta_charset >= 2)
+	    goto unknown;
 	  m_rem = " [unknown]";
 	}
       else if (believe_meta_charset < 0)
@@ -736,8 +740,6 @@ start_pass:
 	}
       }
     }
-  if (believe_meta_charset > 0)		/* Fallback */
-    goto okay;
 
   /* Try auto-detection */
   if ((g_id = guess_charset(lfamset, lfams, histogram, doc_grade)) >= 0)
@@ -804,4 +806,10 @@ blacklisted:
 	meta_charset ? : (byte *) "?", m_rem,
 	source);
   gerror(2407, "Blacklisted charset");
+
+unknown:
+  TRACE("Charsets: server=<%s>%s meta=<%s>%s -> unknown",
+        server_charset ? : (byte *) "?", s_rem,
+	meta_charset ? : (byte *) "?", m_rem);
+  gerror(2407, "Unknown charset");
 }
