@@ -6,12 +6,13 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/clists.h"
-#include "lib/simple-lists.h"
-#include "lib/conf.h"
-#include "lib/chartype.h"
-#include "lib/fastbuf.h"
-#include "lib/url.h"
+#include "ucw/clists.h"
+#include "ucw/simple-lists.h"
+#include "ucw/conf.h"
+#include "ucw/chartype.h"
+#include "ucw/fastbuf.h"
+#include "ucw/url.h"
+#include "ucw/string.h"
 #include "gather/gather.h"
 
 #include <stdio.h>
@@ -22,16 +23,16 @@
 
 /* Structures */
 
-static byte *mode_names[] = { "suffix", "wildcard", "bytes", "isascii", NULL };	// doesn't contain "extension" for purpose
+static const char * const mode_names[] = { "suffix", "wildcard", "bytes", "isascii", NULL };	// doesn't contain "extension" for purpose
 enum cbmode { CBMODE_SUFFIX, CBMODE_WILDCARD, CBMODE_BINARY, CBMODE_IS_ASCII, CBMODE_EXTENSION, CBMODE_END };
 enum cbresult { CBRES_DONT_KNOW, CBRES_TRUE, CBRES_FALSE, CBRES_END };
 
 struct crystal_ball {			/* Magic rules for datomanthy */
 	cnode n;
 	int mode;			/* Crystal ball type */
-	byte *type;			/* Resulting type */
-	byte **rule;			/* Name or contents pattern */
-	byte **compiled;		/* Compiled rule */
+	char *type;			/* Resulting type */
+	char **rule;			/* Name or contents pattern */
+	char **compiled;		/* Compiled rule */
 };
 
 /* Configuration */
@@ -63,7 +64,7 @@ struct extension_record {
 #define HASH_NOCASE
 #define HASH_CONSERVE_SPACE
 #define HASH_TABLE_ALLOC
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 struct content_type_record {
 	byte *content_type;	/* key */
@@ -78,7 +79,7 @@ struct content_type_record {
 #define HASH_USE_POOL cf_pool
 #define HASH_CONSERVE_SPACE
 #define HASH_TABLE_ALLOC
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static byte *snerr = "Syntax error";
 
@@ -388,7 +389,7 @@ cut_inenc_suffix(byte *name, byte *enc)	/* Cut the optional file suffix and retu
 static struct fastbuf *dt_fb;
 static byte dt_head[HEADSIZE];
 static uns dt_size;
-static byte *dt_name, *dt_type, *dt_enc;
+static char *dt_name, *dt_type, *dt_enc;
 static uns dt_name_len;
 
 #define	MAX_GRADE	1000
@@ -475,7 +476,7 @@ evaluate_crystal_ball(struct crystal_ball *f)
 			if (!dt_name || !plausible_name)
 				return CBRES_DONT_KNOW;
 			for (uns idx=0; idx<DARY_LEN(f->rule); idx++)
-				if(match_pattern_nocase(f->rule[idx], dt_name))
+				if(str_match_pattern_nocase(f->rule[idx], dt_name))
 					return CBRES_TRUE;
 			return CBRES_FALSE;
 		case CBMODE_IS_ASCII:
@@ -762,14 +763,14 @@ do_guess_content(void)
 
 		TRACE("Contents: encoding %s%s->%s, type %s%s%s->%s%s for %s",
 			enc_m,
-			dt_enc ? : (byte*) "?",
+			dt_enc ? : "?",
 			enc ? : (byte*) "?",
 			(byte*) (rejected ? "[rejected] " : ""),
 			type_m,
-			dt_type ? : (byte*) "?",
+			dt_type ? : "?",
 			type ? : (byte*) "?",
 			rename_m,
-			dt_name ? : (byte*) "?");
+			dt_name ? : "?");
 		if (name)
 			strcpy(dt_name, name);
 		dt_enc = enc;
