@@ -6,9 +6,9 @@
 
 #include "sherlock/sherlock.h"
 #include "gather/daemon/gatherd.h"
-#include "lib/conf.h"
+#include "ucw/conf.h"
 
-byte *log_name;
+char *log_name;
 uns max_bucket_file_size;
 uns max_host_count;
 uns max_threads = 8;
@@ -37,12 +37,12 @@ uns trickster_err_prob;
 uns trickster_step = 1;
 uns allow_hard_shutdown = 0;
 int compress_level;
-byte *host_file_name = "not/configured";
-byte *host_bak_name = "not/configured";
-byte *queue_file_name = "not/configured";
-byte *lock_name = "not/configured";
-byte *urldb_name = "not/configured";
-byte *md5db_name = "not/configured";
+char *host_file_name = "not/configured";
+char *host_bak_name = "not/configured";
+char *queue_file_name = "not/configured";
+char *lock_name = "not/configured";
+char *urldb_name = "not/configured";
+char *md5db_name = "not/configured";
 
 static struct cf_section gatherd_config = {
   CF_ITEMS {
