@@ -8,14 +8,14 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/url.h"
-#include "lib/db.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/hashfunc.h"
-#include "lib/mempool.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/url.h"
+#include "sherlock/db.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/hashfunc.h"
+#include "ucw/mempool.h"
 #include "sherlock/object.h"
 #include "gather/daemon/gatherd.h"
 
@@ -30,7 +30,7 @@
 
 /*** Globals ***/
 
-sh_time_t now, cycle_start, cycle_last;
+ucw_time_t now, cycle_start, cycle_last;
 
 static uns do_not_expire;
 static uns one_pass;
@@ -42,9 +42,9 @@ static uns revalidate_cycle;
 static int queue_discard_age = 0x7fffffff;
 static int error_discard_age = 0x7fffffff;
 static int robot_expire_age = 0x7fffffff;
-static byte *tmp_queue_name = "db/queue.tmp";
+static char *tmp_queue_name = "db/queue.tmp";
 static uns qk_expire_age = ~0U;
-static byte *stamp_name = "db/expire-stamp";
+static char *stamp_name = "db/expire-stamp";
 static int trace;
 static int queue_postpone = 0x7fffffff;
 static uns queue_bonus_refresh;
@@ -129,14 +129,14 @@ judge_hash(byte *name)
 static int
 judge_filter(byte *url)
 {
-  byte *msg;
+  byte *m;
 
   rfdata.url = url;
-  msg = ref_filter(&rfdata);
-  if (msg)
+  m = ref_filter(&rfdata);
+  if (m)
     {
       if (trace > 1)
-	log(L_DEBUG, "%s: %s", url, msg);
+	log(L_DEBUG, "%s: %s", url, m);
       return JUDGE_FILTER_OUT;
     }
   return JUDGE_ASIS;
@@ -162,7 +162,7 @@ static int
 judge(byte *url, struct urlrec *ur)
 {
   int age, verdict;
-  byte *msg = "OK";
+  byte *m = "OK";
   static int warned_time_skew;
 
   verdict = judge_filter(url);
@@ -176,26 +176,26 @@ judge(byte *url, struct urlrec *ur)
       age = 0;
     }
   if (do_not_expire)
-    msg = "not expired";
+    m = "not expired";
   else if (ur->flags & URF_QUEUED)
     {
       if (age > queue_discard_age)
 	{
-	  msg = "expiring from queue";
+	  m = "expiring from queue";
 	  verdict = JUDGE_EXPIRE;
 	}
       else
-	msg = "already queued";
+	m = "already queued";
     }
   else if (ur->oid >= OID_FIRST_ERROR)
     {
       if (age > error_discard_age)
 	{
-	  msg = "expiring error";
+	  m = "expiring error";
 	  verdict = JUDGE_EXPIRE;
 	}
       else
-	msg = "error marker";
+	m = "error marker";
     }
   else if (age > min_revalidate_age)
     {
@@ -204,14 +204,14 @@ judge(byte *url, struct urlrec *ur)
 	  (j >= j_low && j < j_high) :
 	  (j >= j_low || j < j_high))
 	{
-	  msg = "revalidate";
+	  m = "revalidate";
 	  verdict = JUDGE_REFRESH;
 	}
       else
-	msg = "waiting for revalidation";
+	m = "waiting for revalidation";
     }
   if (trace > 1)
-    log(L_DEBUG, "%s: %s", url, msg);
+    log(L_DEBUG, "%s: %s", url, m);
   return verdict;
 }
 
@@ -280,7 +280,7 @@ ids_close(void)
 #define ASORT_KEY_TYPE oid_t
 #define ASORT_ELT(i) ID(i).oid
 #define ASORT_SWAP(i,j) do { struct id tmp = ID(i); ID(i)=ID(j); ID(j)=tmp; } while(0)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static int
 ids_lookup(oid_t oid)
@@ -303,12 +303,12 @@ ids_lookup(oid_t oid)
 }
 
 static void
-ids_debug(byte *msg)
+ids_debug(byte *m)
 {
   uns i;
   if (trace < 3)
     return;
-  log(L_DEBUG, "%s", msg);
+  log(L_DEBUG, "%s", m);
   for (i=0; i<id_count; i++)
     {
       ASSERT(!i || ID(i).oid > ID(i-1).oid);
@@ -380,7 +380,7 @@ judge_robot(struct qhost *h)
   struct url ur;
   byte buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
   int e, age;
-  byte *msg = "OK";
+  byte *m = "OK";
 
   if (h->robot_id == OID_UNDEFINED)
     return JUDGE_ASIS;
@@ -400,15 +400,15 @@ judge_robot(struct qhost *h)
   e = JUDGE_ASIS;
   age = now - h->robot_time;
   if (do_not_expire)
-    msg = "not expired";
+    m = "not expired";
   else if (age > robot_expire_age)
     {
       e = JUDGE_EXPIRE;
-      msg = "aged out";
+      m = "aged out";
       h->rec_err_count = 0;
     }
   if (trace > 1)
-    log(L_DEBUG, "%s: %s", buf2, msg);
+    log(L_DEBUG, "%s: %s", buf2, m);
   return e;
 }
 
@@ -647,10 +647,9 @@ quick_delete(void)
 
 /*** Remaking the queue ***/
 
-#define SORT_KEY struct qs_key
 #define SORT_PREFIX(x) qs_##x
-#define SORT_PRESORT
-#define SORT_DELETE_INPUT 1
+#define SORT_KEY struct qs_key
+#define SORT_DATA_SIZE(k) ((k).len+8)
 #define SORT_INPUT_FB
 #define SORT_OUTPUT_FB
 
@@ -663,19 +662,13 @@ struct qs_key {
 static inline int
 qs_compare(struct qs_key *a, struct qs_key *b)
 {
-  if (a->host < b->host)
-    return -1;
-  if (a->host > b->host)
-    return 1;
-  if (a->weight > b->weight)
-    return -1;
-  if (a->weight < b->weight)
-    return 1;
+  COMPARE(a->host, b->host);
+  REV_COMPARE(a->weight, b->weight);
   return 0;
 }
 
 static inline int
-qs_fetch_key(struct fastbuf *f, struct qs_key *k)
+qs_read_key(struct fastbuf *f, struct qs_key *k)
 {
   k->weight = bgetl(f);
   k->host = bgeta(f);
@@ -684,34 +677,14 @@ qs_fetch_key(struct fastbuf *f, struct qs_key *k)
 }
 
 static inline void
-qs_copy_data(struct fastbuf *src, struct fastbuf *dest, struct qs_key *k)
+qs_write_key(struct fastbuf *f, struct qs_key *k)
 {
-  bputl(dest, k->weight);
-  bputa(dest, k->host);
-  bputw(dest, k->len);
-  bbcopy(src, dest, k->len+8);
+  bputl(f, k->weight);
+  bputa(f, k->host);
+  bputw(f, k->len);
 }
 
-static inline byte *
-qs_fetch_item(struct fastbuf *f, struct qs_key *k, byte *limit)
-{
-  byte *buf = (byte *)(k+1);
-  if (buf + k->len + 8 > limit)
-    return NULL;
-  breadb(f, buf, k->len+8);
-  return buf + k->len+8;
-}
-
-static inline void
-qs_store_item(struct fastbuf *dest, struct qs_key *k)
-{
-  bputl(dest, k->weight);
-  bputa(dest, k->host);
-  bputw(dest, k->len);
-  bwrite(dest, (byte *)(k+1), k->len+8);
-}
-
-#include "lib/sorter.h"
+#include "ucw/sorter/sorter.h"
 
 #define QL_MAGIC 0x430121dd
 
@@ -726,7 +699,8 @@ rebuild_queue(struct fastbuf *b)
   TRACE("Sorting queue items");
   bflush(b);
   bsetpos(b, 0);
-  b = qs_sort(b);
+  bconfig(b, BCONFIG_IS_TEMP_FILE, 1);
+  b = qs_sort(b, NULL);
 
   TRACE("Rebuilding queue");
   while ((weight = bgetl(b)) != 0xffffffff)
