@@ -6,10 +6,10 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/lists.h"
-#include "lib/mempool.h"
-#include "lib/hashfunc.h"
+#include "ucw/getopt.h"
+#include "ucw/clists.h"
+#include "ucw/mempool.h"
+#include "ucw/hashfunc.h"
 #include "sherlock/object.h"
 #include "gather/daemon/gatherd.h"
 
@@ -25,13 +25,13 @@
 #include <signal.h>
 #include <errno.h>
 
-sh_time_t now;
+ucw_time_t now;
 
 struct thread {
-  node n;
+  cnode n;
   pid_t pid;				/* Process running this thread */
   int pipe_fd;				/* Where is the thread pipe connected to */
-  sh_time_t start_time;			/* Timeout control */
+  ucw_time_t start_time;			/* Timeout control */
   int timed_out;
   struct mempool *pool;			/* Mempool local to this thread */
   struct qhost *host;			/* Host this thread works for */
@@ -225,9 +225,8 @@ gather_document(struct thread *t)
   gather_parse();
   gather_analyse();
   uns diff = gobj_check_update();
-  if (!(diff & (GOBJ_CHG_TEXT_LARGE | GOBJ_CHG_HTTP | GOBJ_CHG_BRAND_NEW)))
+  if (!(diff & (GOBJ_CHG_TEXT_LARGE | GOBJ_CHG_REDIRECT | GOBJ_CHG_HTTP | GOBJ_CHG_BRAND_NEW)))
     gerror(4, "Not changed");
-  gthis->error_msg = "OK";
 
   /* Write the gathered object to a bucket and send description to gatherer master */
   setproctitle("S %.64s", t->url);
@@ -269,7 +268,7 @@ gather_finish(struct thread *t, int err)
   struct qhost *h = t->host;
   struct qnode *qn = t->qnode;
   struct odes *o = t->obj;
-  byte *a, *md5, *msg;
+  byte *a, *md5, *err_msg;
   uns retry_after = 0;
   uns skip_refs = 0;
   byte *log_new = "";
@@ -282,23 +281,23 @@ gather_finish(struct thread *t, int err)
 
   if (!err)
     {
-      if (msg = obj_find_aval(o, '!'))
+      if (err_msg = obj_find_aval(o, '!'))
 	{
-	  err = thread_err = atol(msg);
-	  while (*msg && *msg != ' ')
-	    msg++;
-	  while (*msg == ' ')
-	    msg++;
+	  err = thread_err = atol(err_msg);
+	  while (*err_msg && *err_msg != ' ')
+	    err_msg++;
+	  while (*err_msg == ' ')
+	    err_msg++;
 	}
       else
 	{
 	  log(L_ERROR, "Incomplete object found");
 	  err = 1303;
-	  msg = "Incomplete object";
+	  err_msg = "Incomplete object";
 	}
     }
   else
-    msg = "Subprocess failed";
+    err_msg = "Subprocess failed";
 
   /* We know the object is accepted, but we need to run the filters to alter configuration */
   rfdata.url = t->url;
@@ -421,7 +420,7 @@ gather_finish(struct thread *t, int err)
 	}
     }
 
-  log(L_INFO, "%s: %04d %s [%d%s%s] k=%08x p=%u", t->url, err, msg, t->pid,
+  log(L_INFO, "%s: %04d %s [%d%s%s] k=%08x p=%u", t->url, err, err_msg, t->pid,
       (t->refreshing != OID_UNDEFINED) ? "*" : "",
       log_new, orig_qkey, h->qpriority);
 
@@ -446,9 +445,9 @@ gather_finish(struct thread *t, int err)
 
 /*** Gatherer threads ***/
 
-static list busy_threads;
+static clist busy_threads;
 static uns thread_count;
-static volatile int shut_down;
+static volatile sig_atomic_t shut_down;
 static struct pollfd *poll_table;
 static struct thread **poll_to_thread;
 static int poll_count;
@@ -456,14 +455,14 @@ static int poll_count;
 static void				/* Just sets the "shut_down" variable */
 sig_int(int x UNUSED)
 {
-  log(L_INFO, "Gracious shutdown requested");
+  log(L_INFO | L_SIGHANDLER, "Gracious shutdown requested");
   shut_down = 1;
 }
 
 static void
 sig_term(int x UNUSED)
 {
-  log(L_INFO, "Shutdown requested");
+  log(L_INFO | L_SIGHANDLER, "Shutdown requested");
   shut_down = 2;
 }
 
@@ -472,7 +471,7 @@ init_threads(void)
 {
   struct sigaction siga;
 
-  init_list(&busy_threads);
+  clist_init(&busy_threads);
   poll_table = xmalloc(max_threads * sizeof(struct pollfd));
   poll_to_thread = xmalloc(max_threads * sizeof(struct thread *));
   bzero(&siga, sizeof(siga));
@@ -491,7 +490,7 @@ rebuild_poll_table(void)
   struct thread *t;
 
   poll_count = 0;
-  WALK_LIST(t, busy_threads)
+  CLIST_WALK(t, busy_threads)
     {
       struct pollfd *f = &poll_table[poll_count];
       f->fd = t->pipe_fd;
@@ -540,7 +539,7 @@ run_thread(struct qhost *h, struct qnode *n)
       exit(0);
     }
   TTRACE("... process %d", t->pid);
-  add_tail(&busy_threads, &t->n);
+  clist_add_tail(&busy_threads, &t->n);
   t->pipe_fd = fds[0];
   t->start_time = now;
   t->timed_out = 0;
@@ -577,7 +576,7 @@ thread_died(struct thread *t, int status)
 
   oid = gather_finish(t, err);
   close(t->pipe_fd);
-  rem_node(&t->n);
+  clist_remove(&t->n);
   bclose(t->reply);
   mp_delete(t->pool);
   thread_count--;
@@ -652,7 +651,7 @@ hard_shutdown(void)
   if (!allow_hard_shutdown)
     return;
   obuck_lock_write(&bucket_file);
-  WALK_LIST(t, busy_threads)
+  CLIST_WALK(t, busy_threads)
     {
       if (!k++)
 	log(L_INFO, "Killing subprocesses");
@@ -666,7 +665,7 @@ loop(void)
 {
   for(;;)
     {
-      sh_time_t wait_seconds = 1000000;
+      ucw_time_t wait_seconds = 1000000;
       int i, c, have_waiting;
       struct thread *t;
 
@@ -702,7 +701,7 @@ loop(void)
 	      return;
 	    }
 	}
-      if ((t = (struct thread *)HEAD(busy_threads))->n.next && t->start_time <= now - max_run_time)
+      if ((t = clist_head(&busy_threads)) && t->start_time <= now - max_run_time)
 	{
 	  log(L_ERROR, "Killing subprocess %d (time exceeded)", t->pid);
 	  kill(t->pid, SIGTERM);
