@@ -2,15 +2,15 @@
  *	Sherlock Gatherer Control Utility
  *
  *	(c) 1997--2003 Martin Mares <mj@ucw.cz>
- *	(c) 2005 Robert Spalek <robert@ucw.cz>
+ *	(c) 2005--2009 Robert Spalek <robert@ucw.cz>
  */
 
 #include "sherlock/sherlock.h"
 #include "gather/daemon/gatherd.h"
-#include "lib/url.h"
-#include "lib/getopt.h"
-#include "lib/db.h"
-#include "lib/mempool.h"
+#include "ucw/url.h"
+#include "ucw/getopt.h"
+#include "sherlock/db.h"
+#include "ucw/mempool.h"
 #include "sherlock/object.h"
 
 #include <stdio.h>
@@ -23,7 +23,7 @@
 /* Globals */
 
 static int verbose;
-sh_time_t now;
+ucw_time_t now;
 
 /* Support functions */
 
@@ -179,96 +179,70 @@ struct uhash_key {
   byte url[MAX_URL_SIZE];
 };
 
-#define SORT_KEY struct uhash_key
 #define SORT_PREFIX(x) uhash_##x
-#define SORT_PRESORT
+#define SORT_KEY struct uhash_key
+#define SORT_KEY_SIZE(k) ((k).len + OFFSETOF(struct uhash_key, url) + 1)
 #define SORT_UNIFY
 #define SORT_INPUT_FB
 #define SORT_OUTPUT_FB
 
-static struct fastbuf *uhash_src_fb;
-
 static int
 uhash_compare(struct uhash_key *a, struct uhash_key *b)
 {
   u32 ha = sdbm_hash(a->url, a->len);
   u32 hb = sdbm_hash(b->url, b->len);
-
-  if (ha < hb)
-    return -1;
-  if (ha > hb)
-    return 1;
+  COMPARE(ha, hb);
   return strcmp(a->url, b->url);
 }
 
 static int
-uhash_fetch_key(struct fastbuf *f, struct uhash_key *k)
+uhash_read_key(struct fastbuf *f, struct uhash_key *k)
 {
-  if (f)
+  byte *x = bgets(f, k->url, MAX_URL_SIZE);
+  if (!x)
+    return 0;
+  k->len = x - k->url;
+  return 1;
+}
+
+static inline void
+uhash_write_key(struct fastbuf *fb, struct uhash_key *k)
+{
+  bwrite(fb, k->url, k->len);
+  bputc(fb, '\n');
+}
+
+static inline void
+uhash_write_merged(struct fastbuf *fb, struct uhash_key **keys, void **data UNUSED, uns n UNUSED, void *buf UNUSED)
+{
+  uhash_write_key(fb, *keys);
+}
+
+#include "ucw/sorter/sorter.h"
+
+static struct fastbuf *
+read_urls_from_stdin(void)
+{
+  byte buf[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
+  struct fastbuf *b_in = bfdopen_shared(0, 65536);
+  struct fastbuf *b_out = bopen_tmp(65536);
+  while (bgets(b_in, buf, sizeof(buf)))
     {
-      byte *x = bgets(f, k->url, MAX_URL_SIZE);
-      if (!x)
-	return 0;
-      k->len = x - k->url;
-      return 1;
-    }
-  else
-    {
-      byte buf[MAX_URL_SIZE];
+      if (!buf[0] || buf[0] == '#')
+	continue;
       int err;
-      for(;;)
-	{
-	  if (!bgets(uhash_src_fb, buf, MAX_URL_SIZE))
-	    return 0;
-	  if (!buf[0] || buf[0] == '#')
-	    continue;
-	  if (!(err = url_auto_canonicalize(buf, k->url)))
-	    {
-	      k->len = strlen(k->url);
-	      return 1;
-	    }
-	  if (log_ref_errors)
-	    log(L_ERROR, "%s: %s", buf, url_error(err));
-	}
+      if (!(err = url_auto_canonicalize(buf, buf2)))
+	bputsn(b_out, buf2);
+      else if (log_ref_errors)
+	log(L_ERROR, "%s: %s", buf, url_error(err));
     }
+  bclose(b_in);
+  brewind(b_out);
+  return b_out;
 }
 
-static inline void
-uhash_copy_data(struct fastbuf *src UNUSED, struct fastbuf *dest, struct uhash_key *k)
-{
-  bwrite(dest, k->url, k->len);
-  bputc(dest, '\n');
-}
-
-static inline void
-uhash_merge_data(struct fastbuf *src1, struct fastbuf *src2 UNUSED, struct fastbuf *dest,
-	       struct uhash_key *k1, struct uhash_key *k2 UNUSED)
-{
-  uhash_copy_data(src1, dest, k1);
-}
-
-static inline byte *
-uhash_fetch_item(struct fastbuf *f UNUSED, struct uhash_key *k, byte *limit UNUSED)
-{
-  return k->url + k->len + 1;
-}
-
-static inline void
-uhash_store_item(struct fastbuf *f, struct uhash_key *k)
-{
-  uhash_copy_data(NULL, f, k);
-}
-
-static struct uhash_key *
-uhash_merge_items(struct uhash_key *a, struct uhash_key *b UNUSED)
-{
-  return a;
-}
-
-#include "lib/sorter.h"
-
 static void
-for_each_url(byte *arg, void (*fn)(byte *url), byte *msg)
+for_each_url(byte *arg, void (*fn)(byte *url), byte *m)
 {
   if (arg)
     {
@@ -277,13 +251,14 @@ for_each_url(byte *arg, void (*fn)(byte *url), byte *msg)
     }
   else
     {
+      log(L_INFO, "Canonicalizing URL's");
+      struct fastbuf *f = read_urls_from_stdin();
+      log(L_INFO, "Pre-sorting URL's");
+      f = uhash_sort(f, NULL);
+      log(L_INFO, "%s URL's", m);
+
       byte buf[MAX_URL_SIZE];
       uns cnt = 0;
-      struct fastbuf *f = bfdopen_shared(0, 4096);
-      uhash_src_fb = f;
-      log(L_INFO, "Pre-sorting URL's");
-      f = uhash_sort(NULL);
-      log(L_INFO, "%s URL's", msg);
       while (bgets(f, buf, sizeof(buf)))
 	{
 	  if (buf[0] && buf[0] != '#')
@@ -495,7 +470,7 @@ struct hs {
 #define HASH_WANT_LOOKUP
 
 #define HASH_GIVE_HASHFN
-#include "lib/hashfunc.h"
+#include "ucw/hashfunc.h"
 static inline uns hs_hash(byte *host, uns proto UNUSED, uns port UNUSED)
 {
   return hash_string(host);
@@ -525,7 +500,7 @@ static inline void hs_init_key(struct hs *n, byte *h, uns p, uns t)
 
 #define HASH_AUTO_POOL 65536
 
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static void
 host_stats(void)
@@ -666,117 +641,97 @@ static struct buck2obj_buf *read_buf;
 static struct mempool *reb_pool;
 static uns reb_counter;
 
-#define SORT_KEY struct reb_key
 #define SORT_PREFIX(x) reb_##x
-#define SORT_PRESORT
+#define SORT_KEY struct reb_key
+#define SORT_KEY_SIZE(k) ((k).len + OFFSETOF(struct reb_key, url))
 #define SORT_UNIFY
+#define SORT_HASH_BITS 32
 #define SORT_INPUT_FB
 #define SORT_OUTPUT_FB
 
 static int
 reb_compare(struct reb_key *a, struct reb_key *b)
 {
-  if (a->hash < b->hash)
-    return -1;
-  if (a->hash > b->hash)
-    return 1;
+  COMPARE(a->hash, b->hash);
   return memcmp(&a->fp, &b->fp, sizeof(a->fp));
 }
 
-static int
-reb_fetch_key(struct fastbuf *f, struct reb_key *k)
+static uns
+reb_hash(struct reb_key *a)
 {
-  if (f)
-    {
-      if (!breadb(f, k, sizeof(struct reb_key) - MAX_URL_SIZE))
-	return 0;
-      breadb(f, k->url, k->len);
-      return 1;
-    }
-  else
-    {
-      struct fastbuf *fb;
-      struct obuck_header bhdr;
-      struct odes *obj;
-      byte *url, *x;
+  return a->hash;
+}
 
-    restart:
-      if (!(fb = obuck_slurp_pool(&bucket_file, &bhdr, OBUCK_OID_ANY)))
-	return 0;
+static int
+reb_read_key(struct fastbuf *f, struct reb_key *k)
+{
+  if (!breadb(f, k, sizeof(struct reb_key) - MAX_URL_SIZE))
+    return 0;
+  breadb(f, k->url, k->len);
+  return 1;
+}
+
+static inline void
+reb_write_key(struct fastbuf *f, struct reb_key *key)
+{
+  bwrite(f, key, SORT_KEY_SIZE(*key));
+}
+
+static inline void
+reb_write_merged(struct fastbuf *f, struct reb_key **keys, void **data UNUSED, uns n, void *buf UNUSED)
+{
+  uns i = 0;
+  for (uns j = 1; j < n; j++)
+    if (keys[j]->oid > keys[i]->oid)
+      i = j;
+  reb_write_key(f, keys[i]);
+}
+
+#include "ucw/sorter/sorter.h"
+
+static struct fastbuf *
+reb_input(void)
+{
+  struct fastbuf *out = bopen_tmp(65536);
+  struct fastbuf *fb;
+  struct obuck_header bhdr;
+  struct odes *obj;
+  struct reb_key k;
+  byte *url, *x;
+  while (fb = obuck_slurp_pool(&bucket_file, &bhdr, OBUCK_OID_ANY))
+    {
       mp_flush(reb_pool);
       obj = obj_read_bucket(read_buf, reb_pool, bhdr.type, bhdr.length, fb, NULL, 1);
       if (unlikely(!obj))
-	{
-	  log(L_ERROR, "Cannot parse bucket %08x", bhdr.oid);
-	  goto restart;
-	}
-
+        {
+    	  log(L_ERROR, "Cannot parse bucket %08x", bhdr.oid);
+	  continue;
+    	}
+      
       url = obj_find_aval(obj, 'U');
       if (!url)
-	{
-	  log(L_ERROR, "Bucket %08x has no URL", bhdr.oid);
-	  goto restart;
-	}
+        {
+    	  log(L_ERROR, "Bucket %08x has no URL", bhdr.oid);
+	  continue;
+    	}
       if ((x = obj_find_aval(obj, 'T')) && !strcmp(x, "x-sherlock/robots"))
-	goto restart;
-      k->lastmod = obj_find_anum(obj, 'D', now);
-      k->len = strlen(url) + 1;
-      memcpy(k->url, url, k->len);
-      k->oid = bhdr.oid;
-      k->hash = sdbm_hash(url, k->len-1);
-      fingerprint(url, &k->fp);
+	continue;
+      k.lastmod = obj_find_anum(obj, 'D', now);
+      k.len = strlen(url) + 1;
+      memcpy(k.url, url, k.len);
+      k.oid = bhdr.oid;
+      k.hash = sdbm_hash(url, k.len-1);
+      fingerprint(url, &k.fp);\
       if (!(reb_counter++ % 1024) && verbose)
-	{
-	  printf("%d\r", reb_counter);
-	  fflush(stdout);
-	}
-      return 1;
+        {
+    	  printf("%d\r", reb_counter);
+    	  fflush(stdout);
+    	}
+      reb_write_key(out, &k);
     }
+  return out;
 }
 
-static inline void
-reb_copy_data(struct fastbuf *src UNUSED, struct fastbuf *dest, struct reb_key *k)
-{
-  bwrite(dest, k, sizeof(*k) - MAX_URL_SIZE + k->len);
-}
-
-static inline struct reb_key *
-reb_select(struct reb_key *x, struct reb_key *y)
-{
-  if (x->oid > y->oid)
-    return x;
-  if (x->oid < y->oid)
-    return y;
-  ASSERT(0);
-}
-
-static inline void
-reb_merge_data(struct fastbuf *src1 UNUSED, struct fastbuf *src2 UNUSED, struct fastbuf *dest,
-	       struct reb_key *k1, struct reb_key *k2)
-{
-  reb_copy_data(NULL, dest, reb_select(k1, k2));
-}
-
-static inline byte *
-reb_fetch_item(struct fastbuf *f UNUSED, struct reb_key *k, byte *limit UNUSED)
-{
-  return k->url + k->len;
-}
-
-static inline void
-reb_store_item(struct fastbuf *f, struct reb_key *k)
-{
-  reb_copy_data(NULL, f, k);
-}
-
-static struct reb_key *
-reb_merge_items(struct reb_key *a, struct reb_key *b)
-{
-  return reb_select(a, b);
-}
-
-#include "lib/sorter.h"
-
 static void
 synchronize_urldb(void)
 {
@@ -794,14 +749,14 @@ synchronize_urldb(void)
   read_buf = buck2obj_alloc();
   reb_counter = 0;
   log(L_INFO, "Scanning buckets");
-  f = reb_sort(NULL);
+  f = reb_sort(reb_input(), NULL);
   bucket_close();
   log(L_INFO, "Found %d URL's", reb_counter);
 
   log(L_INFO, "Updating URLdb");
 
   urldb_open();
-  while (reb_fetch_key(f, &k))
+  while (reb_read_key(f, &k))
     {
       ASSERT(k.oid != last_oid);
       if (urldb_lookup(k.url, &ur))
