@@ -8,15 +8,15 @@
 
 #include "sherlock/sherlock.h"
 #include "gather/daemon/gatherd.h"
-#include "lib/url.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/mempool.h"
-#include "lib/pagecache.h"
-#include "lib/lfs.h"
-#include "lib/unaligned.h"
-#include "lib/hashfunc.h"
-#include "lib/heap.h"
+#include "ucw/url.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/mempool.h"
+#include "sherlock/pagecache.h"
+#include "ucw/lfs.h"
+#include "ucw/unaligned.h"
+#include "ucw/hashfunc.h"
+#include "ucw/heap.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -77,7 +77,7 @@ static inline int waiting_heap_less(struct bh_node *a, struct bh_node *b)
   struct qnode *y = (struct qnode *) b;
   return (x->wait_until < y->wait_until);
 }
-#include "lib/binheap.h"
+#include "ucw/binheap.h"
 
 /* The heap of ready queue nodes: ordered by priority and then by sequence numbers */
 
@@ -140,7 +140,7 @@ static inline int host_heap_less(struct bh_node *a, struct bh_node *b)
     return 0;
   return (x->sequence < x->sequence);
 }
-#include "lib/binheap.h"
+#include "ucw/binheap.h"
 
 static clist idle_nodes;		/* Queue nodes in state QS_IDLE */
 static clist active_nodes;		/* Queue nodes in state QS_ACTIVE */
@@ -247,7 +247,7 @@ lookup_qnode(u32 qkey)
 }
 
 static void
-relink_qnode(struct qhost *h, sh_time_t wait_until, int insert_host)
+relink_qnode(struct qhost *h, ucw_time_t wait_until, int insert_host)
 {
   /*
    *  Relink the qnode corresponding to a given host to the appropriate list / heap.
@@ -369,7 +369,7 @@ put_host(struct qhost *h)
 }
 
 int
-host_time_step(sh_time_t *p_wait_seconds)
+host_time_step(ucw_time_t *p_wait_seconds)
 {
   struct qnode *q;
 
@@ -707,7 +707,7 @@ queue_walk_next(uns *pos, struct qitem *qi)
 
   for(;;)
     {
-      if (b && (!p || b->pos != (sh_off_t) (p & ~QUEUE_PAGE_MASK)))
+      if (b && (!p || b->pos != (ucw_off_t) (p & ~QUEUE_PAGE_MASK)))
 	{
 	  pgc_put(queue_cache, b);
 	  b = NULL;
@@ -741,7 +741,7 @@ queue_reset(void)
   ASSERT(flat_queue_mode);
   reset_hosts();
   pgc_cleanup(queue_cache);
-  sh_ftruncate(queue_fd, 0);
+  ucw_ftruncate(queue_fd, 0);
   queue_size = 0;
 }
 
@@ -830,10 +830,10 @@ queue_init(uns flat_mode)
   host_hash = xmalloc_zero(host_hash_size * sizeof(struct qhost *));
   node_hash = xmalloc_zero(key_hash_size * sizeof(struct qnode *));
   load_host_list();
-  if ((queue_fd = sh_open(queue_file_name, O_RDWR | O_CREAT, 0666)) < 0)
+  if ((queue_fd = ucw_open(queue_file_name, O_RDWR | O_CREAT, 0666)) < 0)
     die("Unable to open queue file: %m");
   queue_cache = pgc_open(QUEUE_PAGE_SIZE, queue_cache_size);
-  queue_size = sh_seek(queue_fd, 0, SEEK_END);
+  queue_size = ucw_seek(queue_fd, 0, SEEK_END);
 }
 
 void					/* Cleanup of the queue machinery */
