@@ -6,8 +6,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/bitsig.h"
+#include "ucw/mempool.h"
+#include "ucw/bitsig.h"
+#include "ucw/string.h"
 #include "filter/filter.h"
 #include "gather/daemon/gatherd.h"
 
@@ -166,14 +167,14 @@ add_ref(byte *r, uns flags)
   int nw;
   int add_root = 0;
   struct rfilter_data rfdata;
-  byte *msg, *words[3];
+  char *m, *words[3];
 
   if (ignore_refs && !(flags & URF_INITIAL)) /* If ignore_refs is set, accept only URL's listed initially */
     return;
 
   strcpy(buf0, r);			/* Remove ref-id and check ignoration flag */
   r = buf0;
-  nw = wordsplit(r, words, 3);
+  nw = str_wordsplit(r, words, 3);
   ASSERT(nw >= 0);
   if (nw > 2 && strcmp(words[2], "0"))
     {
@@ -205,10 +206,10 @@ restart:
 
   rfdata.url = url;
   rfdata.qkey = F_UNDEF_INT;
-  if (msg = ref_filter(&rfdata))
+  if (m = ref_filter(&rfdata))
     {
       if (trace_refs)
-	log(L_INFO, "Ref: %s: %s", url, msg);
+	log(L_INFO, "Ref: %s: %s", url, m);
       return;
     }
 
@@ -241,21 +242,21 @@ restart:
   if (h->obj_count[rfdata.section] >= rfdata.section_hard_max)
     {
       watermark = 2;
-      msg = "dropped";
+      m = "dropped";
     }
   else if (h->obj_count[rfdata.section] >= rfdata.section_soft_max)
     {
       watermark = 1;
-      msg = "not queued";
+      m = "not queued";
     }
   else
     {
       watermark = 0;
-      msg = "queued";
+      m = "queued";
     }
   if (trace_refs > 1)
     log(L_DEBUG, "Ref: %s: %s (%d of %d/%d in section %d)",
-	url, msg, h->obj_count[rfdata.section]+1, rfdata.section_soft_max, rfdata.section_hard_max, rfdata.section);
+	url, m, h->obj_count[rfdata.section]+1, rfdata.section_soft_max, rfdata.section_hard_max, rfdata.section);
 
   if (watermark <= 1)
     {
