@@ -8,9 +8,8 @@
 #undef	LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/lists.h"
-#include "lib/fastbuf.h"
+#include "ucw/conf.h"
+#include "ucw/fastbuf.h"
 #include "gather/gather.h"
 
 #include <stdlib.h>
@@ -95,7 +94,7 @@ parser_child(byte **command, int srcfd, int resfd)
 }
 
 int
-external_parse(byte **args)
+external_parse(char **args)
 {
   /* args[0]=dest-ctype, args[1..n-1]=command */
   uns n = DARY_LEN(args);
