@@ -9,14 +9,14 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "ucw/conf.h"
 #include "gather/gather.h"
 
 #include <string.h>
 
 /*** Known parser names ***/
 
-byte *parser_names[] = {
+const char * const parser_names[] = {
 #define P(x) #x,
 #include "gather/format/parsers.h"
 #undef P
@@ -29,14 +29,14 @@ uns trace_parse, max_conversions;
 
 static clist type_hooks, encoding_hooks;
 
-static byte *
+static char *
 parse_init_hook(struct parser_hook *h)
 {
   h->parser = -1;
   return NULL;
 }
 
-static byte *
+static char *
 parse_commit_hook(struct parser_hook *h)
 {
   if (!h->type_patt)
