@@ -7,12 +7,12 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/chartype.h"
+#include "ucw/chartype.h"
 #include "sherlock/index.h"
 #include "sherlock/conf.h"
-#include "lib/unicode.h"
-#include "lib/ff-utf8.h"
-#include "lib/clists.h"
+#include "ucw/unicode.h"
+#include "ucw/ff-unicode.h"
+#include "ucw/clists.h"
 #include "charset/unicat.h"
 #include "lang/lang.h"
 #include "gather/gather.h"
@@ -32,15 +32,18 @@ static uns refresh_threshold;
 static uns log_metas;
 static uns meta_charset;
 static uns meta_language;
+static uns xml_charset;
+static uns xml_language;
 static uns char_ref_hack;
 static uns ignore_unknown_char_refs;
 static uns robot_comments;
-static byte **ignored_metas;
+static uns lt_hack;
+static char **ignored_metas;
 
 struct user_meta {
   cnode n;
   uns attr;
-  byte *meta;
+  char *meta;
 };
 static clist user_metas;
 
@@ -65,7 +68,10 @@ static struct cf_section html_config = {
     CF_UNS("LogMetas", &log_metas),
     CF_UNS("MetaCharset", &meta_charset),
     CF_UNS("MetaLanguage", &meta_language),
+    CF_UNS("XMLCharset", &xml_charset),
+    CF_UNS("XMLLanguage", &xml_language),
     CF_UNS("RobotComments", &robot_comments),
+    CF_UNS("LtHack", &lt_hack),
     CF_LIST("MetaAttr", &user_metas, &html_config_meta),
     CF_STRING_DYN("IgnoreMetas", &ignored_metas, CF_ANY_NUM),
     CF_END
@@ -81,7 +87,7 @@ static void CONSTRUCTOR html_init(void)
 
 static struct fastbuf *html_in, *html_out, *meta_out;
 static int head_mode, inside_head, attrs_seen;
-static byte *found_meta_charset, *found_meta_language;
+static byte *found_meta_charset, *found_meta_language, *found_xml_charset;
 static byte *fake_redirect;
 
 static uns global_robot_control;	/* META ROBOTS */
@@ -93,7 +99,7 @@ static uns local_robot_accum;		/* Which robot control bits were used locally */
 
 /* Reading of input file */
 
-static int ungot_char = -1;
+static int ungot_char;
 
 static int
 get_char(void)
@@ -295,14 +301,14 @@ add_word(uns *wo, int class)
     }
 }
 
-static uns wordbuf[MAX_WORD_LEN];
+static uns wordbuf[MAX_WORD_CHARS];
 static uns wordlen;
 static uns attrs, wattrs;
 
 static void
 flush_word(void)
 {
-  if (wordlen >= MAX_WORD_LEN || !(local_robot_control & RC_INDEX))
+  if (wordlen >= MAX_WORD_CHARS || !(local_robot_control & RC_INDEX))
     {
       /*
        * We want to throw the superlong word out, but we really need to write
@@ -330,13 +336,13 @@ add_char(uns c)
   if (c >= 0x80000000)
     {
       /* URL brackets can split extremely long words */
-      if (wordlen >= MAX_WORD_LEN)
+      if (wordlen >= MAX_WORD_CHARS)
 	flush_word();
       wordbuf[wordlen++] = c;
     }
   else if (Uprint(c) && !Uspace(c))
     {
-      if (wordlen < MAX_WORD_LEN)
+      if (wordlen < MAX_WORD_CHARS)
 	{
 	  wordbuf[wordlen++] = c;
 	  wattrs |= attrs;
@@ -368,7 +374,7 @@ do_add_string(byte *s, int class)
   while (*s)
     {
       uns u;
-      GET_UTF8(s, u);
+      s = utf8_get(s, &u);
       add_char(u);
     }
   flush_word();
@@ -505,13 +511,10 @@ chew_robot_control(byte *t, uns *maskp)
 static void
 add_robot_note(byte *text, uns r, uns mask)
 {
-  byte comment[64];
-  sprintf(comment, "%s:%s%s%s",
-	  text,
+  obj_add_attr_format(gthis->aa, '.', "%s:%s%s%s", text,
 	  (mask & RC_INDEX)   ? ((r & RC_INDEX)   ? " INDEX"   : " NOINDEX")   : "",
 	  (mask & RC_FOLLOW)  ? ((r & RC_FOLLOW)  ? " FOLLOW"  : " NOFOLLOW")  : "",
 	  (mask & RC_ARCHIVE) ? ((r & RC_ARCHIVE) ? " ARCHIVE" : " NOARCHIVE") : "");
-  obj_add_attr(gthis->aa, '.', comment);
 }
 
 static int
@@ -645,6 +648,12 @@ getarg(byte *name)
   return NULL;
 }
 
+static inline byte *
+check_lt(byte *arg)
+{
+  return (lt_hack && arg && strchr(arg, '<')) ? NULL : arg;
+}
+
 static int
 skip_script(int is_style)
 {
@@ -747,6 +756,8 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	case T_HTML:
 	  if ((k = getarg("LANG")) && meta_language)
 	    chew_meta_lang(k);
+	  if ((k = getarg("XML:LANG")) && xml_language)
+	    chew_meta_lang(k);
 	  break;
 	}
       return 0;
@@ -808,7 +819,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	  emit_ref_end(current_a_ref);
 	  current_a_ref = ref;
 	  emit_ref_start(current_a_ref);
-	  if (k = getarg("TITLE"))
+	  if (k = check_lt(getarg("TITLE")))
 	    add_string(k, WT_ALT);
 	}
       break;
@@ -817,7 +828,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
       break;
     case T_IMG:				/* An image */
       ref = image_add_ref(getarg("SRC"), getarg("WIDTH"), getarg("HEIGHT"), rel_nofollow_p());
-      byte *alt = getarg("ALT");
+      byte *alt = check_lt(getarg("ALT"));
       /* Even if ALT is empty, we wish to store the URL brackets if the image is indexable */
       if (ref && !alt && !ref->dont_follow)
 	alt = "";
@@ -826,7 +837,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	  flush_word();
 	  emit_ref_start(ref);
 	  add_string(alt, WT_ALT);
-	  byte *title = getarg("TITLE");
+	  byte *title = check_lt(getarg("TITLE"));
 	  if (title && strcmp(title, alt))
 	    add_string(title, WT_ALT);
 	  emit_ref_end(ref);
@@ -837,11 +848,11 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
       break;
     case T_IFRAME:			/* Inline frame -- we need to index it in a different way */
       ref = html_add_ref('a', getarg("SRC"), rel_nofollow_p());
-      add_string_ref(getarg("TITLE"), WT_ALT, ref);
+      add_string_ref(check_lt(getarg("TITLE")), WT_ALT, ref);
       break;
     case T_AREA:			/* Area -- another nastie */
       ref = html_add_ref('R', getarg("HREF"), rel_nofollow_p());
-      add_string_ref(getarg("ALT"), WT_ALT, ref);
+      add_string_ref(check_lt(getarg("ALT")), WT_ALT, ref);
       break;
     case T_OBJECT:			/* An embedded object */
       if (!t->arg)
@@ -856,7 +867,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	      add_obj_ref(k, getarg("DATA"), getarg("TYPE"));
 	      /* add_string(getarg("STANDBY"), WT_ALT); */
 	      attrs |= A_OBJECT;
-	      add_string(getarg("TITLE"), WT_ALT);
+	      add_string(check_lt(getarg("TITLE")), WT_ALT);
 	    }
 	}
       else
@@ -874,8 +885,8 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
       html_add_ref_full('A', getarg("SRC"), getarg("TYPE"), NULL, rel_nofollow_p());
       break;
     case T_APPLET:			/* An applet */
-      add_string(getarg("ALT"), WT_ALT);
-      add_string(getarg("TITLE"), WT_ALT);
+      add_string(check_lt(getarg("ALT")), WT_ALT);
+      add_string(check_lt(getarg("TITLE")), WT_ALT);
       k = getarg("CODEBASE");
       add_obj_ref(k, getarg("CODE"), "application/x-java");
       add_obj_ref(k, getarg("OBJECT"), "application/x-java");
@@ -888,7 +899,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
       html_add_ref('R', getarg("CITE"), rel_nofollow_p());
       break;
     case T_TABLE:
-      add_string(getarg("SUMMARY"), WT_TEXT);
+      add_string(check_lt(getarg("SUMMARY")), WT_TEXT);
       break;
     case T_BASEFONT:
       if (k = getarg("SIZE"))
@@ -912,7 +923,7 @@ chew_tag(byte *name, byte **anames, byte **avals, uns acount)
 	attrs &= ~(A_SMALLFONT | A_LARGEFONT);
       break;
     case T_SCRIPT:			/* Skip until "</[a-zA-Z]" */
-      add_string(getarg("TITLE"), WT_ALT);
+      add_string(check_lt(getarg("TITLE")), WT_ALT);
       return skip_script(t->arg);
     default:
       die("Unknown tag type %x", t->type);
@@ -1155,7 +1166,7 @@ parse_tag(void)
 		    continue;
 		}
 	      if (k < stop)		/* Attribute values: non-ASCII characters are encoded in UTF-8 */
-		PUT_UTF8(k, c);
+		k = utf8_put(k, c);
 	      c = get_char();
 	    }
 	  c = get_char();
@@ -1171,7 +1182,7 @@ parse_tag(void)
 	    if (Uspace(c) || c == '\r' || c == '\n')
 	      break;
 	    else if (k < stop)
-	      PUT_UTF8(k, c);
+	      k = utf8_put(k, c);
 	    if ((c = get_char()) < 0)
 	      return 1;
 	  }
@@ -1198,6 +1209,7 @@ chew_html(void)
 {
   int c;
 
+  ungot_char = -1;
   wordlen = wattrs = 0;
   current_a_ref = NULL;
   base_font_size = 3;
@@ -1244,30 +1256,82 @@ chew_html(void)
        * will be ignored, but all ref brackets will be properly flushed.
        */
       uns i;
-      for (i=0; i<MAX_WORD_LEN; i++)
+      for (i=0; i<MAX_WORD_CHARS; i++)
 	add_char('.');
     }
   flush_word();
 }
 
+static uns
+skip_white(void)
+{
+  uns c, n = 0;
+  while ((c = get_char()) < 128 && Cspace(c))
+    n++;
+  unget_char(c);
+  return n;
+}
+
+static void
+chew_xml_decl(void)
+{
+  uns c, q;
+  char name[32], val[64], *p;
+  ungot_char = -1;
+  for (p = "<?xml"; *p; p++)
+    if (get_char() != *p)
+      return;
+  while (1)
+    {
+      if (!skip_white())
+	return;
+      for (p = name; (c = get_char()) < 128 && Calpha(c) && p < name + sizeof(name) - 1; p++)
+	*p = c;
+      *p = 0;
+      unget_char(c);
+      skip_white();
+      if (get_char() != '=')
+	return;
+      skip_white();
+      if ((q = get_char()) != '\'' && q != '"')
+	return;
+      for (p = val;  (int)(c = get_char()) >= 0 && c != q && p < val + sizeof(val) - 1; p++)
+	*p = c;
+      *p = 0;
+      if (c != q)
+	return;
+      if (!strcmp(name, "encoding"))
+        {
+	  found_xml_charset = gstrdup(val);
+	  MTRACE("Found XML charset %s", found_xml_charset);
+	  return;
+	}
+    }
+}
+
 int
-html_parse(byte **args UNUSED)
+html_parse(char **args UNUSED)
 {
   /* Scan the document for META tags carrying charset info if asked */
   found_meta_charset = NULL;
   found_meta_language = NULL;
-  if (meta_charset || meta_language)
+  found_xml_charset = NULL;
+  head_mode = 1;
+  if (meta_charset || meta_language || xml_charset)
     {
-      head_mode = 1;
       html_in = fbmem_clone_read(gthis->contents);
-      chew_html();
+      if (meta_charset || meta_language)
+        chew_html();
+      brewind(html_in);
+      if (xml_charset)
+	chew_xml_decl();
       bclose(html_in);
       if (found_meta_language)			/* META language is to be trusted over server language */
 	gthis->language = found_meta_language;
     }
 
   /* Guess charset and convert to UTF-8 */
-  convert_charset(found_meta_charset);
+  convert_charset(found_meta_charset ? : found_xml_charset);
 
   /* Run raw HTML analyser */
   gather_raw_analyse();
