@@ -9,7 +9,7 @@ struct entity {
   unsigned int value;
 };
 
-const struct entity *is_entity(register const char *, register unsigned int);
+const struct entity *is_entity(const char *, unsigned int);
 
 #define X2(x,y) (((y) << 8) | (x))
 #define X3(x,y,z) (((z) << 16) | ((y) << 8) | (x))
@@ -20,7 +20,7 @@ struct tag {
   unsigned int arg;
 };
 
-const struct tag *is_tag(register const char *, register unsigned int);
+const struct tag *is_tag(const char *, unsigned int);
 
 enum tag_id {
   T_IGNORE = 0,
