@@ -17,7 +17,7 @@
 
 /*** Known parser functions ***/
 
-static int (*parser_functions[])(byte **) = {
+static int (*parser_functions[])(char **) = {
 #define P(x) x##_parse,
 #include "gather/format/parsers.h"
 #undef P
@@ -28,6 +28,10 @@ static int (*parser_functions[])(byte **) = {
 void
 gather_parse(void)
 {
+  /* Do not parse redirects */
+  if (gthis->error_code == 1)
+    return;
+
   uns convcnt = 0;
   struct parser_hook *p;
   byte *t;
@@ -84,3 +88,13 @@ switch_content_encoding(void)
   gthis->contents = gthis->temp;
   gthis->temp = NULL;
 }
+
+void
+switch_content_type(byte *what)
+{
+  obj_add_attr(gthis->aa, 'E', what ? what : gthis->content_type);
+  gthis->content_type = NULL;
+  bclose(gthis->contents);
+  gthis->contents = gthis->temp;
+  gthis->temp = NULL;
+}
