@@ -9,5 +9,5 @@ struct glyph {
   unsigned int unival;
 };
 
-const struct glyph *is_glyph(register const char *, register unsigned int);
+const struct glyph *is_glyph(const char *, unsigned int);
 
