@@ -6,8 +6,8 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/fastbuf.h"
-#include "lib/mempool.h"
+#include "ucw/fastbuf.h"
+#include "ucw/mempool.h"
 #include "gather/gather.h"
 #include "gather/format/pdf/pdf.h"
 
@@ -19,7 +19,7 @@ static struct fastbuf fb_stream, *input;
 struct fastbuf *pdf_in, *pdf_stream_in=&fb_stream;
 static byte stringbuf[STRINGBUFLEN];
 
-sh_off_t pdf_filesize;
+ucw_off_t pdf_filesize;
 
 s32 pdf_rootref, pdf_inforef;
 
@@ -101,7 +101,7 @@ pdf_check_nschar(int spacetype) {
 #define pdf_seek_end(pos)   bseek(pdf_in, pos, SEEK_END)
 
 void
-pdf_seek(sh_off_t pos) {
+pdf_seek(ucw_off_t pos) {
     if(pos<0 || pos>=pdf_filesize)
 	gerror(2220,"(pdf_seek) Seek out of file");
     bsetpos(pdf_in, pos);
@@ -109,7 +109,7 @@ pdf_seek(sh_off_t pos) {
 
 /* advanced input calls */
 
-static sh_off_t seekposbuf[MAX_SEEK_POS];
+static ucw_off_t seekposbuf[MAX_SEEK_POS];
 static int seekpos=0;
 
 void
@@ -300,7 +300,7 @@ get_hexastring(void) {
 	    gerror(2203,"(get_hexastring) stringbuf overflow at %d",in_tell());
     } while(d!='>' && d>=0);
 
-    obj.value.str.i=mp_alloc(page_pool,b-stringbuf);
+    obj.value.str.i=mp_alloc(page_pool,b-stringbuf+1);
     obj.value.str.l=b-stringbuf;
     copy_and_decrypt(obj.value.str.i, stringbuf, obj.value.str.l);
 
@@ -381,7 +381,7 @@ get_string(void) {
     }
     if(c==')')
 	b--;
-    obj.value.str.i=mp_alloc(page_pool,b-stringbuf);
+    obj.value.str.i=mp_alloc(page_pool,b-stringbuf+1);
     obj.value.str.l=b-stringbuf;
     copy_and_decrypt(obj.value.str.i, stringbuf, obj.value.str.l);
     return obj;
@@ -438,7 +438,7 @@ check_ref(OBJECT in1) {
 	    c=in_get_char();
 	    if(c=='R' && is_space(in_check_char(),SP_ANYWDELIM)) {
 		c=in1.value.n;
-		if(xreftroot[c/xreft_size] && xreftroot[c/xreft_size][c%xreft_size].position>0)
+	        if((uns)c<xreft_size*xreft_size && xreftroot[c/xreft_size] && xreftroot[c/xreft_size][c%xreft_size].position>0)
 		    in1.type=OT_OBJREF;
 		else
 		    in1.type=OT_NULL;
@@ -526,7 +526,7 @@ get_obj(void) {
 	    }
     }
     if(input==pdf_in && obj.type==OT_INT) {
-	TRACE(500,"get_obj: check_ref()");
+	TRACE(500,"get_obj: check_ref(%d)", obj.value.n);
 	obj=check_ref(obj);
     }
     return obj;
