@@ -29,14 +29,14 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/chartype.h"
 #include "sherlock/index.h"
-#include "lib/mempool.h"
-#include "lib/hashfunc.h"
-#include "lib/math.h"
-#include "lib/ff-utf8.h"
-#include "lib/unicode.h"
+#include "sherlock/math.h"
+#include "ucw/conf.h"
+#include "ucw/chartype.h"
+#include "ucw/mempool.h"
+#include "ucw/hashfunc.h"
+#include "ucw/ff-unicode.h"
+#include "ucw/unicode.h"
 #include "charset/unicat.h"
 #include "gather/gather.h"
 #include "gather/format/pdf/pdf.h"
@@ -93,7 +93,7 @@ pdf_truncate(void) {
 
 static inline void
 check_overflow(void) {
-    if (btell(pdf_out) > (sh_off_t) max_decode_size)
+    if (btell(pdf_out) > (ucw_off_t) max_decode_size)
 	pdf_truncate();
 }
 
@@ -284,9 +284,23 @@ static uns queued_class;				/* Class of the next character we'll write */
 #define WC_BREAK	0x10				/* Force sentence break */
 #define WC_SPACE	0x20				/* Force word break */
 #define WC_META		0x80				/* We're writing meta-information */
-static uns wordbuf[MAX_WORD_LEN];
+static uns wordbuf[MAX_WORD_CHARS];
 static uns wordlen;
 
+/* Some of the standard types may be missing, assume WT_TEXT then */
+#ifndef WT_BIG_HEADING
+#define WT_BIG_HEADING WT_TEXT
+#endif
+#ifndef WT_SMALL_HEADING
+#define WT_SMALL_HEADING WT_TEXT
+#endif
+#ifndef WT_SMALL
+#define WT_SMALL WT_TEXT
+#endif
+#ifndef WT_ALT
+#define WT_ALT WT_TEXT
+#endif
+
 static void
 flush_word(void) {
     struct fastbuf *s_out = (queued_class & WC_META) ? meta_out : pdf_out;
@@ -294,7 +308,7 @@ flush_word(void) {
     TRACE(200, "(flush_word) Wordlen=%d current_class=%x queued_class=%x", wordlen, current_class, queued_class);
     if (!wordlen)
 	return;
-    if (wordlen >= MAX_WORD_LEN) {
+    if (wordlen >= MAX_WORD_CHARS) {
 	/* Oversize words are ignored and replaced by breaks */
 	wordlen = 0;
 	queued_class |= WC_BREAK;
@@ -310,9 +324,9 @@ flush_word(void) {
     for (uns i=0; i<wordlen; i++)
 	bput_utf8(s_out, wordbuf[i]);
     if (pdf_trace >= 200) {
-	byte buf[3*MAX_WORD_LEN+1], *p=buf;
+	byte buf[MAX_WORD_BYTES+1], *p=buf;
 	for (uns i=0; i<wordlen; i++)
-	    PUT_UTF8(p, wordbuf[i]);
+	    p = utf8_put(p, wordbuf[i]);
 	*p++ = 0;
 	LOG("(flush_word) out word: %s", buf);
     }
@@ -359,7 +373,7 @@ add_uchar(uns unicode) {
 	 * What about some non-word chars? (c),(R),<TM> etc.
 	 */
 	TRACE(200,"(add_uchar) add word char %.4x",unicode);
-	if (wordlen < MAX_WORD_LEN)
+	if (wordlen < MAX_WORD_CHARS)
 	    wordbuf[wordlen++] = unicode;
     } else {
 	TRACE(200,"(add_uchar) End of word");
@@ -657,14 +671,18 @@ out_info(void) {
     if(info.type!=OT_DICT)
 	return;
     parse_dict(info.value.d,infodict);
+#ifdef MT_MISC
     obj_deref(&author);
     if(author.type==OT_STRING) {
 	out_pdfstring(author, WC_META | MT_MISC | WC_BREAK);
     }
+#endif
+#ifdef MT_TITLE
     obj_deref(&title);
     if(title.type==OT_STRING) {
 	out_pdfstring(title, WC_META | MT_TITLE | WC_BREAK);
     }
+#endif
 }
 
 static uns bookmarkcnt;
@@ -893,7 +911,7 @@ out_document(void) {
 }
 
 int
-pdf_parse(byte **args UNUSED) {
+pdf_parse(char **args UNUSED) {
     if(gthis->truncated)
 	gerror(2205, "PDF file truncated");
 
