@@ -5,18 +5,18 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/chartype.h"
-#include "lib/ff-utf8.h"
+#include "ucw/conf.h"
+#include "ucw/chartype.h"
+#include "ucw/ff-unicode.h"
 #include "gather/gather.h"
 
 #include <string.h>
 
 static uns robots_trace;
 static uns robots_work_arounds;
-static byte *my_robot_name;
+static char *my_robot_name;
 
-static byte *
+static char *
 robots_commit_config(void *p UNUSED)
 {
   if (!my_robot_name)
@@ -43,7 +43,7 @@ static void CONSTRUCTOR robots_init(void)
 #define XTRACE(x,y...) do { if (robots_trace > 1) log(L_DEBUG, x,##y); } while (0)
 
 int
-robots_parse(byte **args UNUSED)
+robots_parse(char **args UNUSED)
 {
   byte buf[1024], *i, *j, *k, *cmt;
   struct fastbuf *in;
