@@ -9,7 +9,7 @@
 #include "gather/gather.h"
 
 int
-sink_parse(byte **args UNUSED)
+sink_parse(char **args UNUSED)
 {
   gthis->text = fbmem_create(16384);
   return 1;
