@@ -6,8 +6,8 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/index.h"
-#include "lib/unicode.h"
-#include "lib/ff-utf8.h"
+#include "ucw/unicode.h"
+#include "ucw/ff-unicode.h"
 #include "charset/unicat.h"
 #include "gather/gather.h"
 
@@ -22,7 +22,7 @@ is_word_char(int c)
 static void
 flush_word(struct fastbuf *out, u16 *w, int len)
 {
-  if (!len || len >= MAX_WORD_LEN)	/* Enormous words are ignored instead of truncated */
+  if (!len || len >= MAX_WORD_CHARS)	/* Enormous words are ignored instead of truncated */
     return;
   bputc(out, btell(out) ? ' ' : 0x90 | WT_TEXT);
   while (len--)
@@ -30,10 +30,10 @@ flush_word(struct fastbuf *out, u16 *w, int len)
 }
 
 int
-text_parse(byte **args UNUSED)
+text_parse(char **args UNUSED)
 {
   struct fastbuf *in, *out;
-  u16 w[MAX_WORD_LEN];
+  u16 w[MAX_WORD_CHARS];
   int c, wlen=0;
 
   convert_charset(NULL);
@@ -44,7 +44,7 @@ text_parse(byte **args UNUSED)
     {
       if (is_word_char(c))
 	{
-	  if (wlen < MAX_WORD_LEN)
+	  if (wlen < MAX_WORD_CHARS)
 	    w[wlen++] = c;
 	}
       else
