@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "ucw/conf.h"
 #include "gather/gather.h"
 
 #include <sys/time.h>
@@ -16,8 +16,8 @@
 
 struct validator {
   cnode n;
-  byte *ctype_patt;
-  byte *command;
+  char *ctype_patt;
+  char *command;
 };
 
 static clist validator_list;
