@@ -6,7 +6,7 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/object.h"
-#include "lib/mempool.h"
+#include "ucw/mempool.h"
 #include "analyser/analyser.h"
 #include "gather/gather.h"
 
@@ -79,6 +79,10 @@ gather_analysis_needed(void)
 void
 gather_analyse(void)
 {
+  /* Do not analyse redirects */
+  if (gthis->error_code == 1)
+    return;
+
   struct odes *o = gthis->aa;
   struct an_iface ai = {
     .obj = o,
