@@ -5,8 +5,8 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/ipaccess.h"
+#include "ucw/conf.h"
+#include "ucw/ipaccess.h"
 #include "gather/gather.h"
 
 uns max_obj_size = ~0;
@@ -19,7 +19,7 @@ uns min_summed_size;
 clist gaccess_list;
 int min_ims_delay;
 uns max_refresh_age = ~0U;
-byte *gather_filter_name;
+char *gather_filter_name;
 uns gather_min_compression = 100;
 uns trace_resolve;
 uns max_parser_alloc = ~0U;
