@@ -6,9 +6,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/fastbuf.h"
-#include "lib/mempool.h"
-#include "lib/md5.h"
+#include "ucw/fastbuf.h"
+#include "ucw/mempool.h"
+#include "ucw/md5.h"
 #include "sherlock/index.h"
 #include "filter/filter.h"
 #include "gather/gather.h"
@@ -40,6 +40,7 @@ gobj_new(struct mempool *pool)
   g->start_time = tv.tv_sec;
   g->start_time_us = tv.tv_usec;
   g->filter_user_mark = F_UNDEF_INT;
+  g->error_msg = "OK";
   return g;
 }
 
@@ -67,42 +68,43 @@ void
 gerror(int code, char *msg, ...)
 {
   va_list args;
-  byte buf[MAX_URL_SIZE + 1024];
-  int n;
-
   alarm(0);
   va_start(args, msg);
-
-  if (gthis->robot_file_p && code == 1)
-    {
-      /* robots.txt is a redirect which we consider to be an error */
-      code = 2305;
-      msg = "Robot file is a redirect";
-    }
-
-  n = vsnprintf(buf, sizeof(buf), msg, args);
-  if (n >= (int) sizeof(buf) || n < 0)
-    die("gerror: message too long");
   gthis->error_code = code;
-  gthis->error_msg = gstrdup(buf);
+  gthis->error_msg = mp_vprintf(gthis->pool, msg, args);
   va_end(args);
   gthis->error_hook();
   die("error_hook has returned");
 }
 
+void
+gobj_set_redirect(char *msg, ...)
+{
+  va_list args;
+  va_start(args, msg);
+  if (gthis->robot_file_p)
+    {
+      /* robots.txt is a redirect which we consider to be an error */
+      gerror(2305, "Robot file is a redirect");
+    }
+  gthis->error_msg = mp_vprintf(gthis->pool, msg, args);
+  gthis->error_code = 1;
+  va_end(args);
+}
+
 static void
 gobj_calc_fb_sum(byte *md5, struct fastbuf *f)
 {
-  struct MD5Context m;
+  md5_context m;
   byte block[4096];
   uns len;
 
-  MD5Init(&m);
+  md5_init(&m);
   f = fbmem_clone_read(f);
   while (len = bread(f, block, sizeof(block)))
-    MD5Update(&m, block, len);
+    md5_update(&m, block, len);
   bclose(f);
-  MD5Final(md5, &m);
+  memcpy(md5, md5_final(&m), MD5_SIZE);
 }
 
 void
@@ -115,7 +117,7 @@ gobj_calc_sum(void)
 }
 
 byte *
-gobj_parse_url(struct url *url, byte *u, byte *msg, uns allow_rel)
+gobj_parse_url(struct url *url, byte *u, byte *m, uns allow_rel)
 {
   int e;
   byte buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE], buf3[MAX_URL_SIZE], buf4[MAX_URL_SIZE];
@@ -131,7 +133,7 @@ gobj_parse_url(struct url *url, byte *u, byte *msg, uns allow_rel)
       if (!gthis->url && !gthis->base_url)	/* We really have no base */
 	goto urlerr;
       if (!allow_rel && log_base_errors)
-	log(L_ERROR_R, "Relative %s URL encountered: %s", msg, u);
+	log(L_ERROR_R, "Relative %s URL encountered: %s", m, u);
       if (gthis->base_url)
 	e = url_normalize(&ur, &gthis->base_url_s);
       else
@@ -152,7 +154,7 @@ gobj_parse_url(struct url *url, byte *u, byte *msg, uns allow_rel)
   return gstrdup(buf4);
 
  urlerr:
-  gerror(2000+e, "Error parsing %s URL %s: %s", msg, u, url_error(e));
+  gerror(2000+e, "Error parsing %s URL %s: %s", m, u, url_error(e));
 }
 
 struct url *
