@@ -5,8 +5,8 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/fastbuf.h"
-#include "lib/conf.h"
+#include "ucw/fastbuf.h"
+#include "ucw/conf.h"
 #include "gather/gather.h"
 
 #include <stdio.h>
@@ -185,7 +185,8 @@ do_path(byte *path)
   if (strcmp(path, b))
     {
       gobj_add_ref('Y', b);	/* gobj_add_ref fortunately accepts relative redirects */
-      gerror(1, "Name not normalized, simulating redirect");
+      gobj_set_redirect("Name not normalized, simulating redirect");
+      return;
     }
 
   if (stat(path, &st))
@@ -206,7 +207,8 @@ do_path(byte *path)
 	{
 	  sprintf(b, "%s/", path);	/* We know there's enough free space for the slash */
 	  gobj_add_ref('Y', b);
-	  gerror(1, "Directory redirect");
+	  gobj_set_redirect("Directory redirect");
+	  return;
 	}
     }
   else
