@@ -6,13 +6,13 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/chartype.h"
-#include "lib/url.h"
-#include "lib/conf.h"
-#include "lib/mempool.h"
-#include "lib/base64.h"
-#include "lib/clists.h"
-#include "lib/simple-lists.h"
+#include "ucw/chartype.h"
+#include "ucw/url.h"
+#include "ucw/conf.h"
+#include "ucw/mempool.h"
+#include "ucw/base64.h"
+#include "ucw/clists.h"
+#include "ucw/simple-lists.h"
 #include "lang/lang.h"
 #include "sherlock/index.h"
 #include "gather/gather.h"
@@ -31,40 +31,41 @@
 /* Configuration parameters */
 
 static uns max_hdr_lines = 1000;
-static byte *local_admin = "";
-static byte *referer = "";
-static byte *acc_charset = "";
-static byte *acc_encoding = "";
-static byte *acc_lang = "";
-static byte *acc_types = "";
+static char *local_admin = "";
+static char *referer = "";
+static char *acc_charset = "";
+static char *acc_encoding = "";
+static char *acc_lang = "";
+static char *acc_types = "";
 static uns connect_timeout = 60;
 static uns header_timeout = 60;
 static uns body_timeout = 600;
-static byte *proxy = "";
+static char *proxy = "";
 static uns proxy_port = 80;
 static uns trace = 0;
 static uns length_checks = 1;
-static byte *user_agent_cmt = "";
+static char *user_agent = "holmes/" SHER_VER;
+static char *user_agent_cmt = "";
 
 static clist list_charset, list_encoding, list_lang, list_types;
 static clist user_headers;
 
 static void
-merge_list(byte **dest, clist *src)
+merge_list(char **dest, clist *src)
 {
   cf_journal_block(dest, sizeof(byte*));
   simp_node *n;
   uns len = 0;
   CLIST_WALK(n, *src)
     len += strlen(n->s) + 2;
-  byte *c = *dest = cf_malloc(len + 1);
+  char *c = *dest = cf_malloc(len + 1);
   CLIST_WALK(n, *src)
     c += sprintf(c, "%s, ", n->s);
   if (c > *dest)
     c[-2] = 0;
 }
 
-static byte *
+static char *
 http_commit(void *ptr UNUSED)
 {
   merge_list(&acc_charset, &list_charset);
@@ -92,6 +93,7 @@ static struct cf_section http_config = {
     CF_UNS("Trace", &trace),
     CF_UNS("LengthChecks", &length_checks),
     CF_LIST("Header", &user_headers, &cf_string_list_config),
+    CF_STRING("UserAgent", &user_agent),
     CF_STRING("UserAgentComment", &user_agent_cmt),
     CF_END
   }
@@ -119,7 +121,7 @@ timeout(int unused UNUSED)
 /* Set up a network connection */
 
 static int
-http_connect(byte *host, uns port)
+http_connect(char *host, uns port)
 {
   struct sockaddr_in rem;
   int sock;
@@ -399,11 +401,10 @@ send_header(FILE *f, byte *host, uns port, byte *rest)
   if (gthis->if_different_etag)
     sendhdr(f, "If-None-Match: %s", gthis->if_different_etag);
 
-  byte *ua = "User-Agent: holmes/" SHER_VER;
   if (user_agent_cmt[0])
-    sendhdr(f, "%s (%s)", ua, user_agent_cmt);
+    sendhdr(f, "User-Agent: %s (%s)", user_agent, user_agent_cmt);
   else
-    sendhdr(f, ua);
+    sendhdr(f, "User-Agent: %s", user_agent);
 
   if (gthis->auth_user && gthis->auth_pass)
   {
@@ -590,7 +591,8 @@ parse_hdr(void)
 	   */
 	  gobj_add_ref('Y', l);
 	}
-      gerror(1, "HTTP redirect %d: %s", code, response_text);
+      gobj_set_redirect("HTTP redirect %d: %s", code, response_text);
+      return;
     }
   else if (code < 200 || code >= 300)	/* Not a `Success' code */
     gerror(2130, "HTTP error %d: %s", code, response_text);
@@ -783,6 +785,8 @@ download_url(byte *host, uns port, byte *rest)
   send_header(f, host, port, rest);
   recv_header(f);
   parse_hdr();
+  if (gthis->error_code == 1)
+    return;
   alarm(body_timeout);
   recv_body(f);
   fclose(f);
