@@ -27,7 +27,7 @@ gather_download(void)
       byte *last_dwn = obj_find_aval(gthis->refreshing, 'D');
       byte *last_mod = obj_find_aval(gthis->refreshing, 'L');
       if (last_dwn &&
-	  gthis->start_time - (sh_time_t)atol(last_dwn) <= max_refresh_age &&
+	  gthis->start_time - (ucw_time_t)atol(last_dwn) <= max_refresh_age &&
 	  !gather_analysis_needed())
 	{
 	  if (last_mod && min_ims_delay >= 0)
