@@ -5,8 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/fastbuf.h"
-#include "lib/stkstring.h"
+#include "sherlock/object.h"
+#include "ucw/fastbuf.h"
+#include "ucw/stkstring.h"
 #include "indexer/indexer.h"
 #include "indexer/params.h"
 
@@ -132,34 +133,50 @@ params_save(struct index_params *params)
   bclose(b);
 }
 
+void
+params_to_obj(struct index_params *params, struct odes *obj, uns final)
+{
+  obj_add_attr_format(obj, 'v', "%08x", params->version ? : INDEX_VERSION);
+  obj_add_attr_num(obj, 't', params->ref_time);
+  obj_add_attr_num(obj, 'n', params->objects_in);
+  obj_add_attr_num(obj, 's', params->sites);
+  obj_add_attr_num(obj, 'l', params->num_slices);
+  obj_add_attr_num(obj, 'r', params->srand);
+  if (final)
+    {
+      obj_add_attr_num(obj, 'm', params->cards_out);
+    }
+}
+
 /* General copying procedure */
 
 uns
-alloc_read_ary(byte *name, byte *suffix, void *Ptr, uns count, uns record)
+alloc_read_ary(const char *name, const char *suffix, void *Ptr, uns count, uns record)
 {
   void **ptr = Ptr;
-  struct fastbuf *fb = index_bopen(stk_strcat(name, suffix), O_RDONLY);
+  struct fastbuf *fb = index_bopen(stk_strcat(name, suffix), O_RDONLY, 0);
   if (count)
     ASSERT(bfilesize(fb) == count * record);
   else
   {
-    sh_off_t size = bfilesize(fb);
+    ucw_off_t size = bfilesize(fb);
     ASSERT(!(size % record));
     count = size / record;
   }
-  *ptr = xmalloc(count * record);
-  bread(fb, *ptr, count * record);
+  *ptr = big_alloc(count * record);
+  if (count && !breadb(fb, *ptr, count * record))
+    ASSERT(0);
   bclose(fb);
   return count;
 }
 
 void
-write_free_ary(byte *name, byte *suffix, void *Ptr, uns count, uns record)
+write_free_ary(const char *name, const char *suffix, void *Ptr, uns count, uns record)
 {
   void **ptr = Ptr;
-  struct fastbuf *fb = index_bopen(stk_strcat(name, suffix), O_WRONLY | O_CREAT | O_TRUNC);
+  struct fastbuf *fb = index_bopen(stk_strcat(name, suffix), O_WRONLY | O_CREAT | O_TRUNC, 0);
   bwrite(fb, *ptr, count * record);
-  xfree(*ptr);
+  big_free(*ptr, count * record);
   *ptr = NULL;
   bclose(fb);
 }
