@@ -4,6 +4,9 @@
  *	(c) 2003 Martin Mares <mj@ucw.cz>
  */
 
+#ifndef _SHERLOCK_INDEXER_ALPHABET_H
+#define _SHERLOCK_INDEXER_ALPHABET_H
+
 enum alphabet_class {
   AC_SPACE,
   AC_ALPHA,
@@ -18,3 +21,5 @@ enum alphabet_class {
 extern byte alpha_class[];
 
 void alphabet_init(void);
+
+#endif
