@@ -5,9 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
 #include "indexer/indexer.h"
 
 #include <stdio.h>
@@ -17,18 +17,11 @@
 
 struct ble { u32 id; };
 #define SORT_PREFIX(x) blk_##x
-#define SORT_KEY struct ble
-#define SORT_PRESORT
-#define SORT_REGULAR
+#define SORT_KEY_REGULAR struct ble
+#define SORT_INT(k) ((k).id)
 #define SORT_INPUT_FB
 #define SORT_OUTPUT_FB
-static inline int
-blk_compare(struct ble *x, struct ble *y)
-{
-  COMPARE(x->id, y->id);
-  return 0;
-}
-#include "lib/sorter.h"
+#include "ucw/sorter/sorter.h"
 
 static void NONRET
 usage(void)
@@ -62,10 +55,11 @@ main(int argc, char **argv)
   for (int i=0; i<N; i++)
     {
       fn_directory = argv[optind+i];
-      in[i] = index_bopen("card-prints", O_RDONLY);
+      in[i] = index_bopen("card-prints", O_RDONLY, 1);
       bzero(&cp[i], sizeof(cp[i]));
     }
-  struct fastbuf *tmp = bopen_tmp(indexer_fb_size);
+  struct fastbuf *tmp = index_bopen_tmp(1);
+  uns max = 0;
   while (breadb(in[0], &cp[0], sizeof(cp[0])))
     {
       int match = 0;
@@ -81,15 +75,18 @@ main(int argc, char **argv)
 	    }
 	}
       if (match)
-	bputl(tmp, cp[0].cardid);
+        {
+	  bputl(tmp, cp[0].cardid);
+	  max = MAX(max, cp[0].cardid);
+	}
     }
   for (int i=0; i<N; i++)
     bclose(in[i]);
 
   fn_directory = argv[optind];
   brewind(tmp);
-  tmp = blk_sort(tmp);
-  struct fastbuf *out = index_bopen(fn_blacklist, O_WRONLY | O_CREAT | O_TRUNC);
+  tmp = blk_sort(tmp, NULL, max);
+  struct fastbuf *out = index_bopen(fn_blacklist, O_WRONLY | O_CREAT | O_TRUNC, 1);
   uns x, last = ~0U;
   while ((x = bgetl(tmp)) != ~0U)
     {
