@@ -7,19 +7,20 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lists.h"
-#include "lib/simple-lists.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/mempool.h"
-#include "lib/url.h"
+#include "ucw/clists.h"
+#include "ucw/simple-lists.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/mempool.h"
+#include "ucw/url.h"
+#include "ucw/unicode.h"
+#include "ucw/hashfunc.h"
+#include "ucw/chartype.h"
 #include "sherlock/object.h"
 #include "sherlock/attrset.h"
 #include "sherlock/conf.h"
-#include "lib/hashfunc.h"
 #include "sherlock/tagged-text.h"
-#include "lib/unicode.h"
 #include "sherlock/lizard-fb.h"
 #include "charset/unicat.h"
 #include "analyser/analyser.h"
@@ -27,6 +28,12 @@
 #include "indexer/lexicon.h"
 #include "indexer/params.h"
 
+#ifdef CONFIG_IMAGES_SIM
+#include "images/images.h"
+#include "images/object.h"
+#include "images/signature.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -75,13 +82,13 @@ static struct cf_section chewer_config = {
   CF_ITEMS {
     CF_UNS("WordBufSize", &word_buf_size),
     CF_UNS("StringBufSize", &string_buf_size),
-    CF_BITMAP_LOOKUP("StringCats", &string_cats, ((byte*[]) {STRING_TYPE_USER_NAMES, NULL}) ),
+    CF_BITMAP_LOOKUP("StringCats", &string_cats, ((const char* const []) {STRING_TYPE_USER_NAMES, NULL}) ),
     CF_UNS("StringMax", &string_max),
     CF_UNS("WordLimit", &word_limit),
     CF_UNS("MetaLimit", &meta_limit),
     CF_UNS("ExcerptMax", &excerpt_max),
     CF_UNS("DocBufSize", &doc_buf_size),
-    CF_BITMAP_LOOKUP("GiantBanMeta", &giant_ban_meta, ((byte*[]) {META_TYPE_USER_NAMES, NULL}) ),
+    CF_BITMAP_LOOKUP("GiantBanMeta", &giant_ban_meta, ((const char* const []) {META_TYPE_USER_NAMES, NULL}) ),
     CF_UNS("GiantPenalty", &giant_penalty),
     CF_SECTION("TypeWeights", type_weights, &word_weights_config),
     CF_UNS("SwindlerThreshold", &swindler_threshold),
@@ -119,7 +126,7 @@ struct out_index {
   struct fastbuf *card_attrs;
   struct fastbuf *cards_out;
   uns cards_compr, cards_uncompr;
-  sh_off_t total_uncompr;
+  ucw_off_t total_uncompr;
   struct fastbuf *string_index;
   u64 string_cnt, string_dropped_cnt;
   struct fastbuf *word_index;
@@ -128,6 +135,10 @@ struct out_index {
   byte *name;
   byte *directory;
   byte *log_prefix;
+#ifdef CONFIG_IMAGES_SIM
+  uns image_signatures_count;
+  struct fastbuf *image_signatures;
+#endif
 };
 
 #define CARD_ID_MASK 0x1fffffff
@@ -312,7 +323,7 @@ string_init(void)
       log(L_WARN, "StringMax entries don't fit in StringBufSize, increasing StringBufSize to %d", string_max * (uns)sizeof(struct sentry));
       sbuf_size = string_max;
     }
-  string_buf = xmalloc(sbuf_size * sizeof(struct sentry));
+  string_buf = big_alloc(sbuf_size * sizeof(struct sentry));
   sbuf_limit = sbuf_size - string_max;
   ITRACE("Allocated string pool with %d entries", sbuf_size);
 }
@@ -320,7 +331,7 @@ string_init(void)
 static void
 string_open(struct out_index *out)
 {
-  out->string_index = index_bopen(fn_string_index, O_WRONLY | O_CREAT | O_TRUNC);
+  out->string_index = index_bopen(fn_string_index, O_WRONLY | O_CREAT | O_TRUNC, 1);
 }
 
 static inline int
@@ -335,16 +346,15 @@ string_cmp(const struct sentry *a, const struct sentry *b)
 
 #define ASORT_PREFIX(x) string_##x
 #define ASORT_KEY_TYPE struct sentry
-#define ASORT_ELT(i) string_buf[i]
 #define ASORT_LT(x,y) string_cmp(&(x), &(y))
-#include "lib/arraysort.h"
+#include "ucw/sorter/array.h"
 
 static void
 string_flush(void)
 {
   if (!sbuf_count)
     return;
-  string_sort(sbuf_count);
+  string_sort(string_buf, sbuf_count);
 
   uns i = 0;
   while (i < sbuf_count)
@@ -379,7 +389,7 @@ string_flush(void)
       struct fastbuf *fb = out->string_index;
       bwrite(fb, &first->fp, sizeof(struct fingerprint));
       bputl(fb, size);
-      sh_off_t expected_end = btell(fb) + size;
+      ucw_off_t expected_end = btell(fb) + size;
       struct sentry *e = &string_buf[start];
       struct sentry *clast = &string_buf[i];
       while (e < clast)
@@ -407,7 +417,7 @@ string_close(struct out_index *out)
 }
 
 static void
-string_add(byte *s, uns type)
+string_add(char *s, uns type)
 {
   if (!(string_cats & (1 << type)))
     return;
@@ -455,9 +465,9 @@ string_card(struct odes *o)
 {
   for (struct oattr *u = obj_find_attr(o, 'U' + OBJ_ATTR_SON); u; u=u->same)
     {
-      byte *url = obj_find_aval(u->son, 'U');
+      char *url = obj_find_aval(u->son, 'U');
       ASSERT(url);
-      byte buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
+      char buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
       struct url ur;
 #ifdef ST_URL
       string_add(url, ST_URL);
@@ -466,7 +476,7 @@ string_card(struct odes *o)
       if (!url_canon_split(url, buf1, buf2, &ur) && ur.host)
 	{
 	  string_add(ur.host, ST_HOST);
-	  byte *dot = strrchr(ur.host, '.');
+	  char *dot = strrchr(ur.host, '.');
 	  if (dot && dot > ur.host)
 	    {
 	      dot--;
@@ -534,7 +544,7 @@ static uns has_title, is_hypertext;
 static void
 preproc_init(void)
 {
-  doc_buf = xmalloc(doc_buf_size+1) + 1;
+  doc_buf = big_alloc(doc_buf_size+1) + 1;
   doc_buf[-1] = 0;
 }
 
@@ -548,7 +558,7 @@ static void
 preproc_card(struct odes *o)
 {
   byte *w = doc_buf;
-  byte *stop = doc_buf + doc_buf_size;
+  byte *stop = doc_buf + doc_buf_size - 5; /* 5 = space or category changer + UTF-8 character + trailing zero */
 
   byte *version = obj_find_aval(o, 'v');
   if (!version)
@@ -572,83 +582,82 @@ preproc_card(struct odes *o)
 #endif
 
   bzero(char_counter, sizeof(char_counter));
-  uns cur_type = WT_TEXT;
   struct oattr *oa = obj_find_attr(o, 'X');
   byte *r;
-  if (oa && (oa->val[0] < 0x80 || oa->val[0] >= 0xa0))
-    {
-      /*
-       *  If by any accident the card doesn't start with a category switch, add it.
-       *  It doesn't mean the card is invalid -- it can start with a ref bracket followed
-       *  by a category switch, but it's better to be safe than sorry. Also, we'd like
-       *  to keep the "first comes the category switch" invariant for the search server.
-       */
-      *w++ = 0x80 | WT_TEXT;
-      r = oa->val;
-      goto run_for_it;
-    }
+  byte *wsp = w;
+  uns c;
+  uns cat = WT_TEXT;
+  uns written_cat = ~0U; /* Always start with the category changer. We assume this invariant in search server. */
   for (; oa; oa=oa->same)
     {
       r = oa->val;
-      uns c;
-
-      if (w >= stop - strlen(r) - 1)
-	{
-	  trim_counter++;
-	  break;
-	}
-      if ((*r < 0x80 || *r >= 0xa0) && r[-1])
-	*w++ = ' ';
-    run_for_it:
-      while (c = *r++)
-	{
-	  if (c < 0x80)
-	  {
-	    *w++ = c;
-	    char_counter[cur_type]++;
-	  }
-	  else if (c >= 0xc0)
+      c = ' ';
+      do
+        {
+	  /* Space */
+	  if (Cspace(c))
 	    {
+	      if (wsp != w) /* Merge multiple spaces & ignore leading spaces */
+	        {
+	          *w++ = ' ';
+	          wsp = w;
+		}
+	    }
+	  /* Printable character (could be slightly improved by removing few ifs) */
+	  else if ((c ^ 0x80) >= 0x40)
+	    {
+	      if (cat != written_cat)
+	        {
+		  if (wsp == w && w != doc_buf) /* Ignore trailing spaces */
+		    w--;
+		  *w++ = cat | 0x80;
+		  wsp = w;
+		  written_cat = cat &= 0x0f;
+		}
+	      char_counter[cat]++;
 	      *w++ = c;
-	      while (*r && (c & 0x40))
-		{
-		  *w++ = *r++;
-		  c <<= 1;
+	      if (c >= 0x80)
+		do
+		  {
+		    ASSERT(*r & 0x80);
+		    *w++ = *r++;
+		    c <<= 1;
+		  }
+		while (c & 0x40);
+              if (unlikely(w >= stop))
+	        {
+		  w = wsp;			/* Remove the partial word */
+		  trim_counter++;
+		  obj_add_attr(o, '.', "Trimmed text");
+		  goto done;
 		}
-	      char_counter[cur_type]++;
 	    }
-	  else				/* Process a sequence of brackets and category changers */
+	  /* Category changer */
+	  else if (c < 0xa0)
 	    {
-	      uns cat = 0x80;
-	      for(;;)
-		{
-		  if (c < 0xa0)		/* Category changes and breaks are compressed */
-		    cat = (cat & 0x10) | (c & 0x1f);
-		  else
-		    {			/* Brackets are to be removed */
-		      if (c < 0xb0 && *r)
-			r++;
-		    }
-		  c = *r;
-		  if (c < 0x80 || c >= 0xc0)
-		    break;
-		  r++;
-		}
-	      if (!(cat & 0x80))
-		{
-		  *w++ = 0x80 + cat;
-		  cur_type = cat & 0x0f;
+	      cat = (cat & 0x10) | (c & 0x1f);
 #ifdef WT_CONVERT_v1_v2
-		  if (upgrade_wt)	// Upgrade v1->v2: change the word-type
-		    {
-		      cur_type = WT_CONVERT_v1_v2(cur_type);
-		      w[-1] = 0x80 | (cat & 0x10) | cur_type;
-		    }
+	      if (upgrade_wt)	// Upgrade v1->v2: change the word-type
+		cat = (cat & 0x10) | WT_CONVERT_v1_v2(cat & 0x0f);
 #endif
+	      if (wsp != w)
+	        {
+	          *w++ = ' ';
+	          wsp = w;
 		}
 	    }
+	  /* Brackets are to be removed */
+	  else if (c < 0xb0)
+	    {
+	      ASSERT(*r);
+	      r++;
+	    }
 	}
+      while (c = *r++);
     }
+done:
+  if (wsp == w && w != doc_buf)
+    w--;
   *w = 0;
   doc_length = w - doc_buf;
   if (doc_length > doc_length_max)
@@ -775,6 +784,9 @@ static void
 penalize_card(struct fastbuf *cards_mem, struct card_attr *attr, struct card_note *note, uns is_image, uns is_in_catalog)
 {
   bput_attr_format(cards_mem, 'W', "s%d", note->weight_scanner);
+#ifdef CONFIG_WEIGHTS
+  bput_attr_format(cards_mem, 'W', "d%d", note->weight_dynamic);
+#endif
   bput_attr_format(cards_mem, 'W', "m%d", note->weight_merged);
 
   int wt = attr->weight;
@@ -835,7 +847,7 @@ typedef struct lentry {
 #define GBUF_TYPE	struct verbum *
 #define GBUF_PREFIX(x)	uw_##x
 #define GBUF_TRACE(msg...) ITRACEN(2, msg)
-#include "lib/gbuf.h"
+#include "ucw/gbuf.h"
 
 static uns lentry_count, lentry_limit;
 static struct verbum **context_words;
@@ -857,11 +869,11 @@ lex_load(void)
   struct fastbuf *b;
   struct verbum *v, **ctxt_last=NULL;
 
-  b = index_bopen(fn_lex_ordered, O_RDONLY);
+  b = index_bopen(fn_lex_ordered, O_RDONLY, 1);
   numerus_verba = bgetl(b);
   if (lex_context_slots)
     {
-      context_words = xmalloc_zero(sizeof(struct verbum *) * lex_context_slots);
+      context_words = big_alloc_zero(sizeof(struct verbum *) * lex_context_slots);
       ctxt_last = alloca(sizeof(struct verbum *) * lex_context_slots);
     }
   for (uns i=0; i<numerus_verba; i++)
@@ -875,14 +887,14 @@ lex_load(void)
 	v = ctxt_last[ctxt]++;
       else
 	{
-	  byte buf[MAX_WORD_LEN+1];
+	  byte buf[MAX_WORD_BYTES+1];
 	  breadb(b, buf, len);
 	  buf[len] = 0;
 	  v = lh_insert(buf, 0);
 	  if (!v)
 	    die("Malformed lexicon: Duplicate word <%s>", buf);
 	  if (class == WC_CONTEXT)
-	    ctxt_last[ctxt] = context_words[ctxt] = xmalloc_zero(sizeof(struct verbum)*2*lex_context_slots);
+	    ctxt_last[ctxt] = context_words[ctxt] = big_alloc_zero(sizeof(struct verbum)*2*lex_context_slots);
 	}
       v->id = id;
       v->u.count = cnt;
@@ -949,7 +961,7 @@ word_flush_single(uns word_id, lentry *head)
 	/* Write header */
 	bputl(fb, word_id);
 	bputl(fb, size);
-	sh_off_t expos = btell(fb) + size;
+	ucw_off_t expos = btell(fb) + size;
 
 	/* Write body */
 	f = head;
@@ -981,10 +993,9 @@ word_flush_single(uns word_id, lentry *head)
 }
 
 #define ASORT_PREFIX(x) word_##x
-#define ASORT_KEY_TYPE u32
-#define ASORT_ELT(i) used_words[i]->id
-#define ASORT_SWAP(i,j) do { struct verbum *tmp=used_words[i]; used_words[i]=used_words[j]; used_words[j]=tmp; } while(0)
-#include "lib/arraysort.h"
+#define ASORT_KEY_TYPE struct verbum *
+#define ASORT_LT(x,y) ((x)->id < (y)->id)
+#include "ucw/sorter/array.h"
 
 static void
 word_flush(int final)
@@ -992,7 +1003,7 @@ word_flush(int final)
   struct verbum **w, *v;
 
   used_words = used_words_buf.ptr;			// save one dereference
-  word_sort(nr_used_words);
+  word_sort(used_words, nr_used_words);
   used_words[nr_used_words] = NULL;
   for (w=used_words; v = *w; w++)
     {
@@ -1241,7 +1252,7 @@ word_init(void)
 static void
 word_open(struct out_index *out)
 {
-  out->word_index = index_bopen(fn_word_index, O_WRONLY | O_CREAT | O_TRUNC);
+  out->word_index = index_bopen(fn_word_index, O_WRONLY | O_CREAT | O_TRUNC, 1);
 }
 
 static void
@@ -1269,10 +1280,10 @@ cards_init(void)
 static void
 cards_open(struct out_index *out)
 {
-  out->cards_out = index_bopen(fn_cards, O_WRONLY | O_CREAT | O_TRUNC);
+  out->cards_out = index_bopen(fn_cards, O_WRONLY | O_CREAT | O_TRUNC, 1);
   lizard_bwrite(out->cards_out, "", 0);
 
-  out->card_attrs = index_bopen(fn_card_attrs, O_WRONLY | O_CREAT | O_TRUNC);
+  out->card_attrs = index_bopen(fn_card_attrs, O_WRONLY | O_CREAT | O_TRUNC, 1);
   struct card_attr a;
   bzero(&a, sizeof(a));			/* Create dummy document 0 to make all read ID's be >0 */
   bwrite(out->card_attrs, &a, sizeof(a));
@@ -1286,7 +1297,7 @@ card_write_start(struct card_attr *attr)
 
   struct out_index *out = current_out;
   uns align = (1 << CARD_POS_SHIFT) - 1;
-  sh_off_t pos = btell(out->cards_out);
+  ucw_off_t pos = btell(out->cards_out);
   while (pos & align)
     {
       bputc(out->cards_out, 0);
@@ -1320,7 +1331,7 @@ cards_close(struct out_index *out)
   bzero(&a, sizeof(a));			/* Append fake attribute marking end of card file */
   card_write_start(&a);
   bwrite(out->card_attrs, &a, sizeof(a));
-  sh_off_t total_compr = btell(out->cards_out);
+  ucw_off_t total_compr = btell(out->cards_out);
   bclose(out->cards_out);
   bclose(out->card_attrs);
   log(L_INFO, "%sGenerated %d cards: %d compressed, %d uncompressed, compressed to %d%%", out->log_prefix,
@@ -1457,7 +1468,7 @@ card_card(struct odes *o, struct card_attr *attr, struct card_note *note)
 static void
 prints_open(struct out_index *out)
 {
-  out->card_prints = index_maybe_bopen(fn_card_prints, O_WRONLY | O_CREAT | O_TRUNC);
+  out->card_prints = index_maybe_bopen(fn_card_prints, O_WRONLY | O_CREAT | O_TRUNC, 1);
 }
 
 static void
@@ -1502,7 +1513,7 @@ static struct mempool *an_pool;
 
 #define GBUF_TYPE	struct odes *
 #define GBUF_PREFIX(x)	ob_##x
-#include "lib/gbuf.h"
+#include "ucw/gbuf.h"
 static ob_t url_block_buf;
 
 static void
@@ -1597,9 +1608,43 @@ params_close(struct out_index *out)
  *  Processing of image signatures
  */
 
+#ifdef CONFIG_IMAGES_SIM
+
+static void
+images_open(struct out_index *out)
+{
+  out->image_signatures = index_bopen(fn_image_signatures_unsorted, O_WRONLY | O_CREAT | O_TRUNC, 1);
+  out->image_signatures_count = 0;
+  bputl(out->image_signatures, 0);
+}
+
+static void
+images_close(struct out_index *out)
+{
+  brewind(out->image_signatures);
+  bputl(out->image_signatures, out->image_signatures_count);
+  bclose(out->image_signatures);
+  log(L_INFO, "Written %u image signatures", out->image_signatures_count);
+}
+
+static void
+images_card(struct odes *o)
+{
+  struct image_signature sig;
+  if (get_image_obj_signature(&sig, o) && sig.len)
+    {
+      struct fastbuf *image_signatures = current_out->image_signatures;
+      bputl(image_signatures, id_to_card(card_id));
+      bwrite(image_signatures, &sig, image_signature_size(sig.len));
+      current_out->image_signatures_count++;
+    }
+}
+
+#else
 static inline void images_open(struct out_index *out UNUSED) {}
 static inline void images_close(struct out_index *out UNUSED) {}
 static inline void images_card(struct odes *o UNUSED) {}
+#endif
 
 /*
  *  Processing of admin export
@@ -1610,7 +1655,16 @@ static struct fastbuf *admin_out;
 static void
 admin_init(void)
 {
-  admin_out = index_maybe_bopen(fn_admin_export, O_CREAT | O_TRUNC | O_WRONLY);
+  if (admin_out = index_maybe_bopen(fn_admin_export, O_CREAT | O_TRUNC | O_WRONLY, 1))
+    {
+      struct odes *o = obj_new(cf_pool);
+      params_to_obj(&global_params, o, 0);
+      bput_attr_format(admin_out, 'v', "%08x", ADMIN_EXPORT_VERSION);
+      bput_attr_push(admin_out, 'p');
+      obj_write(admin_out, o, BUCKET_TYPE_PLAIN);
+      bput_attr_pop(admin_out);
+      bputc(admin_out, '\n');
+    }
 }
 
 static void
@@ -1621,14 +1675,34 @@ admin_end(void)
 }
 
 static void
-admin_card(int idx, struct card_info *info)
+admin_card(int idx, uns id, struct card_info *info)
 {
   if (admin_out)
     {
+      struct card_note *cn = &info->note;
+      struct card_attr *ca = &info->attr;
       struct admin_export n = {
+	.id = id,
 	.subindex = idx,
-	.flags = info->note.flags,
-	.attr = info->attr,
+	.flags = ca->flags,
+	.note_flags = cn->flags,
+	.weight = ca->weight,
+	.weight_scanner = cn->weight_scanner,
+#ifdef CONFIG_WEIGHTS
+	.weight_dynamic = cn->weight_dynamic,
+#endif
+	.weight_merged = cn->weight_merged,
+#ifdef CONFIG_LASTMOD
+	.age = ca->age,
+#endif
+#ifdef CONFIG_FILETYPE
+	.file_flags = ca->type_flags,
+	.file_type = CA_GET_FILE_TYPE(ca),
+	.file_lang = CA_GET_FILE_LANG(ca),
+#endif
+#ifdef CONFIG_SITES
+	.site = ca->site_id,
+#endif
       };
       memcpy(n.footprint, info->note.footprint, 16);
       bwrite(admin_out, &n, sizeof(n));
@@ -1673,7 +1747,7 @@ chew_card(struct card_info *info, struct odes *o)
   CUSTOM_CHEW_ATTRS(o, attr);
 #endif
   bwrite(current_out->card_attrs, attr, sizeof(*attr));
-  admin_card(idx, info);
+  admin_card(idx, current_out->card_cnt, info);
 
   PROGRESS(fetch_id, "chewer: %d cards (%d%%)",
 	   fetch_id, (int)((float)fetch_id/fetch_num_ids*100));
@@ -1694,12 +1768,12 @@ main(int argc, char **argv)
 
   out_init();
   preproc_init();
-  cards_init();
   word_init();
   string_init();
   params_init();
   analyse_init();
   admin_init();
+  cards_init();
   byte *orig_directory = fn_directory;
   for (uns i=0; i<num_out_indices; i++)
     {
