@@ -4,7 +4,10 @@
  *	(c) 2006 Robert Spalek <robert@ucw.cz>
  */
 
-#include "lib/ff-binary.h"
+#ifndef _SHERLOCK_INDEXER_GRAPH_H
+#define _SHERLOCK_INDEXER_GRAPH_H
+
+#include "ucw/ff-binary.h"
 
 /* Graph edge types */
 
@@ -59,3 +62,5 @@ bget_graph_hdr(struct fastbuf *fb, u32 *node, u32 *deg)
   }
   return 1;
 }
+
+#endif
