@@ -16,17 +16,19 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/clists.h"
-#include "lib/url.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/chartype.h"
-#include "lib/mempool.h"
 #include "sherlock/object.h"
-#include "lib/bitarray.h"
-#include "lib/unicode.h"
+#include "ucw/clists.h"
+#include "ucw/url.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/chartype.h"
+#include "ucw/mempool.h"
+#include "ucw/bitarray.h"
+#include "ucw/unicode.h"
+#include "ucw/regex.h"
+#include "ucw/string.h"
 #include "charset/unicat.h"
 #include "indexer/indexer.h"
 #include "indexer/merges.h"
@@ -43,7 +45,7 @@
 struct url_pattern {
   cnode n;
   regex *rx;
-  byte *replacement;
+  char *replacement;
   uns weight;
   uns use_hardcoded;
 };
@@ -57,15 +59,15 @@ static uns limit_count[5][3][4];
 	   [type: 0=URL, 1=File, 2=Catalog]
 	   [weight: 0..3] */
 
-static byte *
-compile_regexp(uns number UNUSED, byte **pars, regex **rx)
+static char *
+compile_regexp(uns number UNUSED, char **pars, regex **rx)
 {
   cf_journal_block(rx, sizeof(void*));
   *rx = rx_compile(pars[0], 0);
   return NULL;
 }
 
-static byte *
+static char *
 pattern_commit(struct url_pattern *ptr)
 {
   if (ptr->weight > 3)
@@ -86,7 +88,7 @@ static struct cf_section pattern_config = {
   }
 };
 
-static byte *
+static char *
 keywords_init(void *ptr UNUSED)
 {
   memset(limit_count, -1, sizeof(limit_count));
@@ -127,9 +129,9 @@ static struct fastbuf *keywords;
 static uns uk_cnt, nk_cnt;
 
 static int
-hardcoded_parse_url(uns type, byte *url, byte *buf, uns buf_len UNUSED)
+hardcoded_parse_url(uns type, char *url, char *buf, uns buf_len UNUSED)
 {
-  byte url_buf[MAX_URL_SIZE];
+  char url_buf[MAX_URL_SIZE];
   struct url urlp;
   if (url_split(url, &urlp, url_buf))
     return -1;
@@ -144,7 +146,7 @@ hardcoded_parse_url(uns type, byte *url, byte *buf, uns buf_len UNUSED)
 	|| (urlp.port != ~0U && urlp.port != 80)
 	|| urlp.user || urlp.pass)
       return -1;
-    byte *d;
+    char *d;
     if ((d = strchr(urlp.rest, '.')))
     {
       uns l = d - urlp.rest;
@@ -162,8 +164,8 @@ hardcoded_parse_url(uns type, byte *url, byte *buf, uns buf_len UNUSED)
       if (urlp.rest[0] != '/' || urlp.rest[1])
 	return -1;
     }
-    byte *w[3];
-    int pars = sepsplit(urlp.host, '.', w, 3);
+    char *w[3];
+    int pars = str_sepsplit(urlp.host, '.', w, 3);
     if (!pars || pars == 1)
       return -1;
     if (pars == 2 || (pars == 3 && !strncasecmp(w[0], "www", 3)))
@@ -203,7 +205,7 @@ hardcoded_parse_url(uns type, byte *url, byte *buf, uns buf_len UNUSED)
     FileWordPattern	....://[^?]*/(([^/.?=;]+)|([^/?=;]+)\.[^/.?=;]*)/?$	\2\3	2
     FileWordPattern	....://[^?]*/(([^/.?=;]+)|([^/?=;]+)\.[^/.?=;]*)/?\?.*	\2\3	1
 #endif
-    byte *p = strchr(urlp.rest, '?');
+    char *p = strchr(urlp.rest, '?');
     uns level;
     if (p)
     {
@@ -217,10 +219,10 @@ hardcoded_parse_url(uns type, byte *url, byte *buf, uns buf_len UNUSED)
     }
     if (p > urlp.rest + 1 && p[-1] == '/')
       *--p = 0;
-    byte *s = strrchr(urlp.rest, '/');
+    char *s = strrchr(urlp.rest, '/');
     ASSERT(s);
     s++;
-    byte *d = (byte*) strchr(s, '.') ? : p;
+    char *d = strchr(s, '.') ? : p;
     if (d == s || (d != p && strchr(d+1, '.')))
       return -1;
     for (p=s; *p; p++)
@@ -275,7 +277,7 @@ generate_url_keywords(bitarray_t is_image)
   byte url[MAX_URL_SIZE], url2[MAX_URL_SIZE], buf[LINEBUF_SIZE];
   uns i=0;
 
-  ul = index_bopen(fn_urls, O_RDONLY);
+  ul = index_bopen(fn_urls, O_RDONLY, 1);
   while (bgets(ul, url, sizeof(url)))
     {
       if (url_deescape(url, url2))
@@ -364,11 +366,12 @@ lm_got_complex(uns pos, uns cat, word_id_t wroot, word_id_t wctxt, uns after, vo
 #include "indexer/lexmap.h"
 
 static byte **lh_words;		// for friendly dumping
+static uns lh_words_size;
 
 static void
 lh_dump(void)
 {
-  lh_words = xmalloc_zero((lh_id/8 + 1) * sizeof(byte *));
+  lh_words = big_alloc_zero(lh_words_size = ((lh_id/8 + 1) * sizeof(byte *)));
   TRACE(2, "\nLexical hash table:\n\n");
   LH_WALK(v)
   {
@@ -379,13 +382,12 @@ lh_dump(void)
 
 /*** Sorter ***/
 
-#define	SORT_KEY	keyword_t
-#define	SORT_PREFIX(x)	keyword_##x
-#define	SORT_PRESORT
+#define	SORT_PREFIX(x)		keyword_##x
+#define	SORT_KEY_REGULAR	keyword_t
 #define	SORT_INPUT_FB
 #define	SORT_OUTPUT_FB
-#define	CMP(attr)	if (a->attr != b->attr) return (a->attr < b->attr) ? -1 : 1;
-#define	REVCMP(attr)	if (a->attr != b->attr) return (a->attr < b->attr) ? 1 : -1;
+#define	CMP(attr)		COMPARE(a->attr, b->attr)
+#define	REVCMP(attr)		REV_COMPARE(a->attr, b->attr)
 #define	CMP_EQ1st(attr,cmpattr) \
   if ((a->attr == a->cmpattr) ^ (b->attr == b->cmpattr)) \
     return (a->attr == a->cmpattr) ? -1 : 1; \
@@ -403,34 +405,34 @@ keyword_compare(keyword_t *a, keyword_t *b)
 {
   if (sort_phase == 1 || sort_phase == 2)	// prune cheaters with many redirects
   {
-    CMP(target)
-    CMP(type)
-    CMP(weight)
-    if (sort_phase == 1)	//1st pass: site and URL limits
-      { CMP(source_site) }	//2nd pass: total limit
+    CMP(target);
+    CMP(type);
+    CMP(weight);
+    if (sort_phase == 1)			//1st pass: site and URL limits
+      CMP(source_site);				//2nd pass: total limit
     if ((a->source == a->target) ^ (b->source == b->target))
       return (a->source == a->target) ? -1 : 1;
-    REVCMP(source_pagerank)
-    CMP(source)
-    CMP(order)
+    REVCMP(source_pagerank);
+    CMP(source);
+    CMP(order);
     if (trace_level >= 1)
       log(L_WARN_R, "Duplicate record %x of type %c and weight %c", a->source, a->type, a->weight);
     return 0;
   }
-  else if (sort_phase == 3)		// prune common words
+  else if (sort_phase == 3)			// prune common words
   {
-    CMP(word)
-    CMP(type)
-    CMP(weight)
+    CMP(word);
+    CMP(type);
+    CMP(weight);
     return 0;
   }
   else if (sort_phase == 4)		// give the data a nice order
   {
-    CMP(target)
-    CMP_EQ1st(source_site, target_site)
-    CMP_EQ1st(source, target)
-    CMP(type)
-    CMP(order)
+    CMP(target);
+    CMP_EQ1st(source_site, target_site);
+    CMP_EQ1st(source, target);
+    CMP(type);
+    CMP(order);
     if (trace_level >= 1)
       log(L_WARN_R, "Duplicate record %x of type %c", a->source, a->type);
     return 0;
@@ -440,31 +442,7 @@ keyword_compare(keyword_t *a, keyword_t *b)
   return 0;
 }
 
-static inline int
-keyword_fetch_key(struct fastbuf *f, keyword_t *r)
-{
-  return bread(f, r, sizeof(keyword_t));
-}
-
-static inline byte *
-keyword_fetch_item(struct fastbuf *f UNUSED, keyword_t *r, byte *limit UNUSED)
-{
-  return (byte*) (r+1);
-}
-
-static void
-keyword_store_item(struct fastbuf *f, keyword_t *r)
-{
-  bwrite(f, r, sizeof(keyword_t));
-}
-
-static inline void
-keyword_copy_data(struct fastbuf *src UNUSED, struct fastbuf *dest, keyword_t *r)
-{
-  keyword_store_item(dest, r);
-}
-
-#include "lib/sorter.h"
+#include "ucw/sorter/sorter.h"
 
 static void
 dump_binary_stream(struct fastbuf *binary)
@@ -482,13 +460,13 @@ resort(struct fastbuf **F, uns phase)
 {
   sort_phase = phase;
   struct fastbuf *f = *F;
-  bsetpos(f, 0);
-  f = keyword_sort(f);
+  brewind(f);
+  f = keyword_sort(f, NULL);
   brewind(f);
   if (trace_level >= 2)
   {
     dump_binary_stream(f);
-    bsetpos(f, 0);
+    brewind(f);
   }
   *F = f;
 }
@@ -516,7 +494,7 @@ static void
 filter_cheaters(struct fastbuf **F, uns consider_total_limit)
 {
   struct fastbuf *in = *F;
-  struct fastbuf *out = bopen_tmp(indexer_fb_size);
+  struct fastbuf *out = index_bopen_tmp(1);
   keyword_t last = { .word = ~0U, .target = 0, .source_site = 0, .source = 0 }, this;
   uns url_cnt = 0, site_cnt = 0, total_cnt = 0;
   bzero(deleted, sizeof(deleted));
@@ -583,7 +561,7 @@ filter_cheaters(struct fastbuf **F, uns consider_total_limit)
 }
 
 static void
-flush_words(struct fastbuf *in, sh_off_t start_pos, uns count, struct fastbuf *out, keyword_t *last)
+flush_words(struct fastbuf *in, ucw_off_t start_pos, uns count, struct fastbuf *out, keyword_t *last)
 {
   if (!count)
     return;
@@ -625,10 +603,10 @@ static void
 filter_common_words(struct fastbuf **F)
 {
   struct fastbuf *in = *F;
-  struct fastbuf *out = bopen_tmp(indexer_fb_size);
+  struct fastbuf *out = index_bopen_tmp(1);
   keyword_t last, this;
   uns cnt = 0;
-  sh_off_t seek = 0;
+  ucw_off_t seek = 0;
   bzero(deleted, sizeof(deleted));
   last.word = ~0U;
   while (bread(in, &this, sizeof(keyword_t)))
@@ -674,7 +652,7 @@ filter_common_words(struct fastbuf **F)
 static void
 dump_keywords(struct fastbuf *labels, byte *buf, keyword_t *rec, byte min_wt)
 {
-  if (rec->source == ~0U)	//TODO: dump nothing if all words have weight 0
+  if (rec->source == (u32)~0U)	//TODO: dump nothing if all words have weight 0
     return;
   static byte types[3] = { MT_URL_KEYWD, MT_FILE,
 #ifdef MT_CAT_KW_USER
@@ -796,10 +774,9 @@ main(int argc, char **argv)
   if (phases & 1)
   {
     log(L_INFO, "Going to parse URLs into URL and File keywords");
-    keywords = index_bopen(fn_keywords, O_RDWR | O_CREAT | (truncate&1 ? O_TRUNC : O_APPEND));
+    keywords = index_bopen(fn_keywords, O_RDWR | O_CREAT | (truncate&1 ? O_TRUNC : O_APPEND), 0);
 
-    bitarray_t is_image = xmalloc(BIT_ARRAY_BYTES(card_count));
-    bit_array_zero(is_image, card_count);
+    bitarray_t is_image = big_alloc_zero(BIT_ARRAY_BYTES(card_count));
     for (uns i=0; i<card_count; i++)
       if (bring_note(i)->flags & (CARD_NOTE_IMAGE | CARD_NOTE_AUDIO))
 	bit_array_set(is_image, i);
@@ -808,12 +785,12 @@ main(int argc, char **argv)
     generate_url_keywords(is_image);
     merges_unmap();
 
-    xfree(is_image);
+    big_free(is_image, BIT_ARRAY_BYTES(card_count));
     bflush(keywords);
   }
   else
   {
-    keywords = index_bopen(fn_keywords, O_RDONLY);
+    keywords = index_bopen(fn_keywords, O_RDONLY, 1);
   }
 
   if (phases & 2)
@@ -821,7 +798,11 @@ main(int argc, char **argv)
     log(L_INFO, "Going to translate keywords into binary form");
     byte *card_weights;
     READ_ATTR(card_weights, weight);
-    binary = bopen_tmp(indexer_fb_size);
+#ifdef CONFIG_SITES
+    u32 *card_sites;
+    READ_ATTR(card_sites, site_id);
+#endif
+    binary = index_bopen_tmp(1);
     lm_init();
     lh_init();
     byte buf[LINEBUF_SIZE];
@@ -832,15 +813,23 @@ main(int argc, char **argv)
       cnt = sscanf(buf, "%c%x %x %c %n", &kw.type, &kw.source, &kw.target, &kw.weight, &len);
       if (cnt != 4)
         die("Invalid format: %s", buf);
+#ifdef	CONFIG_SITES
+      kw.source_site = card_sites[kw.source];
+      kw.target_site = card_sites[kw.target];
+#else
       kw.source_site = 0;
       kw.target_site = 0;
+#endif
       kw.source_pagerank = card_weights[kw.source];
       lm_doc_start(NULL);
       cplx_pos = -1;
       lm_map_text(buf + len);
     }
     lh_dump();
-    xfree(card_weights);
+#ifdef CONFIG_SITES
+    FREE_ATTR(card_sites);
+#endif
+    FREE_ATTR(card_weights);
     bflush(binary);
     log(L_INFO, "Written %d words", written);
 
@@ -849,17 +838,22 @@ main(int argc, char **argv)
     resort(&binary, 2);
     filter_cheaters(&binary, 1);
     resort(&binary, 3);
+
+    /* Make the fastbuf seekable */
+    struct fastbuf *stream = binary;
+    binary = bopen_file(stream->name, O_RDONLY, &indexer_fb_params); 
     filter_common_words(&binary);
+    bclose(stream);
 
     /* Dump the keywords into labels.  */
     resort(&binary, 4);
-    struct fastbuf *labels = index_bopen(fn_labels_by_id, O_WRONLY | O_CREAT | (truncate&2 ? O_TRUNC : O_APPEND));
+    struct fastbuf *labels = index_bopen(fn_labels_by_id, O_WRONLY | O_CREAT | (truncate&2 ? O_TRUNC : O_APPEND), 0);
     put_attr_set_type(BUCKET_TYPE_V33);
     dump_into_labels(labels, binary);
     bclose(labels);
     bclose(binary);
 
-    xfree(lh_words);
+    big_free(lh_words, lh_words_size);
   }
   bclose(keywords);
   notes_part_unmap();
