@@ -6,8 +6,8 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/mempool.h"
+#include "ucw/conf.h"
+#include "ucw/mempool.h"
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
 
@@ -22,10 +22,10 @@
 struct lexicon_config lexicon_config = {
   .min_len_ign = 1,
   .min_len = 1,
-  .max_len = MAX_WORD_LEN,
-  .max_num_len = MAX_WORD_LEN,
-  .max_mixed_len = MAX_WORD_LEN,
-  .max_ctrl_len = MAX_WORD_LEN,
+  .max_len = MAX_WORD_CHARS,
+  .max_num_len = MAX_WORD_CHARS,
+  .max_mixed_len = MAX_WORD_CHARS,
+  .max_ctrl_len = MAX_WORD_CHARS,
   .max_gap = 3,
   .context_slots = 256
 };
@@ -49,7 +49,7 @@ copy_exc(clist *src, enum word_class class)
     }
 }
 
-static byte *
+static char *
 merge_lists(void *ptr UNUSED)
 {
   CF_JOURNAL_VAR(lex_exceptions);
@@ -67,8 +67,8 @@ merge_lists(void *ptr UNUSED)
 }
 
 #ifndef CONFIG_CONTEXTS
-static byte *
-no_contexts(uns number UNUSED, byte **pars UNUSED, void *ptr UNUSED)
+static char *
+no_contexts(uns number UNUSED, char **pars UNUSED, void *ptr UNUSED)
 {
   return "Support for word contexts not compiled in";
 }
@@ -78,7 +78,7 @@ static struct cf_section word_list_config = {
   CF_TYPE(struct exception),
 #define F(x)	PTR_TO(struct exception, x)
   CF_ITEMS {
-    CF_STRING("Word", F(w)),
+    CF_STRING("Word", (char **)F(w)),
     CF_END
   }
 #undef F
