@@ -4,10 +4,10 @@
  *	(c) 2001--2005 Martin Mares <mj@ucw.cz>
  */
 
-#ifndef _INDEXER_LEXICON_H
-#define _INDEXER_LEXICON_H
+#ifndef _SHERLOCK_INDEXER_LEXICON_H
+#define _SHERLOCK_INDEXER_LEXICON_H
 
-#include "lib/clists.h"
+#include "ucw/clists.h"
 
 /* Word classes */
 
