@@ -12,13 +12,13 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/hashfunc.h"
-#include "lib/math.h"
-#include "lib/mempool.h"
-#include "lib/unicode.h"
+#include "sherlock/math.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/hashfunc.h"
+#include "ucw/mempool.h"
+#include "ucw/unicode.h"
 #include "charset/unicat.h"
 #include "indexer/indexer.h"
 #include "indexer/lexicon.h"
@@ -49,7 +49,7 @@ lex_load_words(struct fastbuf *s, uns cnt)
 {
   for (uns i=0; i<cnt; i++)
     {
-      byte buf[MAX_WORD_LEN+1];
+      byte buf[MAX_WORD_BYTES+1];
       u32 origid = bgetl(s);
       uns class = origid & 7;
       ASSERT(class != WC_COMPLEX);
@@ -186,11 +186,11 @@ find_class_reps(void)
       total += equiv[i] & 0x7fffffff;
   DBG("%d words in non-trivial classes", total);
 
-  f_classes = index_maybe_bopen(fn_lex_classes, O_WRONLY|O_CREAT|O_TRUNC);
+  f_classes = index_maybe_bopen(fn_lex_classes, O_WRONLY|O_CREAT|O_TRUNC, 1);
 
   if (total)			/* There are non-trivial classes */
     {
-      uns *cl = xmalloc(sizeof(uns) * total);
+      uns *cl = big_alloc(sizeof(uns) * total);
       total = 0;
       /* Prepare slots for classes, each class starts with its root */
       for (uns i=1; i<=n_words; i++)
@@ -221,7 +221,7 @@ find_class_reps(void)
 	  while (k < l)
 	    equiv[cl[k++]] = rep;
 	}
-      xfree(cl);
+      big_free(cl, sizeof(uns) * total);
     }
   /* Trivial classes are their own representatives */
   for (uns i=1; i<=n_words; i++)
@@ -246,7 +246,7 @@ generate_context_words(void)
 
   uns cnt = 0;
   uns nn = n_words;
-  ctxt_info = xmalloc_zero(sizeof(struct ctxt_info) * n_context_words);
+  ctxt_info = big_alloc_zero(sizeof(struct ctxt_info) * n_context_words);
   for (uns i=0; i<n_words; i++)
     {
       struct verbum *v = word_array[i];
@@ -254,7 +254,7 @@ generate_context_words(void)
 	{
 	  struct ctxt_info *c = &ctxt_info[cnt];
 	  c->root = v;
-	  c->contexts = xmalloc_zero(sizeof(struct ctxt_verbum) * 2 * lex_context_slots);
+	  c->contexts = big_alloc_zero(sizeof(struct ctxt_verbum) * 2 * lex_context_slots);
 	  for (uns j=0; j<2*lex_context_slots; j++)
 	    {
 	      struct verbum *w = &c->contexts[j].v;
@@ -343,7 +343,7 @@ assign_context_ids(void)
 static struct fastbuf *
 generate_stems(void)
 {
-  struct fastbuf *b = bopen_tmp(indexer_fb_size);
+  struct fastbuf *b = index_bopen_tmp(0);
   struct stemmer *st;
   struct mempool *mp;
 
@@ -405,7 +405,7 @@ generate_syn_classes(struct fastbuf *b)
 
   mp = mp_new(4096);
   log(L_INFO, "Constructing synonymic classes");
-  struct fastbuf *c = bopen(b->name, O_RDONLY, indexer_fb_size);
+  struct fastbuf *c = bopen_file(b->name, O_RDONLY, &indexer_fb_params);
   bsetpos(c, btell(b));
   CLIST_WALK(s, syndict_list)
     {
@@ -486,7 +486,7 @@ stem_equivs(struct fastbuf *s)
 static void
 renumber_stems(struct fastbuf *s)
 {
-  struct fastbuf *d = index_bopen(fn_stems_ordered, O_WRONLY|O_CREAT|O_TRUNC);
+  struct fastbuf *d = index_bopen(fn_stems_ordered, O_WRONLY|O_CREAT|O_TRUNC, 0);
   u32 stid, langmask, x, y;
 
   bsetpos(s, 0);
@@ -525,8 +525,8 @@ accent_lt(struct verbum *x, struct verbum *y)
 
   while (1)
     {
-      GET_UTF8(xx, xu);
-      GET_UTF8(yy, yu);
+      xx = utf8_get(xx, &xu);
+      yy = utf8_get(yy, &yu);
       if (!xu || !yu)
 	break;
       xuu = Uunaccent(xu);
@@ -564,8 +564,8 @@ accentless_same(struct verbum *x, struct verbum *y)
 
   while (1)
     {
-      GET_UTF8(xx, xu);
-      GET_UTF8(yy, yu);
+      xx = utf8_get(xx, &xu);
+      yy = utf8_get(yy, &yu);
       if (!xu || !yu)
 	return (!xu && !yu);
       if (Uunaccent(xu) != Uunaccent(yu))
@@ -575,9 +575,8 @@ accentless_same(struct verbum *x, struct verbum *y)
 
 #define ASORT_PREFIX(id) accent_##id
 #define ASORT_KEY_TYPE struct verbum *
-#define ASORT_ELT(i) word_array[i]
 #define ASORT_LT(x,y) accent_lt(x,y)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array.h"
 
 static void
 accent_equivs(void)
@@ -636,9 +635,8 @@ word_lt(struct verbum *x, struct verbum *y)
 
 #define ASORT_PREFIX(id) word_##id
 #define ASORT_KEY_TYPE struct verbum *
-#define ASORT_ELT(i) word_array[i]
 #define ASORT_LT(x,y) word_lt(x,y)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array.h"
 
 /* Squeezing of word frequencies to range [0,255] */
 
@@ -673,10 +671,10 @@ main(int argc, char **argv)
   }
 
   /* Load words from the lexicon to memory */
-  struct fastbuf *s = index_bopen(fn_lex_raw, O_RDONLY);
+  struct fastbuf *s = index_bopen(fn_lex_raw, O_RDONLY, 0);
   n_words = bgetl(s);
   log(L_INFO, "Loading lexicon: %d words", n_words);
-  word_array = xmalloc(sizeof(struct verbum *) * n_words);
+  word_array = big_alloc(sizeof(struct verbum *) * n_words);
   lh_init();
   lex_load_words(s, n_words);
   bclose(s);
@@ -689,15 +687,15 @@ main(int argc, char **argv)
 #endif
 
   /* Generate array of all words indexed by original word ID */
-  xfree(word_array);
+  big_free(word_array, sizeof(struct verbum *) * n_words);
   n_words = lh_id/8;
   LH_WALK(l)
     if ((l->id & 7) == WC_CONTEXT)
       n_context_words++;
   n_all_words = n_words + 2*lex_context_slots*n_context_words;
-  word_array = xmalloc(sizeof(struct verbum *) * n_all_words);
+  word_array = big_alloc(sizeof(struct verbum *) * n_all_words);
   DBG("Gathering IDs for %d words", n_all_words);
-  id_to_word = xmalloc(sizeof(struct verbum *) * (n_all_words+1));
+  id_to_word = big_alloc(sizeof(struct verbum *) * (n_all_words+1));
   uns cnt = 0;
   LH_WALK(l)
     {
@@ -709,11 +707,11 @@ main(int argc, char **argv)
 
   /* Now we know all the words, so let's group them to equivalence classes and find their representatives */
   log(L_INFO, "Constructing equivalence classes for %d words", n_all_words);
-  equiv = xmalloc(sizeof(uns) * (n_all_words+1));
+  equiv = big_alloc(sizeof(uns) * (n_all_words+1));
   for (uns i=1; i<=n_all_words; i++)
     equiv[i] = 0x80000001;
   log(L_INFO, "Merging classes according to accents");
-  accent_sort(n_words);
+  accent_sort(word_array, n_words);
   accent_equivs();
 #ifdef CONFIG_LANG
   log(L_INFO, "Merging classes according to stems");
@@ -728,7 +726,7 @@ main(int argc, char **argv)
 
   /* Sort the word array and assign new IDs to words */
   log(L_INFO, "Sorting words");
-  word_sort(n_words);
+  word_sort(word_array, n_words);
   lex_assign_ids(word_array, n_words);
 
 #ifdef CONFIG_CONTEXTS
@@ -741,7 +739,7 @@ main(int argc, char **argv)
 
   /* Write out everything we know */
   log(L_INFO, "Writing final lexicon");
-  struct fastbuf *d = index_bopen(fn_lex_ordered, O_WRONLY|O_CREAT|O_TRUNC);
+  struct fastbuf *d = index_bopen(fn_lex_ordered, O_WRONLY|O_CREAT|O_TRUNC, 0);
   bputl(d, n_words);
   for (uns i=0; i<n_words; i++)
     {
