@@ -4,6 +4,9 @@
  *	(c) 2002, Robert Spalek <robert@ucw.cz>
  */
 
+#ifndef _SHERLOCK_INDEXER_MATCHER_H
+#define _SHERLOCK_INDEXER_MATCHER_H
+
 #include "sherlock/sherlock.h"
 
 struct hash_permuter
@@ -25,3 +28,5 @@ struct matcher_context {
 void matcher_init(void);
 struct matcher_context *matcher_new(struct matcher_context *clone);
 uns matcher_compute_minima(u32 *min, struct matcher_context *ctxt, struct oattr *oa);
+
+#endif
