@@ -5,14 +5,14 @@
  *	(c) 2005 Robert Spalek <robert@ucw.cz>
  */
 
-#ifndef _SHERLOCK_PARAMS_H
-#define _SHERLOCK_PARAMS_H
+#ifndef _SHERLOCK_INDEXER_PARAMS_H
+#define _SHERLOCK_INDEXER_PARAMS_H
 
 #include "indexer/lexicon.h"
 
 struct index_params {
   u32 version;				/* INDEX_VERSION */
-  sh_time_t ref_time;			/* Reference time (for document ages etc.) */
+  ucw_time_t ref_time;			/* Reference time (for document ages etc.) */
   struct lexicon_config lex_config;
   u32 objects_in;			/* Number of objects on the input of the indexer */
   u32 sites;				/* Number of unique sites */
@@ -25,12 +25,15 @@ struct index_params {
 };
 
 static inline void
-get_slice_breaks(struct index_params *par, uns *slice_breaks)
+get_slice_start(struct index_params *par, uns *slice_start)
 {
-  uns i;
-  for (i=0; i<par->num_slices; i++)
-    slice_breaks[i] = (i+1)*par->cards_out/par->num_slices;
-  slice_breaks[i] = ~0U;
+  // slice i contains OIDs from the range [ slice_start[i], slice_start[i+1] )
+  for (uns i = 0; i <= par->num_slices; i++)
+    slice_start[i] = (u64)i * par->cards_out / par->num_slices;
 }
 
+struct odes;
+
+void params_to_obj(struct index_params *par, struct odes *obj, uns final);
+
 #endif
