@@ -5,8 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/unaligned.h"
 #include "indexer/indexer.h"
 
 #include <stdio.h>
@@ -14,10 +15,9 @@
 
 /* This is almost the same as fpsort.c */
 
-#define SORT_KEY struct card_print
 #define SORT_PREFIX(x) cp_##x
-#define SORT_PRESORT
-#define SORT_REGULAR
+#define SORT_KEY_REGULAR struct card_print
+#define SORT_HASH_BITS 32
 #define SORT_INPUT_FILE
 #define SORT_OUTPUT_FILE
 
@@ -31,7 +31,13 @@ cp_compare(struct card_print *a, struct card_print *b)
   return 0;
 }
 
-#include "lib/sorter.h"
+static inline uns
+cp_hash(struct card_print *a)
+{
+  return get_u32_be((void *)&a->fp);
+}
+
+#include "ucw/sorter/sorter.h"
 
 int
 main(int argc, char **argv)
