@@ -9,24 +9,24 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/md5.h"
-#include "lib/mempool.h"
-#include "lib/url.h"
-#include "lib/bitarray.h"
-#include "lib/base224.h"
-#include "lib/math.h"
-#include "lib/lfs.h"
-#include "lib/unicode.h"
-#include "lib/hashfunc.h"
-#include "lib/semaphore.h"
-#include "lib/threads.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/md5.h"
+#include "ucw/mempool.h"
+#include "ucw/url.h"
+#include "ucw/bitarray.h"
+#include "ucw/base224.h"
+#include "ucw/lfs.h"
+#include "ucw/unicode.h"
+#include "ucw/hashfunc.h"
+#include "ucw/semaphore.h"
+#include "ucw/threads.h"
 #include "sherlock/object.h"
 #include "sherlock/attrset.h"
 #include "sherlock/tagged-text.h"
+#include "sherlock/math.h"
 #include "charset/unicat.h"
 #include "indexer/indexer.h"
 #include "indexer/matcher.h"
@@ -37,6 +37,12 @@
 #include "lang/lang.h"
 #include "lang/detect.h"
 
+#ifdef CONFIG_IMAGES_DUP
+#include "indexer/images.h"
+#include "images/object.h"
+#include "images/duplicates.h"
+#endif
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -52,10 +58,12 @@ static struct index_params parameters;
 /* Configuration */
 
 static uns scan_batch = 10;
+static uns filter_links;
 
 static struct cf_section scanner_config = {
   CF_ITEMS {
     CF_UNS("BatchSize", &scan_batch),
+    CF_UNS("FilterLinks", &filter_links),
     CF_END
   }
 };
@@ -136,6 +144,15 @@ struct scan_context {
   int audio_bitrate;
   int audio_srate;
   int audio_channels;
+  uns want_xform;
+  uns noindex;
+
+  /* Transformation of links */
+  struct filter_args *xform_args;
+  byte *link;
+  struct url link_s;
+  byte *url_xform;
+  byte link_buf[MAX_URL_SIZE];
 };
 
 static struct scan_context *scan_contexts;
@@ -175,6 +192,7 @@ static uns id_skel = FIRST_ID_SKEL;
 
 static struct fastbuf *urls;
 static struct fastbuf *url_index;
+static struct fastbuf *skel_urls;
 
 static pthread_mutex_t alloc_mutex;		/* Protects access to the global state */
 static sem_t *alloc_flush_sem;			/* Used for waiting for all threads on flush */
@@ -187,7 +205,7 @@ static int
 alloc_open_file(char *name)
 {
   name = index_name(name);
-  int fd = sh_open(name, O_WRONLY | O_CREAT | O_TRUNC, 0666);
+  int fd = ucw_open(name, O_WRONLY | O_CREAT | O_TRUNC, 0666);
   if (fd < 0)
     die("Cannot open %s: %m", name);
   return fd;
@@ -204,18 +222,19 @@ alloc_init(void)
     }
 
   fd_attrs = alloc_open_file(fn_attributes);
-  attr_buf = xmalloc(scan_batch * sizeof(struct card_attr));
+  attr_buf = big_alloc(scan_batch * sizeof(struct card_attr));
   fd_notes = alloc_open_file(fn_notes);
-  note_buf = xmalloc(scan_batch * sizeof(struct card_note));
+  note_buf = big_alloc(scan_batch * sizeof(struct card_note));
 
   if (index_name_defined(fn_notes_skel))
     {
       fd_skel = alloc_open_file(fn_notes_skel);
-      skel_buf = xmalloc(scan_batch * sizeof(struct card_note));
+      skel_buf = big_alloc(scan_batch * sizeof(struct card_note));
     }
 
-  urls = index_maybe_bopen(fn_urls, O_CREAT | O_TRUNC | O_WRONLY);
-  url_index = index_maybe_bopen(fn_url_index, O_CREAT | O_TRUNC | O_WRONLY);
+  urls = index_maybe_bopen(fn_urls, O_CREAT | O_TRUNC | O_WRONLY, 1);
+  url_index = index_maybe_bopen(fn_url_index, O_CREAT | O_TRUNC | O_WRONLY, 1);
+  skel_urls = index_maybe_bopen(fn_skel_urls, O_CREAT | O_TRUNC | O_WRONLY, 1);
 
   pthread_mutex_init(&alloc_mutex, NULL);
   pthread_cond_init(&sync_cond, NULL);
@@ -270,6 +289,8 @@ do_alloc_id(struct scan_context *c, uns is_skeleton)
       c->id = id_skel++;
       c->attr = NULL;
       c->note = &skel_buf[skel_buf_cnt++];
+      if (skel_urls)
+        bputsn(skel_urls, c->url);
     }
   else
     {
@@ -353,8 +374,9 @@ alloc_cleanup(void)
     close(fd_skel);
   bclose(urls);
   bclose(url_index);
+  bclose(skel_urls);
 
-  struct fastbuf *merges = index_maybe_bopen(fn_merges, O_CREAT | O_TRUNC | O_WRONLY);
+  struct fastbuf *merges = index_maybe_bopen(fn_merges, O_CREAT | O_TRUNC | O_WRONLY, 0);
   if (merges)
     {
       for (uns i=0; i<id_out; i++)
@@ -393,8 +415,8 @@ read_next(struct scan_context *c)
     return 0;
 
   PROGRESS_LOCKED(gb->progress_count, alloc_mutex,
-		  "scanner: %d objects -> %d cards, %d skels (%d%%)",
-		  gb->progress_count, id_out, id_skel - FIRST_ID_SKEL,
+		  "scanner: %d objects -> %d cards, %d skels (src %d, %d%%)",
+		  gb->progress_count, id_out, id_skel - FIRST_ID_SKEL, gb->cur_source,
 		  (int)((float)gb->progress_current/gb->progress_max*100));
   c->count_in++;
   c->obj = gb->o;
@@ -431,10 +453,29 @@ static struct filter_binding scan_bindings[] = {
   { "audio_bitrate",	OFFSETOF(struct scan_context, audio_bitrate) },
   { "audio_srate",	OFFSETOF(struct scan_context, audio_srate) },
   { "audio_channels",	OFFSETOF(struct scan_context, audio_channels) },
+#ifdef CONFIG_AREAS
+  /* Area ID */
+  { "area",		OFFSETOF(struct scan_context, area) },
+#endif
+  { "want_xform",	OFFSETOF(struct scan_context, want_xform) },
+  { "noindex",		OFFSETOF(struct scan_context, noindex) },
   { NULL,		0 }
 };
 
-static struct filter *scan_filter_program;
+static struct filter_binding xform_bindings[] = {
+  { "url",		OFFSETOF(struct scan_context, link) },
+  { "protocol",		OFFSETOF(struct scan_context, link_s.protocol) },
+  { "host",		OFFSETOF(struct scan_context, link_s.host) },
+  { "port",		OFFSETOF(struct scan_context, link_s.port) },
+  { "path",		OFFSETOF(struct scan_context, link_s.rest) },
+  { "username",		OFFSETOF(struct scan_context, link_s.user) },
+  { "password",		OFFSETOF(struct scan_context, link_s.pass) },
+  { "url_xform",	OFFSETOF(struct scan_context, url_xform) },
+  { "src_url",		OFFSETOF(struct scan_context, url) },
+  { NULL,		0 }
+};
+
+static struct filter *scan_filter_program, *xform_filter_program;
 
 static void
 scan_filter_init(void)
@@ -442,16 +483,17 @@ scan_filter_init(void)
   if (!indexer_filter_name || !indexer_filter_name[0])
     return;
   scan_filter_program = filter_load(indexer_filter_name, filter_builtin_vars, scan_bindings, NULL);
+  if (filter_links)
+    xform_filter_program = filter_load(indexer_filter_name, filter_builtin_vars, xform_bindings, NULL);
 }
 
 static void
 scan_filter_start(struct scan_context *c)
 {
   if (scan_filter_program)
-    {
-      struct filter *program = filter_clone(scan_filter_program);
-      c->filter_args = filter_intr_new(program);
-    }
+    c->filter_args = filter_intr_new(filter_clone(scan_filter_program));
+  if (xform_filter_program)
+    c->xform_args = filter_intr_new(filter_clone(xform_filter_program));
 }
 
 static byte *
@@ -536,6 +578,8 @@ scan_filter(struct scan_context *c)
   get_image_attrs(c);
   get_audio_attrs(c);
   c->area = 0;
+  c->want_xform = 0;
+  c->noindex = 0;
 
   struct filter_args *a = c->filter_args;
   if (!a)
@@ -548,6 +592,25 @@ scan_filter(struct scan_context *c)
   return filter_intr_run(a);
 }
 
+static int
+xform_filter(struct scan_context *c)
+{
+  byte buf1[MAX_URL_SIZE], buf2[MAX_URL_SIZE];
+  struct filter_args *a = c->xform_args;
+  a->attr = NULL;
+  a->raw = c;
+  a->pool = c->pool;
+  if (url_canon_split(c->link = c->url_xform, buf1, buf2, &c->link_s) || !filter_intr_run(c->xform_args))
+    return 0;
+  if (c->url_xform != c->link)
+    {
+      if (!c->url_xform || url_auto_canonicalize(c->url_xform, c->link_buf))
+	return 0;
+      c->url_xform = c->link_buf;
+    }
+  return 1;
+}
+
 /* Analysers */
 
 static void
@@ -724,30 +787,43 @@ gen_labels_by_id(struct scan_context *c)
       bput_attr_separator(b);
     }
 
+#ifdef CONFIG_SITES
+  byte site[URL_KEY_BUF_SIZE];
+  strcpy(site, c->site_name ?: (byte*)"");
+  site_name_compute(c->url, site, c->site_level, c->note->site_hash);
+  bputl(b, c->id);
+  bputc(b, LABEL_TYPE_BODY);
+  bput_attr_str(b, 't', site);
+  bput_attr_separator(b);
+#endif
+
   fbatomic_commit(b);
 }
 
 static void
 gen_checksums(struct scan_context *c)
 {
-  if (!c->checksums)
-    return;
+  if (c->noindex)
+    {
+      c->attr->flags |= CARD_FLAG_EMPTY;
+      return;
+    }
 
   struct csum csum = { .cardid = c->id };
   u16 buf[4096];
   uns cnt = 0;
   struct oattr *a;
-  struct MD5Context ct;
+  md5_context ct;
   uns accents = 0;
   uns chars = 0;
-  uns lastc = ' ';
+  uns lastcc = ' ';
 
-  MD5Init(&ct);
+  md5_init(&ct);
   for (byte *aname="XM"; *aname; aname++)
     for (a=obj_find_attr(c->obj, *aname); a; a=a->same)
       {
 	byte *z = a->val;
-	uns x, c, u;
+	uns x, cc, u;
 	do
 	  {
 	    GET_TAGGED_CHAR(z, x);
@@ -755,22 +831,23 @@ gen_checksums(struct scan_context *c)
 	      {
 		if (x >= 0x80010000)
 		  continue;
-		c = ' ';
+		cc = ' ';
 	      }
 	    else
-	      c = x;
-	    if (c != lastc || c != ' ')
+	      cc = x;
+	    if (cc != lastcc || cc != ' ')
 	      {
-		lastc = c;
-		u = Uunaccent(c);
-		if (c != u)
+		lastcc = cc;
+		u = Uunaccent(cc);
+		if (cc != u)
 		  accents++;
-		if (Ualnum(c))
+		if (Ualnum(cc))
 		  chars++;
 		buf[cnt++] = u;
 		if (cnt >= ARRAY_SIZE(buf))
 		  {
-		    MD5Update(&ct, (byte *) buf, cnt * sizeof(buf[0]));
+		    if (c->checksums)
+		      md5_update(&ct, (byte *) buf, cnt * sizeof(buf[0]));
 		    cnt = 0;
 		  }
 	      }
@@ -788,7 +865,7 @@ gen_checksums(struct scan_context *c)
 	{
 	  byte *z = a->val;
 	  uns len = str_len(z);
-	  MD5Update(&ct, z, len);
+	  md5_update(&ct, z, len);
 	  chars += len;
 	}
     }
@@ -797,7 +874,7 @@ gen_checksums(struct scan_context *c)
     {
       byte *z = a->val;
       uns len = str_len(z);
-      MD5Update(&ct, z, len + 1);
+      md5_update(&ct, z, len + 1);
       /* chars += len; */
     }
 
@@ -809,24 +886,32 @@ gen_checksums(struct scan_context *c)
         {
 	  note->flags |= CARD_NOTE_AUDIO;
 	  byte *h = obj_find_aval(a->son, 'h');
-	  if (h)
-	    MD5Update(&ct, h, strlen(h));
+	  if (!h)
+	    return;
+	  md5_update(&ct, h, strlen(h));
 	  u32 size = obj_find_anum(a->son, 's', 1);
-	  MD5Update(&ct, (void *)&size, 4);
+	  md5_update(&ct, (void *)&size, 4);
 	  chars += size;
 	}
     }
 
   if (accents > chars/128)
     c->attr->flags |= CARD_FLAG_ACCENTED;
-  if (!chars)
+  if (!chars && reject_empty)
     c->attr->flags |= CARD_FLAG_EMPTY;
   if (chars <= min_summed_size)
     return;
+  if (!c->checksums)
+    return;
 
-  MD5Update(&ct, (byte *) buf, cnt * sizeof(buf[0]));
+  md5_update(&ct, (byte *) buf, cnt * sizeof(buf[0]));
 
-  MD5Final(csum.md5, &ct);
+#ifdef CONFIG_AREAS
+  /* Avoid mixing documents from different areas */
+  md5_update(&ct, (byte *) &note->area, sizeof(note->area));
+#endif
+
+  memcpy(csum.md5, md5_final(&ct), MD5_SIZE);
   bwrite(c->checksums, &csum, sizeof(csum));
 }
 
@@ -836,6 +921,7 @@ gen_links(struct scan_context *c)
   if (!c->links)
     return;
 
+  uns want_xform = c->want_xform && c->xform_args;
   for (struct oattr *a=c->obj->attrs; a; a=a->next)
     if (attr_set_match(&link_attr_set, a))
       for (struct oattr *v=a; v; v=v->same)
@@ -844,13 +930,7 @@ gen_links(struct scan_context *c)
 	  if (a->attr == 'Y')
 	    c->note->flags |= CARD_NOTE_REDIRECT;
 
-	  struct link link;
 	  byte *p;
-	  link.src = (a->attr == 'Y') ? ETYPE_REDIRECT :
-	             (a->attr == 'F') ? ETYPE_FRAME :
-	             (a->attr == 'I') ? ETYPE_IMAGE :
-	             ETYPE_NORMAL;
-	  link.src |= c->id;
 	  if (p = strchr(v->val, ' '))
 	    {
 	      byte *d = p+1;
@@ -860,10 +940,20 @@ gen_links(struct scan_context *c)
 		continue;
 	      *p = 0;
 	    }
-	  url_fingerprint(v->val, &link.fp);
+	  c->url_xform = v->val;
+	  if (!want_xform || xform_filter(c))
+	    {
+	      struct link link;
+	      url_fingerprint(c->url_xform, &link.fp);
+	      link.src = (a->attr == 'Y') ? ETYPE_REDIRECT :
+		  (a->attr == 'F') ? ETYPE_FRAME :
+		  (a->attr == 'I') ? ETYPE_IMAGE :
+		  ETYPE_NORMAL;
+	      link.src |= c->id;
+	      bwrite(c->links, &link, sizeof(link));
+	    }
 	  if (p)
 	    *p = ' ';
-	  bwrite(c->links, &link, sizeof(link));
 	}
 }
 
@@ -923,6 +1013,10 @@ gen_note_basics(struct scan_context *c)
 {
   struct card_note *cn = c->note;
 
+#ifdef CONFIG_AREAS
+  cn->area = c->area;
+#endif
+
   byte *footprint = obj_find_aval(c->obj, 'O');
   if (footprint)
     {
@@ -938,7 +1032,16 @@ gen_attrs(struct scan_context *c)
   struct card_attr *ca = c->attr;
   struct card_note *cn = c->note;
 
-  ca->card = c->get_buck.oid;
+  ca->card = c->get_buck.index;
+#ifdef CONFIG_INDEXER_STORE_OID
+  cn->oid = c->get_buck.oid;
+#endif
+#ifdef CONFIG_SITES
+  ca->site_id = 0;
+#endif
+#ifdef CONFIG_AREAS
+  ca->area = c->area;
+#endif
   ca->weight = initial_weight(c);
   cn->weight_scanner = ca->weight;
   cn->card_bonus = CLAMP(c->card_bonus, -32767, 32767);
@@ -1002,7 +1105,7 @@ gen_ref_texts(struct scan_context *ctx)
 #define MAX_REF_DEPTH 3
 
   struct fastbuf *reftexts = ctx->reftexts;
-  if (!reftexts)
+  if (!reftexts || ctx->noindex >= 2)
     return;
 
   /* Map of references */
@@ -1149,26 +1252,58 @@ gen_ref_texts(struct scan_context *ctx)
 
 #endif
 
-static inline void images_start(struct scan_context *c UNUSED) {}
-static inline void images_stop(struct scan_context *c UNUSED) {}
-static inline void gen_images(struct scan_context *c UNUSED) {}
+#ifdef CONFIG_IMAGES_DUP
 
-#ifdef CONFIG_INDEXER_IMAGES_ONLY
-static uns
-image_filter(struct scan_context *c)
+static void
+images_start(struct scan_context *c)
 {
-  /* Accept only images, audio and objects with at least one link */
-  if (obj_find_attr(c->obj, 'G'))
-    return 1;
-  if (obj_find_attr(c->obj, 'f' + OBJ_ATTR_SON))
-    return 1;
-  for (struct oattr *a=c->obj->attrs; a; a=a->next)
-    if (attr_set_match(&ref_link_attr_set, a))
-      return 1;
-  return 0;
+  c->image_thumbnails = index_maybe_atomic_open(fn_image_thumbnails, scan_contexts[0].image_thumbnails, -(indexer_fb_size/8));
 }
+
+static void
+images_finish(struct scan_context *c)
+{
+  bclose(c->image_thumbnails);
+}
+
+static void
+gen_images(struct scan_context *c)
+{
+  struct fastbuf *fb = c->image_thumbnails;
+  if (!fb || (c->attr->flags & CARD_FLAG_EMPTY))
+    return;
+  struct image_obj_info ioi;
+  if (!get_image_obj_info(&ioi, c->obj))
+    return;
+  struct image_signature sig;
+  if (!get_image_obj_signature(&sig, c->obj))
+    {
+      log(L_WARN, "Image `%s' has no signature", obj_find_aval(c->obj, 'U'));
+      return;
+    }
+  if (!get_image_obj_thumb(&ioi, c->obj, c->pool))
+    {
+      log(L_WARN, "Image `%s' has no thumbnail", obj_find_aval(c->obj, 'U'));
+      return;
+    }
+  struct image_thumb hdr;
+  hdr.id = c->id;
+  hdr.cols = ioi.cols;
+  hdr.rows = ioi.rows;
+  hdr.thumb_cols = ioi.thumb_cols;
+  hdr.thumb_rows = ioi.thumb_rows;
+  hdr.thumb_size = ioi.thumb_size;
+  hdr.thumb_format = ioi.thumb_format;
+  hdr.vector = sig.vec;
+  bwrite(fb, &hdr, sizeof(hdr));
+  bwrite(fb, ioi.thumb_data, ioi.thumb_size);
+  fbatomic_commit(fb);
+}
+
 #else
-static inline uns image_filter(struct scan_context *ctx UNUSED) { return 1; }
+static inline void images_start(struct scan_context *c UNUSED) {}
+static inline void images_finish(struct scan_context *c UNUSED) {}
+static inline void gen_images(struct scan_context *c UNUSED) {}
 #endif
 
 /* Contexts */
@@ -1228,6 +1363,7 @@ scan_finish(struct scan_context *c)
   bclose(c->checksums);
   bclose(c->labels_by_id);
   bclose(c->fingerprints);
+  images_finish(c);
 
   read_finish(c);
   scan_analyse_finish(c);
@@ -1244,7 +1380,8 @@ scan_loop(struct scan_context *c)
       c->url = obj_find_aval(o, 'U');
       if (!c->url)
 	die("Object %x has no URL, probably broken bucket file", c->get_buck.oid);
-      if (obj_find_aval(o, 'r'))			/* Skip robots.txt buckets */
+      byte *ct = obj_find_aval(o, 'T');
+      if (ct && !strcmp(ct, "x-sherlock/robots"))	/* Skip robots.txt buckets */
 	{
 	  c->count_bots++;
 	  continue;
@@ -1252,8 +1389,6 @@ scan_loop(struct scan_context *c)
       c->attr = NULL;					/* For sure */
       c->note = NULL;
 
-      if (!image_filter(c))				/* Skip useless objects for CONFIG_INDEXER_IMAGES_ONLY */
-	continue;
       scan_analyse(c);
       if (scan_filter(c))
 	{
@@ -1370,7 +1505,7 @@ main(int argc, char **argv)
     {
       pthread_attr_t attr;
       if (pthread_attr_init(&attr) < 0 ||
-	  pthread_attr_setstacksize(&attr, indexer_thread_stack_size ? : default_thread_stack_size) < 0)
+	  pthread_attr_setstacksize(&attr, indexer_thread_stack_size ? : ucwlib_thread_stack_size) < 0)
 	ASSERT(0);
       ready_sem = sem_alloc();
       go_sem = sem_alloc();
