@@ -5,8 +5,8 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
 #include "indexer/indexer.h"
 #include "indexer/params.h"
 
