@@ -7,7 +7,7 @@
 #ifndef _LANG_LANG_H
 #define _LANG_LANG_H
 
-#include "lib/clists.h"
+#include "ucw/clists.h"
 
 /*
  *  Each language gets its own code (0..31). Codes are assigned in the
@@ -63,9 +63,9 @@ byte *lang_cf_parse_set(uns c, byte **pars, uns *ptr);
 struct stemmer {
   cnode n;
   u32 lang_mask;			/* Languages this stemmer applies to */
-  byte *name;				/* Name of the stemmer */
+  const char *name;			/* Name of the stemmer */
   int id;				/* ... and its internal ID */
-  byte *params;				/* Additional stemmer-specific parameters */
+  char *params;				/* Additional stemmer-specific parameters */
   void *priv;				/* Data private to the stemmer */
 };
 
@@ -90,7 +90,7 @@ struct mempool;
 clist *lang_stem(struct stemmer *st, struct word_node *w, struct mempool *mp);
 void lang_init_stemmers(void);
 
-extern byte *word_form_names[];
+extern char *word_form_names[];
 struct word_node *word_list_find(clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented);
 struct word_node *word_list_add(struct mempool *mp, clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented);
 struct word_node *word_list_add_unique(struct mempool *mp, clist *l, byte *w, uns word_form, uns stem_form, uns variant, uns unaccented);
@@ -102,7 +102,7 @@ struct word_node *word_list_add_unique(struct mempool *mp, clist *l, byte *w, un
 struct syndict {
   cnode n;
   u32 lang_mask;
-  byte *name;
+  char *name;
   struct fastbuf *fb;
 };
 
