@@ -34,16 +34,17 @@
 #undef LOCAL_DEBUG
 #undef TREE_DEBUG
 
-#include "lib/lib.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/mempool.h"
-#include "lib/clists.h"
-#include "lib/chartype.h"
-#include "lib/hashfunc.h"
-#include "lib/md5.h"
+#include "ucw/lib.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/mempool.h"
+#include "ucw/clists.h"
+#include "ucw/chartype.h"
+#include "ucw/hashfunc.h"
+#include "ucw/md5.h"
+#include "ucw/unaligned.h"
 #include "charset/charconv.h"
 #include "charset/unicat.h"
 #include "lang/stem-dict.h"
@@ -56,7 +57,7 @@
 #include <alloca.h>
 
 #ifdef TREE_DEBUG
-#define TDBG(x...) log(L_DEBUG, x)
+#define TDBG(x...) msg(L_DEBUG, x)
 #else
 #define TDBG(x...) do { } while(0)
 #endif
@@ -98,7 +99,7 @@ lemma_init_data(struct lemma *v)
 #define HASH_GIVE_INIT_DATA
 #define HASH_AUTO_POOL 65536
 #define HASH_WANT_CLEANUP
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static uns
 lemma_identify_variant(byte *lemma)
@@ -134,7 +135,7 @@ suffix_init_data(struct suffix *s)
 #define HASH_WANT_LOOKUP
 #define HASH_GIVE_INIT_DATA
 #define HASH_AUTO_POOL 65536
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static struct suffix **suffix_by_old_id, **suffixes;
 
@@ -157,7 +158,7 @@ suff_lt(struct suffix *x, struct suffix *y)
 #define ASORT_KEY_TYPE struct suffix *
 #define ASORT_ELT(i) suffixes[i]
 #define ASORT_LT(x,y) suff_lt(x,y)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static void
 suffix_gather(void)
@@ -182,7 +183,7 @@ suffix_gather(void)
       for (uns i=0; i<num_suffixes; i++)
 	bprintf(out, "%d\t%d\t%s\n", suffixes[i]->id, suffixes[i]->use_cnt, suffixes[i]->w);
       bclose(out);
-      log(L_INFO, "Dumped suffix table to %s", out_suffixes);
+      msg(L_INFO, "Dumped suffix table to %s", out_suffixes);
     }
 }
 
@@ -235,7 +236,7 @@ pattern_init_key(struct pattern *p, const suffix_id_t *suff, uns flags)
 #define HASH_GIVE_EXTRA_SIZE
 #define HASH_GIVE_INIT_KEY
 #define HASH_AUTO_POOL 65536
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static struct pattern **patterns;
 
@@ -265,7 +266,7 @@ pattern_gather(void)
 	  bputc(out, '\n');
 	}
       bclose(out);
-      log(L_INFO, "Dumped pattern table to %s", out_patterns);
+      msg(L_INFO, "Dumped pattern table to %s", out_patterns);
     }
 }
 
@@ -298,7 +299,7 @@ stem_init_data(struct stem *s)
 #define HASH_WANT_LOOKUP
 #define HASH_GIVE_INIT_DATA
 #define HASH_AUTO_POOL 65536
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static struct stem *
 stem_add(byte *s, struct pattern *patt, struct stem *main)
@@ -350,7 +351,7 @@ stem_lt(struct stem *x, struct stem *y)
 #define ASORT_KEY_TYPE struct stem *
 #define ASORT_ELT(i) stems[i]
 #define ASORT_LT(x,y) stem_lt(x,y)
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static void
 stem_gather(void)
@@ -400,7 +401,7 @@ stem_gather(void)
 	    }
 	}
       bclose(out);
-      log(L_INFO, "Dumped stem table to %s", out_stems);
+      msg(L_INFO, "Dumped stem table to %s", out_stems);
     }
 }
 
@@ -427,8 +428,8 @@ static void
 wl_add_to_list(clist *wl, byte *wo, uns is_lemma)
 {
   struct w *w = mp_alloc(pool, sizeof(*w));
-  if (strlen(wo) > MAX_WORD_LEN)
-    die("Word <%s> is too long, see MAX_WORD_LEN", wo);
+  if (strlen(wo) > MAX_WORD_BYTES)
+    die("Word <%s> is too long, see MAX_WORD_BYTES", wo);
   w->w = mp_strdup(pool, wo);
   w->is_lemma = is_lemma;
   clist_add_tail(wl, &w->n);
@@ -505,7 +506,7 @@ dump(clist *wl)
 {
   struct w *w;
   CLIST_WALK(w, *wl)
-    log(L_DEBUG, "\t%s (%d)", w->w, w->is_lemma);
+    msg(L_DEBUG, "\t%s (%d)", w->w, w->is_lemma);
 }
 #endif
 
@@ -643,7 +644,7 @@ wl_unacc_list(clist *wl)
   struct w *w;
   CLIST_WALK(w, *wl)
     {
-      byte uw[MAX_WORD_LEN+1];
+      byte uw[MAX_WORD_BYTES+1];
       if (unacc_string(uw, w->w) && !wl_member(wl, uw) && !wl_member(ul, uw))
 	{
 	  DBG("Unaccented %s -> %s", w->w, uw);
@@ -664,49 +665,58 @@ struct prep_key {
   byte hash[MD5_SIZE];
 };
 
+#define SORT_PREFIX(x) prep_##x
+#define SORT_KEY_REGULAR struct prep_key
+#define SORT_DATA_SIZE(k) ((k).data_len)
+#define SORT_UNIFY
+#define SORT_INPUT_FB
+#define SORT_OUTPUT_FB
+#define SORT_HASH_BITS 32
+
 static inline int
 prep_compare(struct prep_key *x, struct prep_key *y)
 {
   return memcmp(x->hash, y->hash, MD5_SIZE);
 }
 
-static inline int
-prep_fetch_key(struct fastbuf *f, struct prep_key *k)
+static inline uns
+prep_hash(struct prep_key *x)
 {
-  return breadb(f, k, sizeof(*k));
-}
-
-static void
-prep_copy_data(struct fastbuf *src, struct fastbuf *dest, struct prep_key *k)
-{
-  bwrite(dest, k, sizeof(*k));
-  bbcopy(src, dest, k->data_len);
+  return get_u32_be(x->hash);
 }
 
 static inline void
-prep_merge_data(struct fastbuf *src1, struct fastbuf *src2, struct fastbuf *dest, struct prep_key *k1, struct prep_key *k2)
+prep_write_merged(struct fastbuf *f, struct prep_key **k, void **d, uns n, void *buf UNUSED)
 {
-  k1->prefix_mask |= k2->prefix_mask;
-  bwrite(dest, k1, sizeof(*k1));
-  bbcopy(src1, dest, k1->data_len);
-  ASSERT(k2->data_len == k1->data_len);
-  bskip(src2, k2->data_len);
+  for (uns i = 1; i < n; i++)
+    {
+      k[0]->prefix_mask |= k[i]->prefix_mask;
+      ASSERT(k[0]->data_len == k[i]->data_len);
+    }
+  bwrite(f, k[0], sizeof(**k));
+  bwrite(f, d[0], k[0]->data_len);
   prep_merge_cnt++;
 }
 
-#define SORT_KEY struct prep_key
-#define SORT_PREFIX(x) prep_##x
-#define SORT_UNIFY
-#define SORT_INPUT_FB
-#define SORT_OUTPUT_FB
-#include "lib/sorter.h"
+static inline void
+prep_copy_merged(struct prep_key **k, struct fastbuf **d, uns n, struct fastbuf *f)
+{
+  for (uns i = 1; i < n; i++)
+    {
+      k[0]->prefix_mask |= k[i]->prefix_mask;
+      ASSERT(k[0]->data_len == k[i]->data_len);
+      bskip(d[i], k[0]->data_len);
+    }
+  bwrite(f, k[0], sizeof(**k));
+  bbcopy(d[0], f, k[0]->data_len);
+}
+
+#include "ucw/sorter/sorter.h"
 
 #define ASORT_PREFIX(x) prew_##x
 #define ASORT_KEY_TYPE struct w *
-#define ASORT_ELT(i) words[i]
 #define ASORT_LT(x,y) strcmp(x->w, y->w) < 0
-#define ASORT_EXTRA_ARGS , struct w **words
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static void
 prep_flush(struct fastbuf *f)
@@ -723,7 +733,7 @@ prep_flush(struct fastbuf *f)
   CLIST_WALK(w, wl)
     words[cnt++] = w;
 
-  prew_sort(cnt, words);
+  prew_sort(words, cnt);
   clist_init(&wl);
   for (uns i=0; i<cnt; i++)
     if (!i || strcmp(words[i-1]->w, words[i]->w))
@@ -740,22 +750,22 @@ prep_flush(struct fastbuf *f)
     px_cnt++;
   key.prefix_mask = 1 << prefixes;
 
-  struct MD5Context mc;
-  MD5Init(&mc);
+  md5_context mc;
+  md5_init(&mc);
   uns data_len = 1;
   CLIST_WALK(w, wl)
     {
       uns len = strlen(w->w) + 1;
-      MD5Update(&mc, w->w, len);
-      MD5Update(&mc, &w->is_lemma, 1);
+      md5_update(&mc, w->w, len);
+      md5_update(&mc, &w->is_lemma, 1);
       data_len += len + 1;
     }
-  MD5Final(key.hash, &mc);
+  memcpy(key.hash, md5_final(&mc), MD5_SIZE);
   ASSERT(data_len < 65536);
   key.data_len = data_len;
 
   bwrite(f, &key, sizeof(key));
-  sh_off_t pos0 = btell(f);
+  ucw_off_t pos0 = btell(f);
   CLIST_WALK(w, wl)
     {
       bputc(f, w->is_lemma + 1);
@@ -777,7 +787,7 @@ preprocess(struct fastbuf *in)
   struct fastbuf *tmp = bopen_tmp(65536);
   byte line[256], last[256];
   wl_init();
-  log(L_INFO, "Scanning input");
+  msg(L_INFO, "Scanning input");
   while (bgets(in, line, sizeof(line)))
     {
       int c = 0;
@@ -802,18 +812,15 @@ preprocess(struct fastbuf *in)
   prep_flush(tmp);
   bclose(in);
 
-  log(L_INFO, "Preprocessing %d classes", prep_in_cnt);
+  msg(L_INFO, "Preprocessing %d classes", prep_in_cnt);
   brewind(tmp);
-  tmp = prep_sort(tmp);
+  tmp = prep_sort(tmp, NULL);
 
-  log(L_INFO, "Merged %d classes differing only by prefixes", prep_merge_cnt);
+  msg(L_INFO, "Merged %d classes differing only by prefixes", prep_merge_cnt);
   if (prep_out)
     {
-      bflush(tmp);
-      bconfig(tmp, BCONFIG_IS_TEMP_FILE, 0);
-      if (rename(tmp->name, prep_out) < 0)
-	die("Cannot rename %s to %s: %m", tmp->name, prep_out);
-      log(L_INFO, "Stored preprocessed data to %s", prep_out);
+      bfix_tmp_file(tmp, prep_out);
+      msg(L_INFO, "Stored preprocessed data to %s", prep_out);
       exit(0);
     }
   return tmp;
@@ -826,7 +833,7 @@ chew(struct fastbuf *in)
   while (breadb(in, &k, sizeof(k)))
     {
       wl_init();
-      byte buf[MAX_WORD_LEN+1];
+      byte buf[MAX_WORD_BYTES+1];
       int lemma_var = -1;
       while (bgets(in, buf, sizeof(buf)) && buf[0])
 	{
@@ -874,7 +881,7 @@ gen_suffixes(void)
     for (int j=suffixes[i]->len-1; j>=0; j--)
       bputc(dict_out, suffixes[i]->w[j]);
   ASSERT((uns)btell(dict_out) == spos);
-  log(L_INFO, "Suffix table: %d bytes pointers + %d bytes data", suffix_dir_len, spos-stab_start);
+  msg(L_INFO, "Suffix table: %d bytes pointers + %d bytes data", suffix_dir_len, spos-stab_start);
 }
 
 static void
@@ -899,7 +906,7 @@ gen_patterns(void)
       bwrite(dict_out, p->suff+1, p->suff[0]*sizeof(p->suff[0]));
     }
   ASSERT((uns)btell(dict_out) == ppos);
-  log(L_INFO, "Pattern table: %d bytes pointers + %d bytes data", pattern_dir_len, ppos-ptab_start);
+  msg(L_INFO, "Pattern table: %d bytes pointers + %d bytes data", pattern_dir_len, ppos-ptab_start);
 }
 
 static uns max_vertices, num_vertices, num_son_ids, num_vertices_pass1, num_son_ids_pass1;
@@ -1137,15 +1144,14 @@ prepare_tree(void)
       vt_stats[vertex_types[i]]++;
     }
   vertex_pos[num_vertices] = pos;
-  log(L_INFO, "Planned tree with %d vertices, %d bytes large", num_vertices, pos);
-  log(L_INFO, "Vertex types: %d table, %d skip, %d small + %d large branching",
+  msg(L_INFO, "Planned tree with %d vertices, %d bytes large", num_vertices, pos);
+  msg(L_INFO, "Vertex types: %d table, %d skip, %d small + %d large branching",
       vt_stats[VT_TABLE], vt_stats[VT_SKIP], vt_stats[VT_BRANCH_SMALL], vt_stats[VT_BRANCH_LARGE]);
 }
 
 static void
 calc_stem_pos_range(struct vertex *v, uns start, uns cnt, uns pos)
 {
-  ASSERT(pos);
   while (cnt--)
     {
       struct stem *st = stems[start];
@@ -1188,7 +1194,7 @@ calc_stem_pos(void)
   num_vertices = 0;
   calc_stem_pos_subtree(&root);
   ASSERT(num_vertices == num_vertices_pass1 && num_son_ids == num_son_ids_pass1);
-  log(L_INFO, "Calculated stem positions");
+  msg(L_INFO, "Calculated stem positions");
 
   if (out_stem_pos)
     {
@@ -1200,7 +1206,7 @@ calc_stem_pos(void)
 	    bprintf(f, "%x\t%s\n", ss->tree_pos, (ss==st ? st->w : (byte*)"-\"-"));
 	}
       bclose(f);
-      log(L_INFO, "Dumped stem positions to %s", out_stem_pos);
+      msg(L_INFO, "Dumped stem positions to %s", out_stem_pos);
     }
 }
 
@@ -1224,7 +1230,6 @@ gen_leaf(struct vertex *v, uns id)
 	{
 	  ASSERT(!st->main->main);
 	  uns pos = st->main->tree_pos;
-	  ASSERT(pos);
 	  bputc(dict_out, 0xc0 | ((pos >> 24) & 0xff));
 	  bputc(dict_out, (pos >> 16) & 0xff);
 	  bputc(dict_out, (pos >> 8) & 0xff);
@@ -1240,7 +1245,6 @@ gen_leaf(struct vertex *v, uns id)
 	  for (struct stem *sx=st->sub; sx; sx=sx->sub)
 	    {
 	      uns pos = sx->tree_pos;
-	      ASSERT(pos);
 	      bputc(dict_out, 0x80 | ((pos >> 24) & 0xff));
 	      bputc(dict_out, (pos >> 16) & 0xff);
 	      bputc(dict_out, (pos >> 8) & 0xff);
@@ -1318,7 +1322,7 @@ gen_tree(void)
   gen_subtree(&root);
   ASSERT(num_vertices == num_vertices_pass1 && num_son_ids == num_son_ids_pass1);
   ASSERT((uns)btell(dict_out) == dict_hdr.stem_tree_start + dict_hdr.stem_tree_length);
-  log(L_INFO, "Written tree as promised");
+  msg(L_INFO, "Written tree as promised");
 }
 
 static void
@@ -1346,7 +1350,7 @@ gen_dict(void)
   dict_hdr.magic = STEM_DICT_MAGIC;
   bwrite(dict_out, &dict_hdr, sizeof(dict_hdr));
   bclose(dict_out);
-  log(L_INFO, "Written dictionary to %s", out_dict);
+  msg(L_INFO, "Written dictionary to %s", out_dict);
 }
 
 /*** Main ***/
@@ -1438,24 +1442,25 @@ int main(int argc, char **argv)
 	usage();
       }
 
-  log(L_INFO, "Initializing");
+  msg(L_INFO, "Initializing");
   pool = mp_new(65536);
   suffix_init();
   suffix_lookup("");
   pattern_init();
   stem_init();
-  char_table_init();
+  if (strip_accents || fold_case)
+    char_table_init();
   lemma_init();
 
   struct fastbuf *tmp = preprocess(bfdopen_shared(0, 65536));
-  log(L_INFO, "Chewing");
+  msg(L_INFO, "Chewing");
   chew(tmp);
-  log(L_INFO, "Generated %d stems, %d substems, %d suffixes and %d patterns", num_stems, num_substems, num_suffixes, num_patterns);
-  log(L_INFO, "Encountered %d exceptions, %d splits and %d strips", num_exc, num_splits, num_strips);
-  log(L_INFO, "Found %d lemmata + %d variants", num_lemmata, num_variants);
+  msg(L_INFO, "Generated %d stems, %d substems, %d suffixes and %d patterns", num_stems, num_substems, num_suffixes, num_patterns);
+  msg(L_INFO, "Encountered %d exceptions, %d splits and %d strips", num_exc, num_splits, num_strips);
+  msg(L_INFO, "Found %d lemmata + %d variants", num_lemmata, num_variants);
   lemma_cleanup();
 
-  log(L_INFO, "Postprocessing");
+  msg(L_INFO, "Postprocessing");
   suffix_gather();
   pattern_gather();
   stem_gather();
