@@ -11,8 +11,8 @@
  *  This stemmer supports only conversion of a word to its stem.
  */
 
-#include "lib/lib.h"
-#include "lib/mempool.h"
+#include "ucw/lib.h"
+#include "ucw/mempool.h"
 #include "lang/lang.h"
 #include "lang/stemmer.h"
 
