@@ -9,10 +9,10 @@
 
 #undef LOCAL_DEBUG
 
-#include "lib/lib.h"
-#include "lib/mempool.h"
-#include "lib/fastbuf.h"
-#include "lib/unicode.h"
+#include "ucw/lib.h"
+#include "ucw/mempool.h"
+#include "ucw/fastbuf.h"
+#include "ucw/unicode.h"
 #include "lang/lang.h"
 #include "lang/stemmer.h"
 #include "lang/stem-table.h"
@@ -118,7 +118,7 @@ stem_table_lookup_vertex(struct table_stemmer *ts, byte *w)
 static struct stem_table_vertex *
 stem_table_lookup_word_vertex(struct table_stemmer *ts, byte *src)
 {
-  byte w[3*MAX_WORD_LEN+1];
+  byte w[MAX_WORD_BYTES+1];
   if (!stem_table_conv_in(ts, src, w))
     {
       DBG("<%s>: not a valid word", src);
@@ -159,7 +159,7 @@ stem_table_get_vertex_name(struct table_stemmer *ts, struct stem_table_vertex *v
 static int
 stem_table_conv_vertex_out(struct table_stemmer *ts, struct stem_table_vertex *v, byte *y)
 {
-  byte x[3*MAX_WORD_LEN+1];
+  byte x[MAX_WORD_BYTES+1];
   byte *xx = stem_table_get_vertex_name(ts, v, x+sizeof(x));
   if (ts->charset != CONV_CHARSET_UTF8)
     {
@@ -167,11 +167,11 @@ stem_table_conv_vertex_out(struct table_stemmer *ts, struct stem_table_vertex *v
       cc->source = xx;
       cc->source_end = xx + strlen(xx);
       cc->dest = cc->dest_start = y;
-      cc->dest_end = y + MAX_WORD_LEN;
+      cc->dest_end = y + MAX_WORD_BYTES;
       int status = conv_run(cc);
       if (!(status & CONV_SOURCE_END))
 	{
-	  log(L_ERROR, "stem-table output too long");
+	  msg(L_ERROR, "stem-table output too long");
 	  return 0;
 	}
       *cc->dest = 0;
@@ -200,7 +200,7 @@ stem_table(struct stemmer *st, struct word_node *req, struct mempool *mp)
       struct stem_table_lemma *lm = st_go(ts, lemmata[i]);
       DBG("Found lemma %d with %d variants at %x:", i, lm->num_variants, st_pos(ts, lm));
       uns cnt = (req->stem_form == WORD_FORM_OTHER) ? lm->num_variants : 1;
-      byte buf[MAX_WORD_LEN+1];
+      byte buf[MAX_WORD_BYTES+1];
       if (!stem_table_conv_vertex_out(ts, st_go(ts, lm->variants[0]), buf))
 	continue;
       uns form = (strcmp(buf, req->w) ? WORD_FORM_OTHER : WORD_FORM_LEMMA);
