@@ -1,7 +1,7 @@
 /*
  *	Sherlock Search Engine -- Card Files
  *
- *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
+ *	(c) 1997--2007 Martin Mares <mj@ucw.cz>
  *	(c) 2001--2006 Robert Spalek <robert@ucw.cz>
  *	(c) 2006 Pavel Charvat <pchar@ucw.cz>
  */
@@ -9,13 +9,13 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/hashfunc.h"
-#include "lib/heap.h"
-#include "lib/url.h"
-#include "lib/unicode.h"
-#include "lib/workqueue.h"
-#include "lib/lfs.h"
+#include "ucw/mempool.h"
+#include "ucw/hashfunc.h"
+#include "ucw/heap.h"
+#include "ucw/url.h"
+#include "ucw/unicode.h"
+#include "ucw/workqueue.h"
+#include "ucw/lfs.h"
 #include "sherlock/index.h"
 #include "sherlock/tagged-text.h"
 #include "sherlock/lizard-fb.h"
@@ -40,10 +40,10 @@ static inline void kmp_free(struct kmp_struct *kmp UNUSED, void *ptr UNUSED) {}
 #define KMP_GIVE_ALLOC
 #define KMP_VARS struct mempool *pool;
 #define KMPS_GET_CHAR(kmp,src,s) ((src == s->u.src_end) ? 0 : ((s->c = Clocase(*(src)++)), 1))
-#define KMPS_VARS uns best_len; byte *best_pos, *src_end;
+#define KMPS_VARS uns best_len; char *best_pos, *src_end;
 #define KMPS_INIT(kmp,src,s) s->u.best_len = 0
 #define KMPS_FOUND_CHAIN(kmp,src,s) do{ if (s->out->len > s->u.best_len) { s->u.best_len = s->out->len; s->u.best_pos = src; } }while(0)
-#include "lib/kmp.h"
+#include "ucw/kmp.h"
 
 static inline void *
 kmp_alloc(struct kmp_struct *kmp, uns size)
@@ -66,12 +66,21 @@ show_card_attr(struct query *q, struct database *db, oid_t oid, struct card_attr
   send_reply("B%s", db->name);
   send_reply("O%08x", oid);
   send_reply("w%d", ca->weight);
+#ifdef CONFIG_SITES
+  u64 hash = site_find_hash(&db->sites, ca->site_id);
+  send_reply("T%0*llx", 2*SITE_HASH_SIZE, (long long) hash);
+  if (q->debug & DEBUG_CARD_INFO)
+    send_reply(".Tsite_id=%x", ca->site_id);
+#endif
 #ifdef CONFIG_MERGING_HASHES
   uns mh = 0;
   for (uns i=0; i<SHERLOCK_MERGING_HASH_SIZE; i++)
     mh = (mh << 8) | ca->merging_hash[i];
   send_reply("g%0*x", 2*SHERLOCK_MERGING_HASH_SIZE, mh);
 #endif
+#ifdef CONFIG_AREAS
+  send_reply("n%d", ca->area);
+#endif
 #ifdef CONFIG_LANG
   if (CA_GET_FILE_LANG(ca))
     send_reply("l%s", lang_code_to_name(CA_GET_FILE_LANG(ca)));
@@ -101,6 +110,10 @@ show_card_note(struct query *q, struct result_note *note)
   if (!note)
     return;
   send_reply("Q%d", note->q);
+#ifdef CONFIG_SITES
+  if (note->site_compressed)
+    send_reply("m%d", note->site_compressed);
+#endif
   if (q->debug & DEBUG_CARD_INFO)
     send_reply(".K%08x", note->sec_sort_key);
 }
@@ -114,7 +127,7 @@ static struct ft_word *
 ft_find_word_by_pos(uns pos)
 {
 #define ORIG_LT_POS(ary,i,x) ft.words[i].pos < (x)
-#include "lib/binsearch.h"
+#include "ucw/binsearch.h"
 	uns w = BIN_SEARCH_FIRST_GE_CMP(whatever,ft.num_words,pos,ORIG_LT_POS);
 	if (w < ft.num_words)
 		return ft.words + w;
@@ -128,7 +141,7 @@ ft_find_word_by_orig(byte *orig, struct ft_word *last_word)
 	if (last_word[1].orig == orig)		// avoid a log-factor during linear reading
 		return last_word+1;
 #define ORIG_LT_ORIG(ary,i,x) ft.words[i].orig < (x)
-#include "lib/binsearch.h"
+#include "ucw/binsearch.h"
 	uns w = BIN_SEARCH_FIRST_GE_CMP(whatever,ft.num_words,orig,ORIG_LT_ORIG);
 	if (w < ft.num_words)
 		return ft.words + w;
@@ -167,7 +180,7 @@ kmp_create(struct query *q)
 	if (!highlight_substring)
 		return NULL;
 
-	byte buf[MAX_WORD_LEN+1];
+	byte buf[MAX_WORD_BYTES+1];
 	struct variant *v;
 	uns words_len = 0;
 	CLIST_FOR_EACH(struct database *, db, databases)
@@ -209,9 +222,9 @@ kmp_create(struct query *q)
 /***** Dumper of a block of text *****/
 
 static byte tm_letter[2] = "XM";
-byte *wt_names[9] = { WORD_TYPE_USER_NAMES, NULL };
-byte *mt_names[17] = { META_TYPE_USER_NAMES, NULL };
-byte *st_names[9] = { STRING_TYPE_USER_NAMES, NULL };
+const char * const wt_names[9] = { WORD_TYPE_USER_NAMES, NULL };
+const char * const mt_names[17] = { META_TYPE_USER_NAMES, NULL };
+const char * const st_names[9] = { STRING_TYPE_USER_NAMES, NULL };
 
 struct context_interval
 {
@@ -272,7 +285,7 @@ html_escape(uns c)
 static void
 dump_text_interval(byte *label, struct context_interval *range)
 {
-	byte **type_names = (ft.input_type == FT_TEXT ? wt_names : mt_names);
+	const char * const *type_names = (ft.input_type == FT_TEXT ? wt_names : mt_names);
 	byte *text;
 	uns line_len;
 	uns space;
@@ -291,7 +304,7 @@ dump_text_interval(byte *label, struct context_interval *range)
 	struct ft_word *last_word = ft.words-1;
 	while (text < range->last_char)
 	{
-		uns c = 0;
+		uns c = 0, rep_c = 0, rep_max = 0, rep_n = 0;
 		int empty = 1;
 		byte *bow = text, *eow = text;
 		while (text < range->last_char)
@@ -302,6 +315,10 @@ dump_text_interval(byte *label, struct context_interval *range)
 			  && alpha_class[c] != AC_DIGIT
 			  && alpha_class[c] != AC_LIGATURE))	// assuming that the expansion of the ligature consists of only alphanumerical characters
 				break;
+			if (c == rep_c)
+				if (++rep_n > rep_max)
+					rep_max = rep_n;
+			rep_c = c;
 			empty = 0;
 			eow = text;
 		}
@@ -318,18 +335,21 @@ dump_text_interval(byte *label, struct context_interval *range)
 				else
 					sign = 1;
 			}
-			if (sign == 2)
-				DUMP_CONST(space, "<best>");
-			else if (sign == 1)
-				DUMP_CONST(space, "<found>");
-			if (sign)
+			if (rep_max < filter_repeated_alpha || sign)
+			{
+				if (sign == 2)
+					DUMP_CONST(space, "<best>");
+				else if (sign == 1)
+					DUMP_CONST(space, "<found>");
+				if (sign)
+					space = 0;
+				dump_word(&line_len, bow, eow - bow, space);
+				if (sign == 2)
+					DUMP_CONST(0, "</best>");
+				else if (sign == 1)
+					DUMP_CONST(0, "</found>");
 				space = 0;
-			dump_word(&line_len, bow, eow - bow, space);
-			if (sign == 2)
-				DUMP_CONST(0, "</best>");
-			else if (sign == 1)
-				DUMP_CONST(0, "</found>");
-			space = 0;
+			}
 		}
 		if (c >= 0x80000000)
 		{
@@ -354,17 +374,36 @@ dump_text_interval(byte *label, struct context_interval *range)
 		}
 		else if (eow < range->last_char)			/* slash, ... */
 		{
-			byte *printing = html_escape(c);
-			if (printing)
-				dump_word(&line_len, printing, strlen(printing), space);
+			byte *t = text;
+			uns cnt = 0, next;
+			do
+			{
+				cnt++;
+				text = t;
+				GET_TAGGED_CHAR(t, next);
+			}
+			while (t < range->last_char && next == c);
+			if (cnt >= filter_repeated_nonalpha)
+				space = 1;
 			else
 			{
-				byte b[7];
-				printing = b;
-				PUT_UTF8(printing, c);
-				dump_word(&line_len, b, printing-b, space);
+				byte *printing = html_escape(c), buf[7];
+				uns len;
+				if (printing)
+					len = strlen(printing);
+				else
+				{
+					printing = buf;
+					printing = utf8_put(printing, c);
+					len = printing - buf;
+					printing = buf;
+				}
+				while (cnt--)
+				{
+					dump_word(&line_len, printing, len, space);
+					space = 0;
+				}
 			}
-			space = 0;
 		}
 	}
 	DUMP_MASK(1, "</%s>", type_names[type]);
@@ -419,19 +458,37 @@ printable_characters(byte *pos, byte *pos_end)
 	return len;
 }
 
+static byte *
+skip_spaces(byte *pos, byte *pos_end)
+{
+	while (pos < pos_end)
+	{
+		char *pos0 = pos;
+		uns c;
+		GET_TAGGED_CHAR(pos, c);
+		if (c < 0x80000000 && c != ' ')
+			return pos0;
+	}
+	return pos;
+}
+
 static uns
 contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *ints, uns maxints, int available)
 {
-	if (available <= 0)
-		return 0;
 	ASSERT(pos && pos_end && pos <= pos_end);
 	ASSERT(pos >= ft.text && pos_end <= ft.text_end);
-	TRACE("contint_add() entered with pos %d+%d and context %d",
-		(uns)(pos - ft.text), (uns)(pos_end - pos), available);
+	TRACE("  contint_add: pos %d+%d, available %d, maxlen %d",
+		(uns)(pos - ft.text), (uns)(pos_end - pos), available, (uns)(ft.text_end - ft.text));
+#define INT(in) (int)((in)->first_char - ft.text), (int)((in)->last_char - ft.text), (in)->context_len
+	if (available < (int)(pos_end - pos))
+	{
+		TRACE("    No room for that");
+		return 0;
+	}
 
 	/* Decide whether we hit an existing interval and compute the bounds
 	 * for the stretching operation.  The interval array is sorted and the
-	 * interval are pairwise disjunct and non-empty.  */
+	 * interval are pairwise disjoint and non-empty.  */
 	struct context_interval tmp_int, *I = NULL;
 	byte *softl = ft.text;
 	byte *softr = ft.text_end;
@@ -442,7 +499,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 		|| pos < ints[i].first_char && pos_end > ints[i].last_char)
 		{
 			I = ints + i;
-			TRACE("Already contained by interval %d", i);
+			TRACE("    Already contained by interval %d: (%d,%d)=%d", i, INT(I));
 		}
 		else if (ints[i].last_char <= pos)
 		{
@@ -466,7 +523,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 		I->last_char = pos_end;
 		I->start_type = FT_TYPE_UNSET;
 		I->context_len = printable_characters(pos, pos_end);
-		TRACE("Temporarily created new interval of length %d", I->context_len);
+		TRACE("    Temporarily created new interval (%d,%d)=%d", INT(I));
 		total_added = I->context_len;
 	}
 	else
@@ -478,6 +535,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 			total_added += sub_len;
 			I->context_len += sub_len;
 			I->first_char = pos;
+			TRACE("    Extended to (%d,%d)=%d", INT(I));
 		}
 		if (pos_end > I->last_char)
 		{
@@ -485,6 +543,7 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 			total_added += sub_len;
 			I->context_len += sub_len;
 			I->last_char = pos_end;
+			TRACE("    Extended to (%d,%d)=%d", INT(I));
 		}
 	}
 	available -= total_added;
@@ -493,33 +552,47 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 	int added = 0;
 	int limit1 = available/3;
 	int limit2 = available/2;
+	if (I->last_char == ft.text_end)
+	{
+		limit1 = available*2/3;
+		limit2 = available;
+	}
+	byte *may_break = NULL;
+	byte *nice_break = NULL;
 	uns c;
 	pos = I->first_char;
-	while (pos > softl)
+	while (pos > softl && added < limit2)
 	{
 		byte *pos1;
 		PREV_TAGGED_CHAR(pos, pos1, ft.text, c);
-		if (c < 0x8000000)
+		if (c < 0x80000000)
 		{
 			added++;
-			if (added >= limit2 && IS_WORD_BREAK(c)
-			|| added >= limit1 && IS_SENTENCE_BREAK(c))
-				break;
-		}
-		else if (c >= 0x80000000 && c < 0x80001000)
-		{
-			if (added >= limit1)
-				break;
+			if (IS_WORD_BREAK(c))
+				may_break = pos;
+			else if (added >= limit1 && IS_SENTENCE_BREAK(c))
+				nice_break = pos;
 		}
+		else if (c < 0x80010000)
+			may_break = pos;
 		pos = pos1;
 	}
-	if (added)
-		TRACE("Stretched to the left side by %d characters", added);
-	available -= added;
-	total_added += added;
-	I->context_len += added;
-	I->first_char = pos;
+	if (pos == softl)
+		may_break = pos;
+	if (nice_break)
+		may_break = nice_break;
+	if (may_break && pos < I->first_char)
+	{
+		added = printable_characters(may_break, I->first_char);
+		available -= added;
+		total_added += added;
+		I->context_len += added;
+		I->first_char = may_break;
+		TRACE("    Stretched to the left side by %d characters -> (%d,%d)=%d", added, INT(I));
+	}
+
 	/* Find out the new starting type.  */
+	pos = I->first_char;
 	GET_TAGGED_CHAR(pos, c);
 	while (1)
 	{
@@ -539,47 +612,54 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 	limit1 = available*2/3;
 	limit2 = available;
 	pos = I->last_char;
-	while (pos < softr)
+	may_break = nice_break = NULL;
+	while (pos < softr && added < limit2)
 	{
 		byte *pos1;
 		uns c;
 		pos1 = pos;
 		GET_TAGGED_CHAR(pos, c);
-		if (c < 0x8000000)
+		if (c < 0x80000000)
 		{
 			added++;
-			if (added >= limit2 && IS_WORD_BREAK(c)
-			|| added >= limit1 && IS_SENTENCE_BREAK(c))
+			if (IS_WORD_BREAK(c))
 			{
 				if (Uspace(c))
-					added--, pos = pos1;
-				break;
-			}
-		}
-		else if (c >= 0x80000000 && c < 0x80001000)
-		{
-			if (added >= limit1)
-			{
-				pos = pos1;
-				break;
+					may_break = pos1;
+				else
+					may_break = pos;
 			}
+			else if (added >= limit1 && IS_SENTENCE_BREAK(c))
+				nice_break = pos;
 		}
+		else if (c < 0x80001000)
+			may_break = pos1;
+	}
+	if (pos == softr)
+		may_break = pos;
+	if (nice_break)
+		may_break = nice_break;
+	if (may_break && (added = printable_characters(I->last_char, may_break)))
+	{
+		available -= added;
+		total_added += added;
+		I->context_len += added;
+		I->last_char = may_break;
+		TRACE("    Stretched to the right side by %d characters -> (%d,%d)=%d", added, INT(I));
 	}
-	if (added)
-		TRACE("Stretched to the right side by %d characters", added);
-	available -= added;
-	total_added += added;
-	I->context_len += added;
-	I->last_char = pos;
 
-	if (I->first_char == softl && softl > ft.text)
+	byte *ext_softl = skip_spaces(softl, softr);
+	if ((I->first_char == softl || I->first_char == ext_softl) && softl > ft.text)
 	{
 		/* Merge to the left.  */
 		uns i;
-		for (i = 0; ints[i].last_char != I->first_char; i++);
+		for (i = 0; ints[i].last_char != softl; i++);
+		uns pad = printable_characters(softl, ext_softl);
 		ints[i].last_char = I->last_char;
-		ints[i].context_len += I->context_len;
-		TRACE("Merged with interval %d on the left side", i);
+		ints[i].context_len += I->context_len + pad;
+		TRACE("    Merged with interval %d on the left side (padding %d) -> (%d,%d)=%d", i, pad, INT(&ints[i]));
+		available -= pad;
+		total_added += pad;
 		if (I != &tmp_int)
 		{
 			ASSERT(I == ints + (i+1));
@@ -587,15 +667,19 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 		}
 		I = ints+i;
 	}
-	if (I->last_char == softr && softr < ft.text_end)
+	byte *lc = skip_spaces(I->last_char, softr);
+	if ((I->last_char == softr || lc == softr) && softr < ft.text_end)
 	{
 		/* Merge to the right.  */
 		uns i;
-		for (i = 0; ints[i].first_char != I->last_char; i++);
+		for (i = 0; ints[i].first_char != lc; i++);
+		uns pad = printable_characters(I->last_char, lc);
 		ints[i].first_char = I->first_char;
-		ints[i].context_len += I->context_len;
+		ints[i].context_len += I->context_len + pad;
 		ints[i].start_type = I->start_type;
-		TRACE("Merged with interval %d on the right side", i);
+		TRACE("    Merged with interval %d on the right side (padding %d) -> (%d,%d)=%d", i, pad, INT(&ints[i]));
+		available -= pad;
+		total_added += pad;
 		if (I != &tmp_int)
 		{
 			ASSERT(I == ints + (i-1));
@@ -612,14 +696,15 @@ contint_add(struct query *q, byte *pos, byte *pos_end, struct context_interval *
 				right_neighbour = i + 1;
 		if (i >= maxints)
 		{
-			TRACE("Cannot add a new interval, since the maximum %d has already been reached", maxints);
+			TRACE("    Cannot add a new interval, since the maximum %d has already been reached", maxints);
 			return 0;
 		}
 		contint_insert(ints, maxints, right_neighbour);
 		ints[right_neighbour] = tmp_int;
-		TRACE("Added a new interval before %d", right_neighbour);
+		TRACE("    Added a new interval (%d,%d)=%d before %d", INT(I), right_neighbour);
 	}
 	return total_added;
+#undef INT
 }
 
 static void
@@ -631,22 +716,22 @@ dump_context(struct query *q, uns max_intervals, int context)
 	uns count = 0;
 	for (uns i=0; i<NUM_BESTS && ft.bests[i].pos != POS_NOWHERE; i++)
 	{
-		TRACE("Best %d: pos %d, Q %d", count, ft.bests[i].pos, ft.bests[i].weight);
+		TRACE("    Best %d: pos %d, Q %d", count, ft.bests[i].pos, ft.bests[i].weight);
 		count++;
 	}
 
 	/* Verbose debug messages.  */
 	if (0)
 	{
-		TRACE("Length of text: %d", (int)(ft.text_end-ft.text));
-		char mask[20];
-		sprintf(mask, "Text: %%%d.%ds", (uns)(ft.text_end-ft.text), (uns)(ft.text_end-ft.text));
+		TRACE("  Length of text: %d", (int)(ft.text_end-ft.text));
+		char mask[24];
+		sprintf(mask, "    Text: %%%d.%ds", (uns)(ft.text_end-ft.text), (uns)(ft.text_end-ft.text));
 		TRACE(mask, ft.text);
 		for (uns i=0; i<ft.num_words; i++)
 		{
 			struct ft_word *w = ft.words + i;
-			char mask[20];
-			sprintf(mask, "Word %%3d: l%%2d c%%x \"%%%d.%ds\"", w->olen, w->olen);
+			char mask[24];
+			sprintf(mask, "    Word %%3d: l%%2d c%%x \"%%%d.%ds\"", w->olen, w->olen);
 			TRACE(mask, w->pos, w->olen, w->type, w->orig);
 		}
 	}
@@ -670,13 +755,13 @@ dump_context(struct query *q, uns max_intervals, int context)
 		{
 			struct ft_word *w = ft_find_word_by_pos(ft.bests[i].pos);
 			ADD_CONTEXT(w->orig, w->orig + w->olen, context1,
-				("...Added best interval of length %d", added_context));
+				("    -> Added best interval of length %d", added_context));
 		}
 		/* If we still have an available context, try to spend it
 		 * again, preferring the former intervals.  */
 		for (i=0; i < max_intervals && interval[i].first_char && context > 0; i++)
-			ADD_CONTEXT(interval[i].first_char, interval[i].last_char, context/2,
-				("...Stretched best interval %d by length %d", i, added_context));
+			ADD_CONTEXT(interval[i].last_char, interval[i].last_char, context/2,
+				("    -> Stretched best interval %d by length %d", i, added_context));
 	}
 	if (!interval[0].first_char)
 	{
@@ -684,7 +769,7 @@ dump_context(struct query *q, uns max_intervals, int context)
 		{
 			struct ft_word *w = ft.words;
 			ADD_CONTEXT(w->orig, w->orig + w->olen, context,
-				("...Added beginning of text of length %d", added_context));
+				("    -> Added beginning of text of length %d", added_context));
 		}
 		else if (ft.text_end > ft.text)	/* might consist of non-words only */
 		{
@@ -696,7 +781,7 @@ dump_context(struct query *q, uns max_intervals, int context)
 				byte *pos1 = pos;
 				GET_TAGGED_CHAR(pos1, c);
 				ADD_CONTEXT(pos, pos1, context,
-					("...Added beginning of non-word text of length %d", added_context));
+					("    -> Added beginning of non-word text of length %d", added_context));
 			}
 		}
 	}
@@ -704,13 +789,13 @@ dump_context(struct query *q, uns max_intervals, int context)
 	{
 		int save_context = context;
 		for (uns i=0; i < max_intervals && interval[i].first_char && context > 0; i++)
-			ADD_CONTEXT(interval[i].first_char, interval[i].last_char, context,
-				("...Finally, stretched interval %d by length %d", i, added_context));
+			ADD_CONTEXT(interval[i].last_char, interval[i].last_char, context,
+				("    -> Finally, stretched interval %d by length %d", i, added_context));
 		if (context == save_context)
 			break;
 	}
 
-	/* Dump the titles and the highlited context.  */
+	/* Dump the titles and the highlighted context.  */
 	uns end = 0;
 	for (uns i=0; i<max_intervals && interval[i].first_char; i++)
 		if (interval[i].last_char == ft.text_end)
@@ -720,9 +805,11 @@ dump_context(struct query *q, uns max_intervals, int context)
 	sprintf(label, "<%s w=%x m=%d e=%d>", type_title, ft.word_mask, see >= 0, end);
 	for (uns i=0; i<max_intervals && interval[i].first_char; i++)
 	{
-		TRACE("Result: Interval %d of type %x, bytes %d-%d context %d", i,
-			interval[i].start_type, (uns)(interval[i].first_char - ft.text),
-			(uns)(interval[i].last_char - ft.text), interval[i].context_len);
+		TRACE("  => Result: Interval %d of type %s, bytes %d-%d context %d", i,
+			(ft.input_type == FT_TEXT ? wt_names : mt_names) [interval[i].start_type],
+			(uns)(interval[i].first_char - ft.text),
+			(uns)(interval[i].last_char - ft.text),
+			interval[i].context_len);
 		dump_text_interval(i ? NULL : label, interval + i);
 	}
 }
@@ -904,8 +991,9 @@ parse_brackets(struct query *q, uns type, byte **start, byte *end, struct save_l
 
 	uns head = n->sons ? n->son[0]->start - n->start : 0;
 	uns tail = n->sons ? n->end - n->son[n->sons-1]->end : 0;
-	TRACE("Parsed %c-bracket of length %d=%d+%dsons+%d and weight1 %d, occurences: %s",
-		BT_LETTERS[type >> BT_SHIFT], (int)(n->end - n->start), head, n->sons, tail, n->weight, tmp);
+	TRACE("Parsed %c-bracket of length %d=%d+%dsons+%d and weight1 %d%s%s",
+		BT_LETTERS[type >> BT_SHIFT], (int)(n->end - n->start), head, n->sons, tail, n->weight,
+		((occur[0] || occur[1]) ? ", occurences: " : ""), tmp);
 
 	n->sons |= type;
 	return n;
@@ -1061,7 +1149,7 @@ dump_lines_between(struct query *q, struct save_lm_pos *saved_pos, byte *start,
 #define	ASORT_ELT(i)	((br->son[i]->sons & BT_MASK) | (BT_BASIS - br->son[i]->weight))
 #define	ASORT_SWAP(i,j)	do { struct bracket *tmp=br->son[j]; br->son[j]=br->son[i]; br->son[i]=tmp; } while(0)
 #define	ASORT_EXTRA_ARGS	, struct bracket *br
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static void
 dump_brackets(struct query *q, struct bracket *n)
@@ -1119,16 +1207,32 @@ struct card_fetch {
 static struct work_queue fetch_wqueue;
 static struct worker_pool fetch_wpool;
 
+static inline ucw_off_t
+fetch_pos(struct card_fetch *f)
+{
+  return (ucw_off_t) f->ca->card << CARD_POS_SHIFT;
+}
+
+static inline uns
+fetch_len(struct card_fetch *f)
+{
+  return ((f->ca+1)->card - f->ca->card) << CARD_POS_SHIFT;
+}
+
+static inline int
+fetch_fd(struct card_fetch *f)
+{
+  return f->db->fd_cards;
+}
+
 static void
 fetch_single(struct worker_thread *t UNUSED, struct work *work)
 {
   /* Called by worker threads to service fetch requests */
-  struct card_fetch *w = (struct card_fetch *) work;
-  struct card_attr *ca = w->ca;
-  struct database *db = w->db;
-  int size = ((ca+1)->card - ca->card) << CARD_POS_SHIFT;
-  if ((int)sh_pread(db->fd_cards, w->card, size, (sh_off_t) ca->card << CARD_POS_SHIFT) != size)
-    w->card = NULL;
+  struct card_fetch *f = (struct card_fetch *) work;
+  uns len = fetch_len(f);
+  if ((uns)ucw_pread(fetch_fd(f), f->card, len, fetch_pos(f)) != len)
+    f->card = NULL;
 }
 
 static void
@@ -1148,7 +1252,8 @@ fetch_start(struct query *q)
         {
 	  struct card_fetch *f = &fetches[i];
 	  f->w.go = fetch_single;
-	  f->card = mp_alloc(q->pool, ((f->ca+1)->card - f->ca->card) << CARD_POS_SHIFT);
+	  f->card = mp_alloc(q->pool, fetch_len(f));
+	  log_fetch(q, fetch_fd(f), fetch_pos(f), fetch_len(f));
 	}
       for (uns i=0; i<n; i++)
         work_submit(&fetch_wqueue, &fetches[i].w);
@@ -1160,9 +1265,9 @@ fetch_start(struct query *q)
         {
 	  struct card_fetch *f = &fetches[i];
           struct mmap_request *m = &mmaps[i];
-          m->u.req.fd = f->db->fd_cards;
-          m->u.req.start = (sh_off_t) f->ca->card << CARD_POS_SHIFT;
-          m->u.req.end = (sh_off_t) (f->ca+1)->card << CARD_POS_SHIFT;
+          m->u.req.fd = fetch_fd(f);
+          m->u.req.start = fetch_pos(f);
+          m->u.req.end = m->u.req.start + fetch_len(f);
           m->userdata = i;
 	  f->done = 1;
 	}
@@ -1208,7 +1313,7 @@ fetch_decompress(struct card_fetch *f, byte **end)
   fetch_wait(f);
   if (!f->card)
     {
-      log(L_ERROR, "Cannot load object %08x from database %s: %m", (uns)(f->ca - f->db->card_attrs), f->db->name);
+      log(L_ERROR, "Cannot load object %08x from database %s", (uns)(f->ca - f->db->card_attrs), f->db->name);
       return NULL;
     }
 
@@ -1247,6 +1352,8 @@ show_card(struct query *q, struct card_attr *ca, struct result_note *note, byte
 {
 	oid_t oid;
 	struct database *db = attr_to_db(ca, &oid);
+	TRACE("*** Dumping card %08x from db %s ***", (uns)oid, db->name);
+
 	ft_context = ft_init_card(q, db, ca);
 	ASSERT(ft_context);
 	show_card_attr(q, db, oid, ca);
