@@ -6,6 +6,7 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/index.h"
+#include "ucw/string.h"
 #include "indexer/params.h"
 #include "search/sherlockd.h"
 
@@ -15,7 +16,7 @@
 #include <sys/time.h>
 
 static void
-cmd_databases(int argc UNUSED, byte **argv UNUSED)
+cmd_databases(int argc UNUSED, char **argv UNUSED)
 {
   add_err("+000 OK");
   add_reply("^DATABASES");
@@ -47,7 +48,7 @@ cmd_databases(int argc UNUSED, byte **argv UNUSED)
 
 struct command {
   char *name;
-  void (*handler)(int argc, byte **argv);
+  void (*handler)(int argc, char **argv);
   int argmin, argmax, passwd;
 };
 
@@ -60,11 +61,11 @@ void
 do_command(struct query *q)
 {
   byte *c = q->cmd;
-  byte *words[16], **argv;
+  char *words[16], **argv;
   int argc;
   struct command *cmd = cmds;
 
-  argc = wordsplit(c, words, ARRAY_SIZE(words));
+  argc = str_wordsplit(c, words, ARRAY_SIZE(words));
   if (argc <= 0)
     {
       add_err("-109 Malformed command");
