@@ -8,8 +8,8 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/bbuf.h"
+#include "ucw/mempool.h"
+#include "ucw/bbuf.h"
 #include "sherlock/index.h"
 #include "sherlock/tagged-text.h"
 #include "search/sherlockd.h"
@@ -58,7 +58,7 @@ ft_init_voc(struct ref_context *c)
 	w = w->root;
       SLIST_FOR_EACH(struct variant *, v, w->variants)
 	{
-	  byte ww[MAX_WORD_LEN+1];
+	  byte ww[MAX_WORD_BYTES+1];
 	  word_extract_variant(w, v, ww);
 	  DBG("\tInserting <%s> (wid=%d, class=%d)", ww, i, w->word_class);
 	  voc_key_t key;
@@ -352,7 +352,8 @@ ft_match(struct ref_context *c, struct ft_results *res)
     }
   ft_trail_end(c);
 
-  struct ft_word *w = res->words + ft_add_word();
+  uns ftw_index = ft_add_word();
+  struct ft_word *w = res->words + ftw_index;
   w->pos = POS_NOWHERE;
   w->orig = res->text_end;
   w->olen = 0;
