@@ -12,14 +12,14 @@ struct ft_word {
   ref_pos_t pos;			/* Position (as counted by lexmapper) */
   byte olen;				/* Length of original text */
   byte type;				/* WT_xxx or FT_TYPE_UNSET */
-  word weight;				/* Match weight at this position (0=no match, 1=highlight only) */
+  u16 weight;				/* Match weight at this position (0=no match, 1=highlight only) */
 };
 
 #define FT_TYPE_UNSET 0xff
 
 struct ft_best {
   uns pos;				/* Position with encoded meta type, POS_NOWHERE if this slot is empty */
-  word weight;				/* Match weight */
+  u16 weight;				/* Match weight */
   byte word_index;			/* Which word matched */
   byte rfu;
 };
