@@ -6,7 +6,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/chartype.h"
+#include "ucw/chartype.h"
 #include "search/sherlockd.h"
 
 #include <stdio.h>
@@ -83,6 +83,10 @@ STRING_TYPE_NAMES
 #ifdef CONFIG_EXPLAIN
   { "EXPLAIN",			EXPLAIN,	0 },
 #endif
+#ifdef CONFIG_IMAGES_SIM
+  { "IMAGESIM",			IMAGESIM,	0 },
+  { "SIG",			SIG,		0 },
+#endif
 #define CUSTOM_MATCH_KWD(id,keywd,parse) { #keywd, CUSTOM, OFFSETOF(struct query, id##_placeholder) },
   CUSTOM_MATCH_PARSE
 #undef CUSTOM_MATCH_KWD
@@ -115,7 +119,7 @@ yylex(void)
       byte x;
       uns k;
 
-      while (Calpha(*pos))
+      while (Calnum(*pos))
 	{
 	  *pos = Cupcase(*pos);
 	  pos++;
