@@ -5,10 +5,10 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/stkstring.h"
+#include "ucw/mempool.h"
+#include "ucw/stkstring.h"
 #include "sherlock/index.h"
-#include "lib/unicode.h"
+#include "ucw/unicode.h"
 #include "indexer/params.h"
 #include "search/sherlockd.h"
 
@@ -35,11 +35,12 @@ prof_t prof_send;
 #undef P
 
 static prof_t *profiler_current;
+static timestamp_t query_timer;
 
 static void
 profiler_init(void)
 {
-  init_timer();
+  init_timer(&query_timer);
 #define P(x) prof_init(&prof_##x)
   PROFILERS(;);	/* Initialize all profilers */
 #undef P
@@ -74,7 +75,7 @@ profiler_show(struct query *q)
   add_footer("T%s", stats);
 #endif
 
-  q->time_total = get_timer();
+  q->time_total = get_timer(&query_timer);
   add_footer("t%d", q->time_total);
 }
 
@@ -313,7 +314,7 @@ format_norm_query(struct query *q, byte *buf, uns size)
 #undef LATE_INT_ATTR
 #undef LATE_SMALL_SET_ATTR
 
-  byte *custom_key = CUSTOM_MATCH_CACHE_KEY(q, q->pool);
+  char *custom_key = CUSTOM_MATCH_CACHE_KEY(q, q->pool);
   if (custom_key)
     {
       uns len = strlen(custom_key);
@@ -712,6 +713,15 @@ check_words(struct query *q)
       if (tracing)
 	add_reply(".B Near %d: mask %08x", i, p->word_mask);
     }
+#ifdef CONFIG_IMAGES_SIM
+  for (i=0; i<q->nimage_sims; i++)
+    {
+      struct image_sim *s = q->image_sims[i];
+      s->boolean_id = id++;
+      if (tracing)
+	add_reply(".B ImageSim %d: bool ID %d", i, s->boolean_id);
+    }
+#endif
   if (id >= max_bools)
     {
       add_err("-118 Boolean expression too complex");
@@ -939,6 +949,8 @@ optimistic_expr(struct expr *x)
     case EX_OR:
       l = optimistic_expr(x->u.op.l);
       r = optimistic_expr(x->u.op.r);
+      if (x->type == EX_OR && (!l || !r))
+	return NULL;
       if (!l)
 	return r;
       if (!r)
@@ -1056,7 +1068,7 @@ show_query_stats(struct query *q)
 	    if (qq[-1] == ' ')
 	      {
 		qq--;
-		PUT_UTF8(qq, 0xa0);
+		qq = utf8_put(qq, 0xa0);
 	      }
 	  add_reply("W%s %d %d %d %d %d%d%08x%d%d%d%llx", disp, w->var_count, w->ref_total_len / 1024,
 		    (q->results->status ? ((w->boolean_id < 0) ? -1 : 0) : (int)w->doc_count),
@@ -1320,7 +1332,7 @@ init_query(struct query *q)
 #undef SMALL_SET_ATTR
 #undef LATE_INT_ATTR
 #undef LATE_SMALL_SET_ATTR
-  CUSTOM_MATCH_INIT(q)
+  CUSTOM_MATCH_INIT(q);
   q->custom_sorting = global_sorting;
   q->custom_sort_reverse = global_sort_reverse;
 }
