@@ -11,12 +11,12 @@
 #undef DEBUG_SLICES
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
-#include "lib/unaligned.h"
-#include "lib/heap.h"
-#include "lib/prefetch.h"
-#include "lib/unicode.h"
-#include "lib/threads.h"
+#include "ucw/mempool.h"
+#include "ucw/unaligned.h"
+#include "ucw/heap.h"
+#include "ucw/prefetch.h"
+#include "ucw/unicode.h"
+#include "ucw/threads.h"
 #include "sherlock/index.h"
 #include "indexer/params.h"
 #include "search/sherlockd.h"
@@ -115,7 +115,7 @@ trail_enlarge(struct ref_context *c, struct trail_entry *t)
 #define ASORT_ELT(i) trail[i].pos
 #define ASORT_SWAP(i,j) do { struct trail_entry tmp=trail[i]; trail[i]=trail[j]; trail[j]=tmp; } while (0)
 #define ASORT_EXTRA_ARGS , struct trail_entry *trail
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static inline struct trail_entry *
 get_trail(struct ref_context *c)
@@ -186,7 +186,7 @@ map_chains(struct ref_context *c)
 		  v->refchain_len, ch->word_index, ch->bool_index, ch->noaccent_only, ch->penalty, ch->lang_mask);
 	    mm[j].u.req.fd = q->dbase->fd_refs;
 	    mm[j].u.req.start = v->refchain_start;
-	    mm[j].u.req.end = MIN(v->refchain_start + (sh_off_t)v->refchain_len, q->dbase->ref_file_size);
+	    mm[j].u.req.end = MIN(v->refchain_start + (ucw_off_t)v->refchain_len, q->dbase->ref_file_size);
 	    mm[j].userdata = j;
 	    w->ref_total_len += v->refchain_len;
 	    q->stat_len_chains += v->refchain_len;
@@ -215,6 +215,8 @@ select_slices_trivial(struct ref_context *c)
    */
   c->chains = c->raw_chains;
   c->num_chains = c->num_raw_chains;
+  c->start_oid = 0;
+  c->end_oid = c->dbase->params->cards_out;
 }
 
 static void
@@ -226,6 +228,8 @@ select_slices(struct ref_context *c, uns slice)
    */
   c->chains = ref_buf_alloc(&c->buffers->sliced_chains, c->num_raw_chains * sizeof(struct chain));
   c->num_chains = 0;
+  c->start_oid = c->dbase->slice_start[slice];
+  c->end_oid = c->dbase->slice_start[slice + 1];
   uns num_slices = c->dbase->params->num_slices;
   DBG("Selecting slice #%d:", slice);
   for (uns i=0; i<c->num_raw_chains; i++)
@@ -300,6 +304,9 @@ init_ref_context(struct query *q, struct ref_buffers *buffers, struct ref_contex
   /* Initialize a full ref context, including reference chains and result heaps. */
 
   struct ref_context *c = init_bare_ref_context(q, buffers);
+#ifdef CONFIG_SITES
+  c->site_only = site_find_id(&q->dbase->sites, q->site_hash);
+#endif
   c->match_heap.max_matches = q->results->max_matches;
   c->match_heap.site_max = q->site_max;
   local_match_init(q->pool, &c->match_heap);
@@ -315,6 +322,8 @@ init_ref_context(struct query *q, struct ref_buffers *buffers, struct ref_contex
   c->ref_heap = ref_buf_alloc(&c->buffers->ref_heap, sizeof(struct ref_heap_entry) * (c->num_raw_chains+1));
   c->matched_chains = ref_buf_alloc(&c->buffers->matched_chains, sizeof(struct chain_match) * (c->num_raw_chains+1));
 
+  CUSTOM_REF_INIT(c);
+
   return c;
 }
 
@@ -340,13 +349,33 @@ merge_ref_context(struct ref_context *c)
   struct global_match_heap *gh = q->match_heap;
   struct local_match_note *ln;
   struct global_match_note *gn;
+#ifdef CONFIG_SITES
+  /*
+   * If we have two documents from the same site, clear the second one's
+   * site compression counter to avoid adding it twice to the global total.
+   */
+  if (lh->site_max)
+    for (uns i=1; i<=lh->num_matches; i++)
+      {
+	ln = lh->heap[i];
+	if (ln->hash_next && ln->hash_next->n.attr->site_id == ln->n.attr->site_id)
+	  ln->hash_next->n.site_compressed = 0;
+      }
+  gh->site_compressed += lh->site_compressed;
+#endif
   while (ln = local_match_delete_min(lh))
     {
       HDBG("Match with Q=%d, ssk=%x", ln->n.q, ln->n.sec_sort_key);
       gn = global_match_get_note(gh);
       gn->n = ln->n;
+#ifdef CONFIG_SITES
+      gn->site_hash = site_find_hash(&q->dbase->sites, ln->n.attr->site_id);
+      HDBG("\tTranslated site ID %08x -> hash %016Lx (compressed %d)", ln->n.attr->site_id, gn->site_hash, gn->n.site_compressed);
+#endif
       global_match_insert(gh, gn);
     }
+
+  CUSTOM_REF_CLEANUP(c);
 }
 
 /* Explanation mode */
@@ -537,7 +566,7 @@ match_near(struct ref_context *c, struct phrase *p)
 	            {
 	              running_weight += cluster[ci].joiner;
 	              if (running_weight < 0)
-	    	        {
+		        {
 			  ct = ci;
 			  running_weight = running_q = 0;
 			  cluster_word_mask = 0;
@@ -653,6 +682,10 @@ match_early_card_attrs(struct ref_context *c UNUSED, struct query *q UNUSED, str
 #ifdef CONFIG_LASTMOD
   if (attrs->age < q->age_min || attrs->age > q->age_max)
     return 0;
+#endif
+#ifdef CONFIG_SITES
+  if (c->site_only && c->site_only != attrs->site_id)
+    return 0;
 #endif
   if (attrs->flags & CARD_FLAG_DUP)
     return 0;
@@ -685,7 +718,11 @@ get_sec_sort_key(struct result_note *note, struct ref_context *c, struct card_at
       key = oid;
       break;
     case PARAM_SITE:
+#ifdef CONFIG_SITES
+      key = attrs->site_id;
+#else
       key = 0;
+#endif
       break;
 #ifdef CONFIG_LASTMOD
     case PARAM_AGE:
@@ -731,6 +768,9 @@ refs_card_record(struct ref_context *c, struct card_attr *attr, oid_t oid, int q
   HDBG("Matched OID %08x with Q=%d", oid, qq);
   note->n.attr = attr;
   note->n.q = qq;
+#ifdef CONFIG_SITES
+  note->n.site_compressed = 0;
+#endif
   get_sec_sort_key(&note->n, c, attr, oid);
   local_match_insert(&c->match_heap, note);
 }
@@ -835,6 +875,29 @@ refs_card(struct ref_context *c, struct card_attr *attr, oid_t oid, u32 words_fo
 	}
     }
 
+#ifdef CONFIG_IMAGES_SIM
+  /* Process images */
+  for (uns i = 0; i<c->nimage_sims; i++)
+    {
+      struct image_sim *s = c->image_sims + i;
+      int sq;
+      if (c->image_sims_bools_seen & (1 << s->boolean_id))
+        {
+	  sq = image_sim_q(s);
+	  IF_EXPLAINING(
+            if (unlikely(c->do_explain))
+	      {
+	        EXPLAIN(c, "ImageSim #%d: Q=%d", i, sq);
+	        image_sim_explain(c, s, oid, image_sim_explain_msg, c);
+	      }
+	  );
+	  qq += sq;
+	  /* static_weight_mul++; */
+	  bool |= 1 << s->boolean_id;
+	}
+    }
+#endif
+
   /* Check boolean expression */
   DBG("Final bool mask: %04x", bool);
   if (!(q->bool_map[bool >> 5] & (1 << (bool & 31))))
@@ -897,6 +960,12 @@ refs_card_any(struct ref_context *c, struct card_attr *attr, oid_t oid)
   int qq = 0;
   uns static_weight_mul = 0;
 
+  if (!match_early_card_attrs(c, q, attr, oid))
+    {
+      DBG("\tEarly attributes don't match");
+      return;
+    }
+
   /* Check custom matchers */
 #if defined(CUSTOM_MATCH_ANY) || defined(CUSTOM_MATCH)
   int custom_Q = 0;
@@ -1195,7 +1264,7 @@ refs_go(struct ref_context *c)
   HEAP_INIT(struct ref_heap_entry, rheap, rcnt, REF_HEAP_LESS, REF_HEAP_SWAP);
 
 #ifdef CONFIG_ALLOW_ANY
-  oid_t last_oid = 0;
+  oid_t last_oid = c->start_oid;
 #endif
 
   while (rcnt > 0)
@@ -1222,6 +1291,11 @@ refs_go(struct ref_context *c)
 	  uns len = get_chain_len(&p);
 
 	  DBG("\tFound in ref chain %d size %d", (int)(ch-c->chains), len);
+#ifdef CONFIG_IMAGES_SIM
+	  if (c->image_sims_bool_mask & (1 << ch->bool_index))
+	    c->image_sims[ch->word_index].dist = *(u32 *)p;
+	  else
+#endif
 	    {
 	      mch->chain = ch;
 	      mch->pos = p;
@@ -1264,16 +1338,16 @@ refs_go(struct ref_context *c)
 
       mch->chain = NULL;
       u32 words_seen = refs_chains(c, attr, c->matched_chains);
+#ifdef CONFIG_IMAGES_SIM
+      c->image_sims_bools_seen = bool;
+#endif
       refs_card(c, attr, oid, words_seen);
     }
 
 #ifdef CONFIG_ALLOW_ANY
   if (q->bool_map[0] & 1)
-    {
-      struct card_attr *attrs_end = c->dbase->card_attrs_end;
-      for (struct card_attr *attr = attrs + ++last_oid; attr != attrs_end; last_oid++, attr++)
-	refs_card_any(c, attr, last_oid);
-    }
+    for (struct card_attr *attr = attrs + ++last_oid; last_oid < c->end_oid; last_oid++, attr++)
+      refs_card_any(c, attr, last_oid);
 #endif
 }
 
@@ -1326,7 +1400,7 @@ process_refs_threaded(struct ref_context *c)
 
   pthread_attr_t attr;
   if (pthread_attr_init(&attr) < 0 ||
-      pthread_attr_setstacksize(&attr, default_thread_stack_size) < 0)
+      pthread_attr_setstacksize(&attr, ucwlib_thread_stack_size) < 0)
     ASSERT(0);
 
   /*
@@ -1417,9 +1491,16 @@ query_finish_refs(struct query *q)
       oid_t oid;
       attr_to_db(n->attr, &oid);
       log(L_DEBUG, "%3d %08x %6d ssk=%x sc=%d", i, oid, n->sec_sort_key, n->q,
+#ifdef CONFIG_SITES
+	  n->site_compressed
+#else
 	  0
+#endif
 	  );
 #endif
     }
   add_reply("N%d", r->num_matches);
+#ifdef CONFIG_SITES
+  add_reply("m%d", h->site_compressed);
+#endif
 }
