@@ -5,9 +5,9 @@
  *	(c) 2005 Robert Spalek <robert@ucw.cz>
  */
 
-#include "lib/bbuf.h"
-#include "lib/unaligned.h"
-#include "lib/unicode.h"
+#include "ucw/bbuf.h"
+#include "ucw/unaligned.h"
+#include "ucw/unicode.h"
 
 #ifdef CONFIG_LANG
 #include "lang/lang.h"
@@ -107,12 +107,24 @@ struct ref_context {
   uns num_raw_chains;
   uns thread_id;
   uns thread_slice_mask;
+  uns start_oid;
+  uns end_oid;
 
   /* Variables internal to fulltext.c */
   struct vocabolario *ft_voc;		/* Vocabolario used during fulltext matching */
   uns ft_lang_mask;
   uns ft_is_accented;
 
+#ifdef CONFIG_IMAGES_SIM
+  /* Similar images */
+  uns nimage_sims;
+  struct image_sim *image_sims;
+  uns image_sims_bool_mask;
+  uns image_sims_bools_seen;
+#endif
+
+  CUSTOM_REF_VARS
+
   struct ref_word words[];
 };
 
