@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/mempool.h"
+#include "ucw/mempool.h"
 #include "search/sherlockd.h"
 
 #include <stdio.h>
@@ -22,13 +22,13 @@ init_reply_buf(struct reply_buf *r, struct mempool *p)
 }
 
 static struct reply *
-make_reply(struct reply_buf *rb, byte *msg, va_list args)
+make_reply(struct reply_buf *rb, byte *m, va_list args)
 {
   byte buf[4096];
   struct reply *r;
   int l;
 
-  l = vsnprintf(buf, sizeof(buf), msg, args);
+  l = vsnprintf(buf, sizeof(buf), m, args);
   if (l < 0 || l >= (int)sizeof(buf))
     l = sprintf(buf, ".!TOO LONG!");
   if (log_replies > 1)
