@@ -1,18 +1,18 @@
 /*
  *	Sherlock Search Engine
  *
- *	(c) 1997--2006 Martin Mares <mj@ucw.cz>
+ *	(c) 1997--2007 Martin Mares <mj@ucw.cz>
  */
 
-#include "lib/clists.h"
-#include "lib/slists.h"
-#include "lib/bitarray.h"
+#include "ucw/clists.h"
+#include "ucw/slists.h"
+#include "ucw/bitarray.h"
 #include "sherlock/index.h"
 #include "indexer/sites.h"
 #include "search/images.h"
 
 #define PROFILE_TOD
-#include "lib/profile.h"
+#include "ucw/profile.h"
 
 /* compile-time parameters */
 
@@ -25,19 +25,20 @@
 
 /* config.c */
 
-extern byte *log_name, *status_name;
-extern uns log_incoming, log_rejected, log_requests, log_replies;
+extern char *log_name, *status_name;
+extern uns log_incoming, log_rejected, log_requests, log_replies, log_fetches;
 extern uns port, listen_queue, connection_timeout, hydra_processes, slice_threads;
-extern byte *control_password;
+extern char *control_password;
 extern clist databases;
 extern clist spell_common_pairs;
 extern clist spell_phrases;
+extern clist spell_kb_trans;
 extern uns num_matches, max_matches, cache_size, max_output_matches;
 extern uns max_words, max_word_matches, max_phrases, max_nears, max_bools;
 extern uns global_accent_mode, wildcard_asterisks, wildcard_qmarks, max_wildcard_zone, min_wildcard_prefix_len;
 extern uns global_context_chars, global_intervals, highlight_substring, global_site_max;
 extern uns global_meta_chars[16];
-extern byte *url_attributes;
+extern char *url_attributes;
 extern uns global_url_max, global_max_redir_brack, global_max_cat_brack, global_morphing, global_spelling, global_synonyming;
 extern uns doc_weight_scale, word_weight_scale, word_bonus;
 extern uns global_allow_approx, prox_penalty;
@@ -53,12 +54,13 @@ extern uns near_bonus_word, near_penalty_gap, near_bonus_connect, near_min_weigh
 extern uns blind_match_penalty, misaccent_penalty, stem_penalty, morph_penalty, synonymum_penalty;
 extern uns spell_good_freq, spell_min_len, spell_margin, spell_dwarf, spell_dwarf_margin, global_syn_expand, spell_common_penalty;
 extern uns spell_add_penalty, spell_del_penalty, spell_mod_penalty, spell_xpos_penalty, spell_accent_penalty;
+extern uns filter_repeated_nonalpha, filter_repeated_alpha;
 extern clist access_list;
 
 struct database {
   cnode node;
-  byte *name;
-  byte *directory;
+  char *name;
+  char *directory;
   enum {
     DB_PART_WORDS = 1,
     DB_PART_STRINGS = 2,
@@ -66,9 +68,10 @@ struct database {
     DB_PART_IMAGE_SIGNATURES = 8
   } parts;
   uns is_optional;
-  byte **blacklists;
+  char **blacklists;
   struct mempool *pool;			/* Contains all information local to this database */
   bitarray_t dup_flags;
+  uns slice_start[HARD_MAX_SLICES + 1];
 
   /* Parameters */
   struct index_params *params;
@@ -77,14 +80,14 @@ struct database {
   oid_t num_ids;
   int fd_cards, fd_refs;
   struct card_attr *card_attrs, *card_attrs_end;
-  sh_off_t card_file_size, ref_file_size;
+  ucw_off_t card_file_size, ref_file_size;
 
   /* Words */
   uns word_weights[8];
   uns meta_weights[16][4];
   uns lexicon_words, lexicon_complexes, lexicon_file_size;
   struct lex_entry **lex_array;
-  uns lex_by_len[MAX_WORD_LEN+2];
+  uns lex_by_len[MAX_WORD_CHARS+2];
   struct lex_entry ***cplx_array;
   uns stems_file_size;
   clist stem_block_list, syn_block_list;
@@ -101,6 +104,11 @@ struct database {
   /* Fingerprints */
   struct fastbuf *fb_card_prints;
 
+#ifdef CONFIG_SITES
+  /* Sites */
+  struct site_mapping_table sites;
+#endif
+
   /* Similar images */
   int fd_image_signatures;
   uns image_clusters_depth;
@@ -114,10 +122,17 @@ struct spell_pair {
 
 struct spell_phrase {
   cnode n;
-  byte *src, *dest;
+  char *src, *dest;
   uns penalty;
 };
 
+struct spell_kb_tran {
+  cnode n;
+  uns penalty;
+  clist pairs;
+  void *table;
+};
+
 /*** Reply buffers: we keep fragments of the reply in them (reply.c) ***/
 
 struct reply {
@@ -359,7 +374,7 @@ struct query {				/* Keep the most frequently used parts near the top */
   uns ncard_fetches;
 
   /* Connection */
-  sh_time_t established;		/* Time this connection was established */
+  ucw_time_t established;		/* Time this connection was established */
   int fd;				/* Socket file descriptor */
   int fd_err;				/* Set in case an error has occured */
   int q_status;				/* Error code returned */
@@ -438,7 +453,7 @@ struct variant {
   byte penalty;				/* Extra negative weight */
   byte flags;				/* VF_xxx */
   byte rfu;
-  sh_off_t refchain_start;		/* Refchain corresponding to this variant */
+  ucw_off_t refchain_start;		/* Refchain corresponding to this variant */
   uns refchain_len;			/* (can be empty) */
 };
 
@@ -474,6 +489,9 @@ struct result_note {
   struct card_attr *attr;		/* Attributes of this card */
   int q;
   u32 sec_sort_key;
+#ifdef CONFIG_SITES
+  int site_compressed;			/* Number of documents ignored from this site */
+#endif
 };
 
 struct results {			/* Query results we cache */
@@ -484,7 +502,7 @@ struct results {			/* Query results we cache */
   byte *request;			/* Normalized request */
   u32 db_mask;				/* A set of databases */
   uns max_matches;			/* Result limit */
-  sh_time_t create_time, access_time;	/* Time created/last used */
+  ucw_time_t create_time, access_time;	/* Time created/last used */
 
   /* The results */
   struct mempool *pool;			/* Pool holding this structure and everything referenced by it */
@@ -546,9 +564,9 @@ void show_results(struct query *q);
 void prefetch_results(struct query *q);
 void prefetch_results_cleanup(struct query *q);
 
-extern byte *wt_names[];
-extern byte *mt_names[];
-extern byte *st_names[];
+extern const char * const wt_names[];
+extern const char * const mt_names[];
+extern const char * const st_names[];
 
 /* words.c */
 
@@ -564,7 +582,7 @@ void word_extract_variant(struct word *w, struct variant *v, byte *buf);
 void strings_init(struct database *db);
 struct expr *string_analyse(struct query *q, struct expr *e);
 void string_analyse_simple(struct query *q, clist *l);
-int string_db_find_refchain(struct query *q, struct database *db, byte *s, sh_off_t *refchain_start, uns *refchain_len);
+int string_db_find_refchain(struct query *q, struct database *db, byte *s, ucw_off_t *refchain_start, uns *refchain_len);
 
 /* spell.c */
 
@@ -577,7 +595,7 @@ struct mmap_request {
   union {
     struct {
       int fd;
-      sh_off_t start, end;
+      ucw_off_t start, end;
     } req;
     struct {
       void *start, *end;
@@ -589,8 +607,9 @@ struct mmap_request {
 void memory_init(void);
 void memory_setup(struct query *q);
 void memory_flush(struct query *q);
-void *mmap_region(struct query *q, int fd, sh_off_t start, sh_off_t end);
+void *mmap_region(struct query *q, int fd, ucw_off_t start, ucw_off_t end);
 int mmap_regions(struct query *q, struct mmap_request *reqs, int count);
+void log_fetch(struct query *q, int fd, ucw_off_t start, ucw_off_t size);
 
 extern struct lizard_buffer *liz_buf;
 
