@@ -7,8 +7,8 @@
 #undef LOCAL_DEBUG
 
 #include "sherlock/sherlock.h"
-#include "lib/unicode.h"
-#include "lib/conf.h"
+#include "ucw/unicode.h"
+#include "ucw/conf.h"
 #include "charset/unicat.h"
 #include "search/sherlockd.h"
 #include "search/lexicon.h"
@@ -30,6 +30,7 @@ enum spell_found {
   SPELL_FOUND_MOD,
   SPELL_FOUND_XPOS,
   SPELL_FOUND_PHRASE,
+  SPELL_FOUND_KB_TRAN,
 };
 
 struct spell_best {
@@ -50,13 +51,27 @@ struct spell_phrase_group {
   int dif;
 };
 
+struct spell_kb_node {
+  uns x, y;
+};
+
 static struct spell_best spell_best[HARD_MAX_SPELLS];
 static uns spell_max, spell_n, spell_threshold, spell_accent_mode;
 static uns *spell_known_vars, spell_n_known_vars;
 static byte *spell_wa;
 static struct spell_phrase_item *spell_phrase;
+static struct spell_kb_tran *spell_kb_tran;
 static clist spell_phrase_groups;
 
+#define HASH_PREFIX(x) spell_kb_##x
+#define HASH_NODE struct spell_kb_node
+#define HASH_KEY_ATOMIC x
+#define HASH_TABLE_DYNAMIC
+#define HASH_ZERO_FILL
+#define HASH_WANT_LOOKUP
+#define HASH_WANT_FIND
+#include "ucw/hashtable.h"
+
 void
 spell_init(void)
 {
@@ -84,6 +99,18 @@ spell_init(void)
 	}
       clist_add_tail(&groups[i]->items, &item->n);
     }
+  CLIST_FOR_EACH(struct spell_kb_tran *, t, spell_kb_trans)
+    {
+      struct spell_kb_table *table = t->table = cf_malloc_zero(sizeof(*table));
+      spell_kb_init(table);
+      CLIST_FOR_EACH(struct spell_pair *, p, t->pairs)
+        {
+	  struct spell_kb_node *node = spell_kb_lookup(table, p->x);
+	  if (node->y)
+	    die("Found a duplicate source character 0x%x in Search.SpellKBTranslation.Pairs", p->x);
+	  node->y = p->y;
+	}
+    }
 }
 
 static inline int
@@ -125,8 +152,8 @@ spell_differs(struct lex_entry *l, uns li, uns oi, uns len)
     UTF8_SKIP(ow);
   while (len-- && lw < le)
     {
-      GET_UTF8(lw, lu);
-      GET_UTF8(ow, ou);
+      lw = utf8_get(lw, &lu);
+      ow = utf8_get(ow, &ou);
       diff += spell_diff_char(lu, ou);
     }
   return diff;
@@ -173,6 +200,9 @@ spell_found(uns id, uns unacc_id, uns pos, enum spell_found found_what)
     case SPELL_FOUND_PHRASE:
       pts -= spell_phrase->penalty + spell_differs(l, 0, 0, pos-spell_phrase->len-spell_phrase->dif) + spell_differs(l, pos, pos-spell_phrase->dif, ~0U);
       break;
+    case SPELL_FOUND_KB_TRAN:
+      pts -= spell_kb_tran->penalty;
+      break;
     }
 
   if (spell_n == spell_max && spell_best[spell_n-1].pts >= pts)
@@ -212,12 +242,12 @@ spell_found(uns id, uns unacc_id, uns pos, enum spell_found found_what)
  done:
   if (IS_TRACING)
     {
-      byte buf[MAX_WORD_LEN+1];
+      byte buf[MAX_WORD_BYTES+1];
       lex_extract(id, buf);
       add_reply(".Z <%s> freq=%d pts=%d: %s", buf, l->freq, pts, msg);
     }
 #ifdef LOCAL_DEBUG
-  byte buf[MAX_WORD_LEN+1];
+  byte buf[MAX_WORD_BYTES+1];
   lex_extract(id, buf);
   DBG("Found <%s> freq=%d pts=%d what=%d@%d: %s", buf, l->freq, pts, found_what, pos, msg);
 #endif
@@ -234,9 +264,9 @@ spell_extract_suffix(int id, uns prefix_len, byte *buf)
     UTF8_SKIP(w);
   while (w < we)
     {
-      GET_UTF8(w, u);
+      w = utf8_get(w, &u);
       u = Uunaccent(u);
-      PUT_UTF8(buf, u);
+      buf = utf8_put(buf, u);
     }
   *buf = 0;
 }
@@ -248,7 +278,7 @@ spell_check_rest(int l, int r, uns prefix_len, byte *suffix, enum spell_found fo
    * all words which are equal to suffix after removing the prefix.
    */
   int r0 = r;
-  byte buf[2*MAX_WORD_LEN+1];
+  byte buf[MAX_WORD_BYTES+1];
   DBG("\t\t\tSearching [%d,%d] for <%s>, prefix_len=%d", l, r, suffix, prefix_len);
   while (l < r)		     /* Invariant: first word lies in [l,r] */
     {
@@ -277,7 +307,7 @@ spell_nth_char(int id, uns n)
   for (uns i=0; i<n; i++)
     UTF8_SKIP(w);
   uns u;
-  GET_UTF8(w, u);
+  w = utf8_get(w, &u);
   return Uunaccent(u);
 }
 
@@ -333,7 +363,7 @@ spell_check_del(byte *wu, uns len)
   for (uns i=0; i<len && l<=r; i++)
     {
       uns u;
-      GET_UTF8(wu, u);
+      wu = utf8_get(wu, &u);
       DBG("\ti=%d (char %x) [%d,%d]", i, u, l, r);
       spell_check_rest(l, r, i, wu, SPELL_FOUND_DEL);
       spell_restrict(&l, &r, i, u);
@@ -350,7 +380,7 @@ spell_check_add(byte *wu, uns len)
     {
       byte *c = wu;
       uns u;
-      GET_UTF8(wu, u);
+      wu =  utf8_get(wu, &u);
       DBG("\ti=%d (char %x) [%d,%d]", i, u, l, r);
       int l1=l, r1;
       while (l1 <= r)
@@ -373,7 +403,7 @@ spell_check_mod(byte *wu, uns len)
   for (uns i=0; i<len; i++)
     {
       uns u;
-      GET_UTF8(wu, u);
+      wu = utf8_get(wu, &u);
       DBG("\ti=%d (char %x) [%d,%d]", i, u, l, r);
       int l1=l, r1;
       while (l1 <= r)
@@ -397,16 +427,16 @@ spell_check_xpos(byte *wu, uns len)
       byte *t;
       uns u1, u2;
       t = wu;
-      GET_UTF8(t, u1);
-      GET_UTF8(t, u2);
+      t = utf8_get(t, &u1);
+      t = utf8_get(t, &u2);
       t = wu;
-      PUT_UTF8(t, u2);
-      PUT_UTF8(t, u1);
+      t = utf8_put(t, u2);
+      t = utf8_put(t, u1);
       spell_check_rest(l, r, i, wu, SPELL_FOUND_XPOS);
       t = wu;
-      PUT_UTF8(t, u1);
+      t = utf8_put(t, u1);
       wu = t;
-      PUT_UTF8(t, u2);
+      t = utf8_put(t, u2);
       spell_restrict(&l, &r, i, u1);
     }
 }
@@ -427,7 +457,7 @@ spell_check_phrases(byte *wu, uns len)
       for (uns i = 1; i <= len; i++)
         {
           uns u1;
-          GET_UTF8(wu, u1);
+          wu = utf8_get(wu, &u1);
           wa[i] = wu;
           la[i] = la[i - 1];
           ra[i] = ra[i - 1];
@@ -443,7 +473,7 @@ spell_check_phrases(byte *wu, uns len)
 		    for (; l <= r; j++)
 		      {
 		        uns u2;
-		        GET_UTF8(d, u2);
+		        d = utf8_get(d, &u2);
 		        if (!u2)
 		          {
 			    spell_phrase = p;
@@ -459,6 +489,41 @@ spell_check_phrases(byte *wu, uns len)
     }
 }
 
+static void
+spell_check_kb_trans(byte *wu, uns len UNUSED)
+{
+  byte buf[MAX_WORD_BYTES + 4];
+  CLIST_FOR_EACH(struct spell_kb_tran *, t, spell_kb_trans)
+    {
+      byte *d = buf;
+      uns found = 0;
+      for (const byte *s = wu; *s; )
+        {
+	  uns u;
+	  s = utf8_get(s, &u);
+	  struct spell_kb_node *n = spell_kb_find(t->table, u);
+	  if (n)
+	    {
+	      u = Uunaccent(n->y);
+	      found = 1;
+	    }
+	  d = utf8_put(d, u);
+	  if (d >= buf + MAX_WORD_BYTES)
+	    goto next;
+	}
+      *d = 0;
+      if (found)
+        {
+	  int l = current_dbase->lex_by_len[len];
+	  int r = current_dbase->lex_by_len[len + 1] - 1;
+          DBG("check_kb_trans: `%s', interval=[%d,%d]", buf, l, r);
+	  spell_kb_tran = t;
+          spell_check_rest(l, r, 0, buf, SPELL_FOUND_KB_TRAN);
+	}
+next: ;
+    }
+}
+
 static void
 spell_word(uns idx)
 {
@@ -480,7 +545,7 @@ spell_word(uns idx)
 
   /* Prepare unaccented word and check length */
   spell_wa = w->word;
-  byte wu[MAX_WORD_LEN+3];
+  byte wu[MAX_WORD_BYTES+3];
   if (!word_unaccent_utf8(spell_wa, wu))
     return;
   uns len = utf8_strlen(wu);
@@ -498,6 +563,9 @@ spell_word(uns idx)
   spell_check_xpos(wu, len);
   /* Check phrases */
   spell_check_phrases(wu, len);
+  /* Check keyboard layouts */
+  if (spell_accent_mode)
+    spell_check_kb_trans(wu, len);
 
   /* If the word is a dwarf, we should consider restarting the search with a lower threshold
    * if there are no matches, but we'll better do it in a single pass by clever filtering.
@@ -560,7 +628,7 @@ spell_check(struct query *q)
 	  spell_word(i);
 	  for (j=0; j<spell_n; j++)
 	    {
-	      byte wbuf[MAX_WORD_LEN+1];
+	      byte wbuf[MAX_WORD_BYTES+1];
 	      lex_extract(spell_best[j].id, wbuf);
 	      add_reply("S%s %s %d", wbuf, w->word, spell_best[j].pts);
 	    }
