@@ -20,9 +20,9 @@
 #ifndef _SEARCH_VOCABOLARIO_H
 #define _SEARCH_VOCABOLARIO_H
 
-#include "lib/mempool.h"
-#include "lib/unaligned.h"
-#include "lib/unicode.h"
+#include "ucw/mempool.h"
+#include "ucw/unaligned.h"
+#include "ucw/unicode.h"
 #include "charset/unicat.h"
 
 #include <string.h>
@@ -53,7 +53,7 @@ void voc_dump(struct vocabolario *voc);
 typedef struct {
   uns len;
   uns hash;
-  byte k[2*MAX_WORD_LEN+4];
+  byte k[MAX_WORD_BYTES+4];
 } voc_key_t;
 
 static inline u32 PURE
@@ -69,8 +69,6 @@ static inline uns
 voc_key_finish(struct vocabolario *voc, voc_key_t *key, byte *k)
 {
   byte *kk = key->k;
-  if (k - kk > MAX_WORD_LEN)	/* Unfortunately, this could be false due to lowercasing */
-    return 0;
 #if CPU_ALLOW_UNALIGNED
   *(u32 *)k = 0;
 #else
@@ -95,7 +93,7 @@ voc_key_ucs2(struct vocabolario *voc, voc_key_t *key, u16 *w, uns len)
     {
       uns u = *w++;
       u = Utolower(u);
-      PUT_UTF8(k, u);
+      k = utf8_put(k, u);
     }
   return voc_key_finish(voc, key, k);
 }
@@ -107,11 +105,11 @@ voc_key_utf8(struct vocabolario *voc, voc_key_t *key, byte *w)
   for(;;)
     {
       uns u;
-      GET_UTF8(w, u);
+      w = utf8_get(w, &u);
       if (!u)
 	break;
       u = Utolower(u);
-      PUT_UTF8(k, u);
+      k = utf8_put(k, u);
     }
   return voc_key_finish(voc, key, k);
 }
