@@ -13,10 +13,10 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/bucket.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/lfs.h"
-#include "lib/conf.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/lfs.h"
+#include "ucw/conf.h"
 
 #include <string.h>
 #include <fcntl.h>
@@ -33,9 +33,10 @@ static struct obuck_params obuck_params = {
   .shake_security = 0,
   .slurp_buflen = 65536,
   .prefetch_size = 8192,
+  .max_size = ~(u64)0,
 };
 
-byte *bucket_file_name = "db/objects";
+char *bucket_file_name = "db/objects";
 struct obuck bucket_file;
 
 static struct cf_section obuck_config = {
@@ -46,6 +47,7 @@ static struct cf_section obuck_config = {
     CF_UNS("ShakeSecurity", &obuck_params.shake_security),
     CF_UNS("SlurpBufSize", &obuck_params.slurp_buflen),
     CF_UNS("PrefetchSize", &obuck_params.prefetch_size),
+    CF_U64("MaxSize", &obuck_params.max_size),
     CF_END
   }
 };
@@ -58,7 +60,7 @@ static void CONSTRUCTOR obuck_init_config(void)
 /*** Internal operations ***/
 
 static void
-obuck_broken(struct obuck *obuck, char *msg, sh_off_t pos)
+obuck_broken(struct obuck *obuck, char *msg, ucw_off_t pos)
 {
   die("Object pool %s corrupted: %s (pos=%llx)", obuck->name, msg, (long long) pos);
 }
@@ -155,7 +157,7 @@ obuck_relock_read_to_scan(struct obuck *obuck)
 struct fb_bucket {
   struct fastbuf fb;
   struct obuck *obuck;
-  sh_off_t start_pos;
+  ucw_off_t start_pos;
   uns bucket_size;
   byte buffer[0];
 };
@@ -179,8 +181,8 @@ obuck_fb_refill(struct fastbuf *f)
     return 0;
   f->buffer = FB_BUCKET(f)->buffer;	/* Could have been trimmed by bdirect_read_commit_modified() */
   bufsize = f->bufend - f->buffer;
-  sh_off_t start = FB_BUCKET(f)->start_pos;
-  sh_off_t pos = start + sizeof(struct obuck_header) + f->pos;
+  ucw_off_t start = FB_BUCKET(f)->start_pos;
+  ucw_off_t pos = start + sizeof(struct obuck_header) + f->pos;
   if (remains <= bufsize)
     {
       datasize = remains;
@@ -188,7 +190,7 @@ obuck_fb_refill(struct fastbuf *f)
     }
   else
     size = datasize = bufsize;
-  int l = sh_pread(FB_BUCKET(f)->obuck->fd, f->buffer, size, pos);
+  int l = ucw_pread(FB_BUCKET(f)->obuck->fd, f->buffer, size, pos);
   if (l < 0)
     die("Error reading bucket: %m");
   if ((unsigned) l != size)
@@ -205,12 +207,12 @@ obuck_fb_refill(struct fastbuf *f)
 }
 
 static int
-obuck_fb_seek(struct fastbuf *f, sh_off_t pos, int whence)
+obuck_fb_seek(struct fastbuf *f, ucw_off_t pos, int whence)
 {
   ASSERT(whence == SEEK_SET || whence == SEEK_END);
   if (whence == SEEK_END)
     pos += FB_BUCKET(f)->bucket_size;
-  ASSERT(pos >= 0 && pos <= (sh_off_t) FB_BUCKET(f)->bucket_size);
+  ASSERT(pos >= 0 && pos <= (ucw_off_t) FB_BUCKET(f)->bucket_size);
   f->pos = pos;
   return 1;
 }
@@ -220,13 +222,22 @@ obuck_fb_spout(struct fastbuf *f)
 {
   int l = f->bptr - f->buffer;
   char *c = f->buffer;
-
+  ucw_off_t start = FB_BUCKET(f)->start_pos;
+  ucw_off_t pos = start + sizeof(struct obuck_header) + f->pos;
+  struct obuck *obuck = FB_BUCKET(f)->obuck;
+  if (l >= obuck->max_size - pos)
+    {
+      if (ucw_ftruncate(obuck->fd, start) < 0)
+	obuck_broken(obuck, "Maximum bucket file size exceeded and failed to truncate incomplete bucket", start);
+      obuck_broken(obuck, "Maximum bucket file size exceeded", start);
+    }
+  f->pos += l;
   while (l)
     {
-      int z = sh_pwrite(FB_BUCKET(f)->obuck->fd, c, l, FB_BUCKET(f)->start_pos + sizeof(struct obuck_header) + f->pos);
+      int z = ucw_pwrite(obuck->fd, c, l, pos);
       if (z <= 0)
 	die("Error writing bucket: %m");
-      f->pos += z;
+      pos += z;
       l -= z;
       c += z;
     }
@@ -248,33 +259,46 @@ bucket_close(void)
 }
 
 void
-bucket_set_name(byte *name)
+bucket_set_name(char *name)
 {
   CF_JOURNAL_VAR(bucket_file_name);
   bucket_file_name = name;
 }
 
-void
-obuck_init(struct obuck *obuck, byte *name, int writeable)
+static void
+obuck_update_params(struct obuck *obuck)
 {
-  sh_off_t size;
+  u64 max_size = obuck->p.max_size - 4 - OBUCK_ALIGN;
+  if (obuck->p.shake_security)
+    max_size -= obuck->p.shake_buflen;
+  ASSERT(max_size < obuck->p.max_size);
+  max_size = MIN(max_size, (u64)OBUCK_OID_FIRST_SPECIAL << OBUCK_SHIFT);
+  max_size = MIN(max_size, ((u64)1 << ((sizeof(ucw_off_t) * 8) - 1)) - 1);
+  obuck->max_size = max_size;
+}
+
+void
+obuck_init(struct obuck *obuck, char *name, int writeable)
+{
+  ucw_off_t size;
 
   ASSERT(name && *name);
   obuck->p = obuck_params;
   obuck->name = name;
-  obuck->fd = sh_open(name, (writeable ? O_RDWR | O_CREAT : O_RDONLY), 0666);
+  obuck->fd = ucw_open(name, (writeable ? O_RDWR | O_CREAT : O_RDONLY), 0666);
   if (obuck->fd < 0)
     die("Unable to open bucket file %s: %m", obuck->name);
   obuck_lock_read(obuck);
-  size = sh_seek(obuck->fd, 0, SEEK_END);
+  size = ucw_seek(obuck->fd, 0, SEEK_END);
   if (size)
     {
       /* If the bucket pool is not empty, check consistency of its end */
       u32 check;
-      if (sh_pread(obuck->fd, &check, 4, size-4) != 4 ||
+      if (ucw_pread(obuck->fd, &check, 4, size-4) != 4 ||
 	  check != OBUCK_TRAILER)
 	obuck_broken(obuck, "Missing trailer of last object", size - 4);
     }
+  obuck_update_params(obuck);
   obuck_unlock(obuck);
 }
 
@@ -298,7 +322,7 @@ static void
 obuck_get(struct obuck *obuck, oid_t oid, struct obuck_context *ctx)
 {
   ctx->pos = obuck_get_pos(oid);
-  if (sh_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos) != sizeof(ctx->hdr))
+  if (ucw_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos) != sizeof(ctx->hdr))
     obuck_broken(obuck, "Short header read", ctx->pos);
   if (ctx->hdr.magic != OBUCK_MAGIC)
     obuck_broken(obuck, "Missing magic number", ctx->pos);
@@ -340,7 +364,7 @@ obuck_find_next(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 	ctx->pos += obuck_bucket_size(ctx->hdr.length);
       if (!(flags & OBUCK_NO_LOCK))
         obuck_lock_read(obuck);
-      c = sh_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos);
+      c = ucw_pread(obuck->fd, &ctx->hdr, sizeof(ctx->hdr), ctx->pos);
       if (!(flags & OBUCK_NO_LOCK))
         obuck_unlock(obuck);
       if (!c)
@@ -394,14 +418,14 @@ obuck_fetch_oid(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 {
   oid_t oid = ctx->hdr.oid;
   ASSERT(oid < OBUCK_OID_FIRST_SPECIAL);
-  sh_off_t pos = ctx->pos = obuck_get_pos(oid);
+  ucw_off_t pos = ctx->pos = obuck_get_pos(oid);
   uns prefetch = MAX(sizeof(struct obuck_header), obuck->p.prefetch_size);
   struct obuck_fetch_oid_fb *b = xmalloc(sizeof(*b) - sizeof(struct obuck_header) + prefetch);
   bzero(b, sizeof(*b));
   if (!(flags & OBUCK_NO_LOCK))
     obuck_lock_read(obuck);
-  sh_off_t size = sh_pread(obuck->fd, (byte *)&b->buf, prefetch, pos);
-  if (size < (uns)sizeof(b->buf))
+  ucw_off_t size = ucw_pread(obuck->fd, (byte *)&b->buf, prefetch, pos);
+  if (size < (ucw_off_t)sizeof(b->buf))
     obuck_broken(obuck, "Short header read", pos);
   if (b->buf.magic != OBUCK_MAGIC)
     obuck_broken(obuck, "Missing magic number", pos);
@@ -412,8 +436,8 @@ obuck_fetch_oid(struct obuck *obuck, struct obuck_context *ctx, uns flags)
   ctx->hdr = b->buf;
   uns size2 = b->buf.length + 4 + ((uns)(OBUCK_ALIGN - sizeof(struct obuck_header) - b->buf.length - 4) & (OBUCK_ALIGN - 1));
   b = xrealloc(b, sizeof(*b) + size2);
-  sh_off_t rest = size2 + sizeof(struct obuck_header) - size;
-  if (rest > 0 && sh_pread(obuck->fd, ((byte *)&b->buf) + size, rest, pos + size) != rest)
+  ucw_off_t rest = size2 + sizeof(struct obuck_header) - size;
+  if (rest > 0 && ucw_pread(obuck->fd, ((byte *)&b->buf) + size, rest, pos + size) != rest)
     obuck_broken(obuck, "Short read", pos);
   if (!(flags & OBUCK_NO_LOCK))
     obuck_unlock(obuck);
@@ -428,8 +452,8 @@ obuck_fetch_oid(struct obuck *obuck, struct obuck_context *ctx, uns flags)
 oid_t
 obuck_predict_last_oid(struct obuck *obuck)
 {
-  sh_stat_t buf;
-  sh_fstat(obuck->fd, &buf);
+  ucw_stat_t buf;
+  ucw_fstat(obuck->fd, &buf);
   return (oid_t)(buf.st_size >> OBUCK_SHIFT);
 }
 
@@ -439,10 +463,10 @@ obuck_create(struct obuck *obuck)
   ASSERT(!obuck->write_fb);
 
   obuck_lock_append(obuck);
-  sh_off_t start = sh_seek(obuck->fd, 0, SEEK_END);
+  ucw_off_t start = ucw_seek(obuck->fd, 0, SEEK_END);
   if (start & (OBUCK_ALIGN - 1))
     obuck_broken(obuck, "Misaligned file", start);
-  if (start >= ((sh_off_t) OBUCK_OID_FIRST_SPECIAL << OBUCK_SHIFT))
+  if (start >= obuck->max_size)
     obuck_broken(obuck, "Maximum bucket file size exceeded", start);
   obuck->create_hdr.magic = OBUCK_INCOMPLETE_MAGIC;
   obuck->create_hdr.oid = start >> OBUCK_SHIFT;
@@ -485,7 +509,7 @@ obuck_create_end(struct obuck *obuck, struct fastbuf *b, uns type, struct obuck_
   bputl(b, OBUCK_TRAILER);
   bflush(b);
   ASSERT(!((FB_BUCKET(b)->start_pos + sizeof(obuck->create_hdr) + b->pos) & (OBUCK_ALIGN - 1)));
-  if (sh_pwrite(obuck->fd, &obuck->create_hdr, sizeof(obuck->create_hdr), FB_BUCKET(b)->start_pos) != sizeof(obuck->create_hdr))
+  if (ucw_pwrite(obuck->fd, &obuck->create_hdr, sizeof(obuck->create_hdr), FB_BUCKET(b)->start_pos) != sizeof(obuck->create_hdr))
     die("Bucket header update failed: %m");
   obuck_unlock_append(obuck);
   memcpy(hdrp, &obuck->create_hdr, sizeof(obuck->create_hdr));
@@ -499,7 +523,7 @@ obuck_delete(struct obuck *obuck, oid_t oid)
   struct obuck_context ctx;
   obuck_get(obuck, oid, &ctx);
   ctx.hdr.oid = OBUCK_OID_DELETED;
-  sh_pwrite(obuck->fd, &ctx.hdr, sizeof(ctx.hdr), ctx.pos);
+  ucw_pwrite(obuck->fd, &ctx.hdr, sizeof(ctx.hdr), ctx.pos);
   obuck_unlock(obuck);
 }
 
@@ -591,9 +615,9 @@ obuck_slurp_pool(struct obuck *obuck, struct obuck_header *hdrp, oid_t next_oid)
 /*** Shakedown ***/
 
 static inline void
-shake_write(struct obuck *obuck, void *addr, int len, sh_off_t pos)
+shake_write(struct obuck *obuck, void *addr, int len, ucw_off_t pos)
 {
-  int l = sh_pwrite(obuck->fd, addr, len, pos);
+  int l = ucw_pwrite(obuck->fd, addr, len, pos);
   if (l != len)
     {
       if (l < 0)
@@ -607,11 +631,11 @@ static inline void
 shake_sync(struct obuck *obuck)
 {
   if (obuck->p.shake_security > 1)
-    sh_fdatasync(obuck->fd);
+    ucw_fdatasync(obuck->fd);
 }
 
 static void
-shake_write_backup(struct obuck *obuck, sh_off_t bpos, byte *norm_buf, int norm_size, byte *fragment, int frag_size, sh_off_t frag_pos, int more_size)
+shake_write_backup(struct obuck *obuck, ucw_off_t bpos, byte *norm_buf, int norm_size, byte *fragment, int frag_size, ucw_off_t frag_pos, int more_size)
 {
   struct obuck_header *bhdr;
   int boff = 0;
@@ -654,7 +678,7 @@ shake_write_backup(struct obuck *obuck, sh_off_t bpos, byte *norm_buf, int norm_
       while (l < more_size)
 	{
 	  int j = MIN(more_size-l, 65536);
-	  if (sh_pread(obuck->fd, auxbuf, j, frag_pos + frag_size + l) != j)
+	  if (ucw_pread(obuck->fd, auxbuf, j, frag_pos + frag_size + l) != j)
 	    die("obuck_shakedown read error: %m");
 	  shake_write(obuck, auxbuf, j, bpos);
 	  bpos += j;
@@ -664,7 +688,7 @@ shake_write_backup(struct obuck *obuck, sh_off_t bpos, byte *norm_buf, int norm_
 }
 
 static void
-shake_erase(struct obuck *obuck, sh_off_t start, sh_off_t end)
+shake_erase(struct obuck *obuck, ucw_off_t start, ucw_off_t end)
 {
   if (start > end)
     die("shake_erase called with negative length, that's a bug");
@@ -689,16 +713,16 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 {
   byte *buf;						/* Shakedown buffer and its size */
   int buflen = ALIGN_TO(obuck->p.shake_buflen, OBUCK_ALIGN);
-  byte *msg;						/* Error message we will print */
-  sh_off_t rstart, wstart;				/* Original and new position of buffer start */
-  sh_off_t r_bucket_start, w_bucket_start;		/* Original and new position of the current bucket */
+  char *err;						/* Error message we will print */
+  ucw_off_t rstart, wstart;				/* Original and new position of buffer start */
+  ucw_off_t r_bucket_start, w_bucket_start;		/* Original and new position of the current bucket */
   int roff, woff;					/* Orig/new position of the current bucket relative to buffer start */
   int rsize;						/* Number of original bytes in the buffer */
   int l;						/* Raw size of the current bucket */
   int changed = 0;					/* "Something has been altered" flag */
   int wrote_anything = 0;				/* We already did a write to the bucket file */
   struct obuck_header *rhdr, *whdr;			/* Original and new address of header of the current bucket */
-  sh_off_t r_file_size;					/* Original size of the bucket file */
+  ucw_off_t r_file_size;					/* Original size of the bucket file */
   int more;						/* How much does the last bucket overlap the buffer */
 
   buf = xmalloc(buflen);
@@ -707,7 +731,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 
   /* We need to be the only accessor, all the object ID's are becoming invalid */
   obuck_lock_write(obuck);
-  r_file_size = sh_seek(obuck->fd, 0, SEEK_END);
+  r_file_size = ucw_seek(obuck->fd, 0, SEEK_END);
   ASSERT(!(r_file_size & (OBUCK_ALIGN - 1)));
   if (r_file_size >= (0x100000000 << OBUCK_SHIFT) - buflen)
     die("Bucket file is too large for safe shakedown. Shaking down with Bucket.ShakeSecurity=0 will still work.");
@@ -728,7 +752,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
       if (rhdr->magic != OBUCK_MAGIC ||
 	  rhdr->oid != OBUCK_OID_DELETED && rhdr->oid != (oid_t)(r_bucket_start >> OBUCK_SHIFT))
 	{
-	  msg = "header mismatch";
+	  err = "header mismatch";
 	  goto broken;
 	}
       l = obuck_bucket_size(rhdr->length);
@@ -736,7 +760,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 	{
 	  if (rhdr->oid != OBUCK_OID_DELETED)
 	    {
-	      msg = "bucket longer than ShakeBufSize";
+	      err = "bucket longer than ShakeBufSize";
 	      goto broken;
 	    }
 	  /* Empty buckets are allowed to be large, but we need to handle them extra */
@@ -752,7 +776,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 	    }
 	  if (GET_U32((byte *)rhdr + l - 4) != OBUCK_TRAILER)
 	    {
-	      msg = "missing trailer";
+	      err = "missing trailer";
 	      goto broken;
 	    }
 	}
@@ -827,7 +851,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 
       /* And refill the buffer */
       r_bucket_start = rstart+rsize;	/* Also needed for error messages */
-      l = sh_pread(obuck->fd, buf+rsize, MIN(buflen-rsize, r_file_size - r_bucket_start), r_bucket_start);
+      l = ucw_pread(obuck->fd, buf+rsize, MIN(buflen-rsize, r_file_size - r_bucket_start), r_bucket_start);
       DBG("Read  %llx %x (%x inherited)", (long long)r_bucket_start, l, rsize);
       if (l < 0)
 	die("obuck_shakedown read error: %m");
@@ -835,12 +859,12 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 	{
 	  if (!more)
 	    break;
-	  msg = "unexpected EOF";
+	  err = "unexpected EOF";
 	  goto broken;
 	}
       if (l & (OBUCK_ALIGN-1))
 	{
-	  msg = "garbage at the end of file";
+	  err = "garbage at the end of file";
 	  goto broken;
 	}
       rsize += l;
@@ -848,7 +872,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
     }
 
   DBG("Finished at position %llx", (long long) wstart);
-  sh_ftruncate(obuck->fd, wstart);
+  ucw_ftruncate(obuck->fd, wstart);
   shake_sync(obuck);
 
   obuck_unlock(obuck);
@@ -857,7 +881,7 @@ obuck_shakedown(struct obuck *obuck, int (*kibitz)(struct obuck_header *old, oid
 
  broken:
   log(L_ERROR, "Error during object pool shakedown: %s (pos=%lld, id=%x), gathering debris",
-      msg, (long long) r_bucket_start, (uns)(r_bucket_start >> OBUCK_SHIFT));
+      err, (long long) r_bucket_start, (uns)(r_bucket_start >> OBUCK_SHIFT));
   /*
    * We can attempt to clean up the bucket file by erasing everything between the last
    * byte written and the next byte to be read. If the secure mode is switched on, we can
