@@ -1,17 +1,36 @@
 #!/usr/bin/perl
 #	Relevance benchmark of Sherlock search server
 #	(c) 2003, Robert Spalek <robert@ucw.cz>
+#	(c) 2007, Martin Mares <mj@ucw.cz>
 
+use Getopt::Long;
 use strict;
 use warnings;
 
 use lib 'lib/perl5';
 use Sherlock::Query();
+use Data::Dumper;
+
+sub usage
+{
+	die "Usage: cat tests | bin/bench-rel [options]
+Options:
+  --server=<server>:<port>   Ask custom search server or mux (default: localhost:8192)
+  --prefix=<str>             Custom query prefix
+  --debug                    Show debug messages
+";
+}
 
 my $search_svr = 'localhost:8192';
-my $query_prefix = 'show 1..20 db "main" filetype = {"text", "html", "pdf"} context 0 titlelen 0 urls 256 morph 1 spell 0 syn 0 accents 0 sitemax 2';
+my $query_prefix = 'show 1..20 context 0 urls 256 morph 2 spell 2 syn 0 accents 0 sitemax 1';
 my $debug_level = 0;
 
+GetOptions(
+	"server=s" => \$search_svr,
+	"prefix=s" => \$query_prefix,
+	"debug!" => \$debug_level,
+) or usage();
+
 sub debug($$$);
 sub pos2weight($);
 sub normalize_url($);
@@ -39,8 +58,9 @@ foreach (@tests)
 foreach my $query (@tests)
 {
 	my $q = new Sherlock::Query($search_svr);
+	debug($q, $query, "Query");
 	my $res = $q->query("$query_prefix $query");
-	debug($q, $res, "Response on query $query");
+	debug($q, $res, "Response");
 	my @queryurls;
 	if ($res =~ /^\+/)
 	{
@@ -49,15 +69,15 @@ foreach my $query (@tests)
 		foreach my $c (@{$q->{CARDS}})
 		{
 			$i++;
-			my $urlrecU = [ $i, $c->{"Q"}, 'U' ];
-			my $urlrecy = [ $i, $c->{"Q"}, 'y' ];
-			my $urlrecb = [ $i, $c->{"Q"}, 'b' ];
-			foreach my $url (@{$c->{"(U"}})
+			my $urlrecU = [ $i, $c->get("Q"), 'U' ];
+			my $urlrecy = [ $i, $c->get("Q"), 'y' ];
+			my $urlrecb = [ $i, $c->get("Q"), 'b' ];
+			foreach my $url ($c->getarray("(U"))
 			{
 				# concatenate the URL's of the card to the big list
-				my @cardurls = map { (normalize_url($_), $urlrecy); } (@{$url->{"y"}});
-				@queryurls = (@queryurls, (normalize_url($url->{U}), $urlrecU), @cardurls);
-				@queryurls = (@queryurls, (normalize_url($url->{b}), $urlrecb)) if $url->{b};
+				my @cardurls = map { (normalize_url($_), $urlrecy); } ($url->getarray("y"));
+				@queryurls = (@queryurls, normalize_url($url->get('U')), $urlrecU, @cardurls);
+				@queryurls = (@queryurls, normalize_url($url->get('b')), $urlrecb) if $url->get('b');
 			}
 			debug($q, $c, "Card $i");
 		}
@@ -81,7 +101,8 @@ foreach my $query (@tests)
 	my $grade = 0;
 	my $max_grade = 0;
 	my $i = 1;
-	foreach my $url (@exp)
+	my $url;
+	foreach $url (@exp)
 	{
 		$grade += $exp->{$url} * pos2weight($fnd->{$url});
 		$max_grade += $exp->{$url} * pos2weight([$i++]) if $exp->{$url} > 0;
@@ -89,12 +110,12 @@ foreach my $query (@tests)
 	my $rel_grade = $grade / $max_grade;
 	print "Total grade:\t$rel_grade = $grade / $max_grade\n";
 	print "Found URL's:\n";
-	foreach my $url (@fnd)
+	foreach $url (@fnd)
 	{
 		print format_url($fnd, $exp, $url);
 	}
 	print "Expected URL's that have not been found:\n";
-	foreach my $url (@exp)
+	foreach $url (@exp)
 	{
 		print format_url($fnd, $exp, $url) if !$fnd->{$url};
 	}
@@ -119,7 +140,13 @@ sub debug($$$)
 	my ($q, $what, $title) = @_;
 	if ($debug_level) {
 		print "$title: \n" if $title;
-		$q->format(sub { print $_[0]; }, $what);
+		if (ref $what eq "") {
+			print "\t$what\n";
+		} elsif (ref $what eq "Sherlock::Object") {
+			$what->write_indented(\*STDOUT, "\t");
+		} else {
+			print Dumper($what);
+		}
 	}
 }
 
