@@ -8,14 +8,15 @@
 
 #include "sherlock/sherlock.h"
 #include "sherlock/bucket.h"
-#include "lib/fastbuf.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/mempool.h"
-#include "lib/bitarray.h"
-#include "lib/clists.h"
-#include "lib/lizard.h"
-#include "lib/bbuf.h"
+#include "ucw/fastbuf.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/mempool.h"
+#include "ucw/bitarray.h"
+#include "ucw/clists.h"
+#include "ucw/lizard.h"
+#include "ucw/bbuf.h"
+#include "ucw/regex.h"
 #include "sherlock/object.h"
 #include "sherlock/objread.h"
 
