@@ -45,6 +45,7 @@ main(int argc, char **argv)
   int c, sk, querylen, anslen, t;
   byte buf[1024];
   byte *x;
+  timestamp_t timer;
 
   while ((c = getopt(argc, argv, "p:d:t:w:")) >= 0)
     switch (c)
@@ -71,7 +72,7 @@ main(int argc, char **argv)
   querylen = strlen(query);
 
   signal(SIGALRM, timeout);
-  init_timer();
+  init_timer(&timer);
 
   bzero(&sa, sizeof(sa));
   sa.sin_family = AF_INET;
@@ -79,7 +80,7 @@ main(int argc, char **argv)
   phase = "DNS";
   alarm(resolve_timeout);
   if (!(he = gethostbyname(server_name)))
-    result(STATE_CRITICAL, "ERROR: Cannot resolve %s - %m", server_name);
+    result(STATE_CRITICAL, "ERROR: Cannot resolve %s - %s", server_name, hstrerror(h_errno));
   memcpy(&sa.sin_addr, he->h_addr, he->h_length);
 
   phase = "Connect";
@@ -104,7 +105,7 @@ main(int argc, char **argv)
   while (x < buf + anslen && *x != ' ' && *x != '\n')
     x++;
   *x = 0;
-  t = get_timer();
+  t = get_timer(&timer);
   if (buf[0] == '-')
     result(STATE_CRITICAL, "ERROR: %s", buf+1);
   else if (buf[0] == '+')
