@@ -1,10 +1,10 @@
 /*
- *	Sherlock Checking Plugins for Netsaint -- Common Stuff
+ *	Sherlock Checking Plugins for Nagios -- Common Stuff
  *
  *	(c) 2001--2004 Martin Mares <mj@ucw.cz>
  */
 
-/* NetSaint status codes */
+/* Nagios status codes */
 
 #define STATE_CRITICAL  2
 #define STATE_WARNING   1
