@@ -1,6 +1,6 @@
 #!/usr/bin/perl
 #	Dumper of cards (might be specified in 7 ways)
-#	(c) 2003--2004, Robert Spalek <robert@ucw.cz>
+#	(c) 2003--2007, Robert Spalek <robert@ucw.cz>
 #	(c) 2004, Martin Mares <mj@ucw.cz>
 
 use Getopt::Long;
@@ -24,6 +24,7 @@ my $to_stdout = 0;
 my $dump = 1;
 my $resolve = 0;
 my $gatherer = 0;
+my $shepherd = 0;
 my $allbuckets = 0;
 my $withlinks = 0;
 my $withsimilar = 0;
@@ -43,13 +44,14 @@ Options:
 	--nodump	... do not actually dump, but do things around
 	--resolve	... resolve BUCKETNR->ID and CARDID->OID
 	--gatherer	... use v2.6 gatherer database instead of url-list
+	--shepherd	... use v3.11 shepherd database instead of url-list
 Descriptors:
   Bucket-file:
 	--allbuckets
 	--url URL
 	--bucket BUCKETNR
 	--id ID		... stage1 object ID, intervals are also allowed
-	  --withlinks
+	  --withlinks	... documents that point TO this document
 	  --withsimilar
 	  --withmerges
 	  --withcards
@@ -74,6 +76,7 @@ GetOptions(
 	"dump!" => \$dump,
 	"resolve!" => \$resolve,
 	"gatherer!" => \$gatherer,
+	"shepherd!" => \$shepherd,
 	"bucket=s@" => \@buckets,
 	"url=s@" => \@urls,
 	"turl=s@" => \@URLs,
@@ -112,6 +115,8 @@ if ($#urls >= 0)
 	$" = ", "; print STDERR "urls: @urls\n";
 	if ($gatherer) {
 		@buckets = (@buckets, url2bucket(@urls));
+	} elsif ($shepherd) {
+		@buckets = (@buckets, shepurl2bucket(@urls));
 	} else {
 		@bids = (@bids, url2id(@urls));
 	}
@@ -219,6 +224,27 @@ sub url2bucket
 	} split /\n/, $found;
 }
 
+sub shepurl2bucket
+{
+	my @urls = @_;
+	$" = " ";
+	if ($shep_host ne "")
+	{
+		$program .= "ssh $shep_host '(cd $shep_dir; ";
+		$program .= "./bin/shep -l --url @urls";
+		$program .= ")'\n";
+	}
+	else
+	{
+		$program .= "./bin/shep -l --url @urls";
+	}
+	my $found = `$program`;
+	return map {
+		if (/^[0-9a-f]*:[0-9a-f]*\s+([0-9a-f]*)/) { ($1) }
+		else { () }
+	} split /\n/, $found;
+}
+
 sub url2id
 {
 	my @urls = map { "\"$_\"" } @_;
@@ -252,7 +278,7 @@ sub URL2oid
 		{
 			my $c = $q->{CARDS}->[0];
 			if ($c)
-			{ $c->{O} }
+			{ $c->get('O') }
 			else { () }
 		} else {
 			printf STDERR "Cannot resolve $_: $stat\n";
@@ -276,14 +302,17 @@ sub id2card
 sub get_links
 {
 	my ($bid) = @_;
-	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $bid`;
+	my $renum = `./bin/idxdump -f $idxdir/link-graph-goes -m $bid`;
+	$renum =~ /^0*$bid -> ([0-9a-f]*)$/ or return ($bid);
+	my $graph_idx = `./bin/idxdump -f $idxdir/link-graph-index -G $1`;
 	$graph_idx =~ /.* -> ([0-9a-f]*)/ or return ($bid);
+	if ($1 eq "ffffffffff") { return ($bid); }
 	my $target = `./bin/idxdump -f $idxdir/link-graph -g $1`;
 	$target =~ /Vertex ([0-9a-f]*) /
 	and hex $1 == hex $bid
 		or return ($bid);
 	return $bid, map {
-		if (/-> ([0-9a-f]*)($| \[[fr])/)
+		if (/<- ([0-9a-f]*)($| \[[fr])/)
 		{ $1 }
 		else { () }
 	} split /\n/, $target;
@@ -382,8 +411,8 @@ sub translate_oid2id
 		my @urls = `./bin/idxdump -f $idxdir/cards -c $cardnr[0] | grep ^[Uy]`;
 		map {
 			s/^[Uy](.*)\n/$1/;
-			if ($gatherer) {
-				my @bucketnr = url2bucket($_);
+			if ($gatherer or $shepherd) {
+				my @bucketnr = $gatherer ? url2bucket($_) : shepurl2bucket($_);
 				print STDERR "OID $oid <- BUCKETNR $bucketnr[0]\n";
 				search_position("Bucket", "attributes", "a", @bucketnr);
 			} else {
