@@ -5,9 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
+#include "ucw/conf.h"
 
-byte *terminal_charset = "utf-8";
+char *terminal_charset = "utf-8";
 uns line_len = 78;
 
 static struct cf_section dumper_config = {
