@@ -4,5 +4,5 @@
  *	(c) 2001 Robert Spalek <robert@ucw.cz>
  */
 
-extern byte *terminal_charset;
+extern char *terminal_charset;
 extern uns line_len;
