@@ -18,8 +18,13 @@ if ! [ -f bin/daemon-helper -a -f cf/sherlock ] ; then
 fi
 GCONTROL="bin/gcontrol$UCW_CF"
 
+#ifdef CONFIG_SHEPHERD
+DHO="--pid-file=lock/shepherd.pid"
+NAME=shepherd
+#else
 DHO="--pid-file=lock/scheduler.pid --status-file=lock/scheduler.slot"
 NAME=scheduler
+#endif"
 
 case "$1" in
 	start)		echo -n "Starting $NAME..."
@@ -44,9 +49,23 @@ case "$1" in
 				exit 1
 			fi
 			;;
+#ifdef CONFIG_SHEPHERD
+	reload)		echo -n "Reloading $NAME... "
+			bin/shepherd --check-config
+			if bin/daemon-helper --reload $DHO ; then
+				echo "done."
+			elif [ $? == 102 ] ; then
+				echo "not running."
+			else
+				echo "FAILED."
+				exit 1
+			fi
+			;;
+#else
 	reload)		$GCONTROL stop
 			$GCONTROL start
 			;;
+#endif
 	restart)	$GCONTROL stop
 			$GCONTROL start
 			;;
