@@ -7,16 +7,17 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/getopt.h"
-#include "lib/fastbuf.h"
-#include "lib/ff-binary.h"
-#include "lib/chartype.h"
-#include "lib/conf.h"
-#include "lib/url.h"
-#include "lib/unicode.h"
-#include "lib/stkstring.h"
-#include "lib/binsearch.h"
-#include "lib/bbuf.h"
+#include "ucw/getopt.h"
+#include "ucw/fastbuf.h"
+#include "ucw/ff-binary.h"
+#include "ucw/chartype.h"
+#include "ucw/conf.h"
+#include "ucw/url.h"
+#include "ucw/unicode.h"
+#include "ucw/string.h"
+#include "ucw/stkstring.h"
+#include "ucw/binsearch.h"
+#include "ucw/bbuf.h"
 #include "sherlock/object.h"
 #include "sherlock/lizard-fb.h"
 #include "sherlock/tagged-text.h"
@@ -30,6 +31,11 @@
 #include "utils/dumpconfig.h"
 #include "lang/lang.h"
 
+#ifdef CONFIG_IMAGES_SIM
+#include "images/images.h"
+#include "images/signature.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -87,7 +93,7 @@ struct strset_item {
 #define HASH_WANT_FIND
 #define HASH_AUTO_POOL 1024
 #define HASH_TABLE_DYNAMIC
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 struct strset {
 	struct strset_hash_table hash;
@@ -154,7 +160,7 @@ struct u64set_item {
 
 #define GBUF_PREFIX(x) u64set_buf_##x
 #define GBUF_TYPE struct u64set_item
-#include "lib/gbuf.h"
+#include "ucw/gbuf.h"
 
 struct u64set {
 	uns count;
@@ -173,7 +179,7 @@ u64set_init(struct u64set *set)
 #define ASORT_ELT(i) list[i].start
 #define ASORT_SWAP(i, j) do{ struct u64set_item s = list[i]; list[i] = list[j]; list[j] = s; }while(0)
 #define ASORT_EXTRA_ARGS , struct u64set_item *list
-#include "lib/arraysort.h"
+#include "ucw/sorter/array-simple.h"
 
 static inline void
 u64set_commit(struct u64set *set)
@@ -277,7 +283,7 @@ zoek_append(struct zoek_def *defs, byte *c)
 			break;
 		else if (*s == '=')
 		{
-	  		*s = 0;
+			*s = 0;
 			while (strcasecmp(defs->name, c))
 			{
 				defs++;
@@ -299,11 +305,11 @@ get_annot(int id)
 	if (!annotate)
 		return "";
 	if (unlikely(!annot_fb))
-		annot_fb = index_bopen("url-list", O_RDONLY);
+		annot_fb = index_bopen("url-list", O_RDONLY, 0);
 	if (annot_last_id + 1 != id) {
 		if (unlikely(!annot_index_fb))
-			annot_index_fb = index_bopen("url-index", O_RDONLY);
-		bsetpos(annot_index_fb, (sh_off_t)id * BYTES_PER_O);
+			annot_index_fb = index_bopen("url-index", O_RDONLY, 0);
+		bsetpos(annot_index_fb, (ucw_off_t)id * BYTES_PER_O);
 		bsetpos(annot_fb, bgeto(annot_index_fb));
 	}
 	if (!bgets(annot_fb, annot_buf, sizeof(annot_buf)))
@@ -344,8 +350,14 @@ dump_card_attr(u64 id, void *tmp)
 	{
 		bprintf(output, "Attribute %x:\n", (u32) id);
 		bprintf(output, "Card:\t%x\n", a->card);
+#ifdef CONFIG_SITES
+		bprintf(output, "Site:\t%x\n", a->site_id);
+#endif
 #ifdef CONFIG_MERGING_HASHES
 		bprintf(output, "MrgHash:\t%s\n", mh);
+#endif
+#ifdef CONFIG_AREAS
+		bprintf(output, "Area:\t%d\n", a->area);
 #endif
 		bprintf(output, "Weight:\t%d\n", a->weight);
 		bprintf(output, "Flags:\t%s\n", at);
@@ -372,9 +384,17 @@ dump_card_attr(u64 id, void *tmp)
 	{
 		bprintf(output, "%8x %8x %8x %-8s %6d %3d %s %3d %-20s %s\n",
 			(u32) id, a->card,
+#ifdef CONFIG_SITES
+		       a->site_id,
+#else
 		       0,
+#endif
 		       mh,
+#ifdef CONFIG_AREAS
+		       a->area,
+#else
 		       0,
+#endif
 		       a->weight, at,
 #ifdef CONFIG_LASTMOD
 		       a->age,
@@ -406,7 +426,7 @@ static void
 dump_note(u64 id, void *tmp)
 {
 	struct card_note *n = tmp;
-	byte *attrs = "GOIPRA**";
+	byte *attrs = "GOIPRAT*";
 	byte at[9];
 
 	if (!n)
@@ -416,18 +436,31 @@ dump_note(u64 id, void *tmp)
 			byte header[SITE_HASH_SIZE*2+1], format[10];
 			sprintf(format, "%%-%d.%ds", SITE_HASH_SIZE*2, SITE_HASH_SIZE*2);
 			sprintf(header, format, "SiteHash");
-			bprintf(output, "ID         Area CBonus USize  Scn Ook Dyn Mrg %s %s Footprint\n", attrs, header);
+			bprintf(output, "ID         OID      Area CBonus USize  Scn Ook Dyn Mrg %s %s Footprint\n", attrs, header);
 		}
 		return;
 	}
 	str_format_flags(at, attrs, n->flags);
-	bprintf(output, "%8x %6d %6d %6d %3d %3d %3d %3d %s ", (u32) id,
+	bprintf(output, "%8x %8x %6d %6d %6d %3d %3d %3d %3d %s ", (u32) id,
+#ifdef CONFIG_INDEXER_STORE_OID
+	       n->oid,
+#else
 	       0,
+#endif
+#ifdef CONFIG_AREAS
+	       n->area,
+#else
+	       0,
+#endif
 	       n->card_bonus,
 	       n->useful_size,
 	       n->weight_scanner,
 	       0,
+#ifdef CONFIG_WEIGHTS
+	       n->weight_dynamic,
+#else
 	       0,
+#endif
 	       n->weight_merged,
 	       at);
 	for (int i=0; i<SITE_HASH_SIZE; i++)
@@ -462,22 +495,41 @@ dump_card_info(u64 id, void *tmp)
 	bprintf(output, "         %8x\n", info->orig_card);
 }
 
+static u64
+dump_admin_export_header(struct fastbuf *f)
+{
+	struct odes *o = obj_new(cf_pool);
+	if (obj_read(f, o) <= 0)
+		die("Corrupted export header");
+	if (verbose)
+	{
+		bprintf(output, "Params:\n");
+		bput_object(output, o);
+		bputc(output, '\n');
+	}
+	return btell(f);
+}
+
 static void
 dump_admin_export(u64 id, void *tmp)
 {
 	struct admin_export *exp = tmp;
-	byte *attrs = "GOIPR***", at[9];
+	byte *note_flags = "GOIPRAT*", *flags = "EADMF123", note_flags_buf[9], flags_buf[9];
 	if (!exp)
 	{
-		dump_card_attr(0, NULL);
 		if (!bare)
-			bprintf(output, "Footprint                         Idx %s\n", attrs);
+			bprintf(output, "ID       Footprint                         Idx Id       " \
+			    "%s %s Scn Ook Dyn Mrg Wei TF Lng Typ\n", flags, note_flags);
 		return;
 	}
-	str_format_flags(at, attrs, exp->flags);
-	dump_card_attr(id, &exp->attr);
+	str_format_flags(flags_buf, flags, exp->flags);
+	str_format_flags(note_flags_buf, note_flags, exp->note_flags);
+	bprintf(output, "%08x ", (u32)id);
 	dump_footprint(output, exp->footprint);
-	bprintf(output, " %3u %s\n", exp->subindex + 1, at);
+	bprintf(output, " %3u %08x %s %s %3d %3d %3d %3d %3d %02x %3d %3d\n",
+	    exp->subindex + 1, exp->id, flags_buf, note_flags_buf,
+	    exp->weight_scanner, exp->weight_oook, exp->weight_dynamic,
+	    exp->weight_merged, exp->weight, exp->file_flags, exp->file_lang, exp->file_type);
 }
 
 static void
@@ -553,6 +605,12 @@ dump_fingerprint(u64 id, void *tmp)
 	}
 }
 
+static struct u64set zoek_signature_card_id;
+static struct zoek_def signature_zoeks[] = {
+	{ "CardID",	&zoek_signature_card_id,	ZOEK_U64SET },
+	{ NULL }
+};
+
 static void
 dump_signatures(u64 id, void *tmp)
 {
@@ -564,6 +622,8 @@ dump_signatures(u64 id, void *tmp)
 			bprintf(output, "Pos      CardID   Signatures\n");
 		return;
 	}
+	if (!u64set_find(&zoek_signature_card_id, *c))
+		return;
 	if (verbose)
 	{
 		bprintf(output, "Checksum at %x:\n", (u32) id);
@@ -663,7 +723,9 @@ dump_card(u64 start, void *tmp)
 		}
 	}
 	while (btell(f) & ((1 << CARD_POS_SHIFT) - 1))
-		bgetc(f);
+		if (bgetc(f) < 0)
+			break;
+	bputc(output, '\n');
 }
 
 static byte *wt_names[8] = { WORD_TYPE_USER_NAMES };
@@ -694,7 +756,7 @@ dump_single_label(struct parsed_attr *pa)
 {
 	byte *p = pa->val, *end = pa->val + pa->len;
 	bprintf(output, "\t%c", pa->attr);
-	if (raw)
+	if (raw || pa->attr == 'H')
 		bwrite(output, p, pa->len);
 	else while (p < end)
 	{
@@ -738,7 +800,7 @@ dump_labels(u64 start, void *tmp)
 		u64set_find(&zoek_labels_redir_id, label.redir_id)); 
 	if (!filtered)
 	{
-		bprintf(output, "Label at %08qx:\n", (long long) start);
+		bprintf(output, "Label at %08llx:\n", (long long) start);
 		bprintf(output, "Merged ID:\t%08x\n", label.merged_id); // Stage2 ID
 		bprintf(output, "URL ID:\t\t%08x\n", label.url_id);
 		bprintf(output, "Redirect ID:\t%08x\n", label.redir_id);
@@ -746,7 +808,7 @@ dump_labels(u64 start, void *tmp)
 		bprintf(output, "Flags:\t%s\n", str_format_flags(fl, label_flags, label.flags));
 		bprintf(output, "Labels:\n");
 	}
-	sh_off_t stop = btell(f) + label.count;
+	ucw_off_t stop = btell(f) + label.count;
 	get_attr_set_type(BUCKET_TYPE_V33);
 	while (btell(f) < stop)
 	{
@@ -778,7 +840,7 @@ dump_labels_id(u64 start, void *tmp)
 	byte fl[9];
 	str_format_flags(fl, label_flags, bgetc(f));
 	if (!filtered)
-		bprintf(output, "Label block at %08qx for URL %08x (%s)\n", (long long) start, orig_id, fl);
+		bprintf(output, "Label block at %08llx for URL %08x (%s)\n", (long long) start, orig_id, fl);
 	get_attr_set_type(BUCKET_TYPE_V33);
 	while (1)
 	{
@@ -806,7 +868,7 @@ static struct zoek_def lex_zoeks[] = {
 static void
 dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 {
-	sh_off_t ref_pos = GET_O(l->ref_pos);
+	ucw_off_t ref_pos = GET_O(l->ref_pos);
 	uns chlen = GET_U16(l->ch_len) << 12;
 	id |= l->class;
 #ifdef CONFIG_CONTEXTS
@@ -828,7 +890,7 @@ dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 	if (verbose)
 	{
 		bprintf(output, "Word ID %x:\n", id);
-		bprintf(output, "References:\t%08qx + %07x\n", (long long) ref_pos, chlen);
+		bprintf(output, "References:\t%08llx + %07x\n", (long long) ref_pos, chlen);
 		bprintf(output, "Class:\t\t%s\n", (l->class > 7 ? "????" : word_classes[l->class]));
 		bprintf(output, "Frequency:\t%d\n", freq);
 		bprintf(output, "Context:\t%04x\n", ctxt);
@@ -837,7 +899,7 @@ dump_lex_entry(struct lex_entry *l, uns id, struct fastbuf *f)
 	}
 	else
 	{
-		bprintf(output, "%8x %8qx+%07x %s %3d %04x %3d ", id, (long long) ref_pos, chlen,
+		bprintf(output, "%8x %8llx+%07x %s %3d %04x %3d ", id, (long long) ref_pos, chlen,
 		       (l->class > 7 ? "????" : word_classes[l->class]), freq, ctxt, l->length);
 	}
 	bwrite(output, word, l->length);
@@ -956,7 +1018,7 @@ dump_lex_temp(u64 pos, void *tmp)
 	if (verbose)
 	{
 		bprintf(output, "Record %x:\n", (uns) pos);
-		bprintf(output, "Word ID %x:\n", id);
+		bprintf(output, "Word ID %x:\n", id >> 3);
 		bprintf(output, "Count:\t\t%d\n", count);
 		bprintf(output, "Word class:\t\t%s\n", word_classes[class]);
 		bprintf(output, "Ctxt class:\t\t%d\n", context);
@@ -965,7 +1027,7 @@ dump_lex_temp(u64 pos, void *tmp)
 	}
 	else
 	{
-		bprintf(output, "%8x %8x %10d %s %04x %3d ", (uns) pos, id, count, word_classes[class], context, length);
+		bprintf(output, "%8x %8x %10d %s %04x %3d ", (uns) pos, id >> 3, count, word_classes[class], context, length);
 	}
 	bwrite(output, word, length);
 	bprintf(output, verbose ? "\n\n" : "\n");
@@ -981,8 +1043,8 @@ dump_lex_stats(u64 start, void *tmp)
 
 	enum wtype { TOTAL, BARE, ACCENTED, DIGITS, MIXED, NTYPES };
 	byte *tnames[] = { "Total", "Bare", "Accented", "Numbers", "Mixed" };
-	uns wcnt[MAX_WORD_LEN+1][NTYPES];
-	uns wlen[MAX_WORD_LEN+1][NTYPES];
+	uns wcnt[MAX_WORD_BYTES+1][NTYPES];
+	uns wlen[MAX_WORD_BYTES+1][NTYPES];
 	uns cnt = bgetl(f);
 	bzero(wcnt, sizeof(wcnt));
 	bzero(wlen, sizeof(wlen));
@@ -993,8 +1055,8 @@ dump_lex_stats(u64 start, void *tmp)
 		bget_context(f);
 		uns len = bgetc(f);
 		uns reclen = 9 + len;
-		ASSERT(len && len <= MAX_WORD_LEN);
-		byte buf[MAX_WORD_LEN];
+		ASSERT(len && len <= MAX_WORD_BYTES);
+		byte buf[MAX_WORD_BYTES];
 		bread(f, buf, len);
 		int alpha = 0;
 		int digits = 0;
@@ -1002,7 +1064,7 @@ dump_lex_stats(u64 start, void *tmp)
 		for (byte *x=buf; x<buf+len;)
 		{
 			uns c;
-			GET_UTF8(x, c);
+			x = utf8_get(x, &c);
 			if (c >= '0' && c <= '9')
 				digits++;
 			else
@@ -1037,7 +1099,7 @@ dump_lex_stats(u64 start, void *tmp)
 	for (uns j=0; j<NTYPES; j++)
 		bprintf(output, " %11s Cnt/Len", tnames[j]);
 	bputc(output, '\n');
-	for (uns i=0; i<=MAX_WORD_LEN; i++)
+	for (uns i=0; i<=MAX_WORD_BYTES; i++)
 	{
 		if (!i)
 			bprintf(output, "Sum:");
@@ -1060,7 +1122,7 @@ dump_graph(u64 start, void *tmp)
 	if (!f)
 		return;
 	bget_graph_hdr(f, &src, &deg);
-	bprintf(output, "Vertex %x (degree %d) at %08qx:\n", src, deg, (long long) start);
+	bprintf(output, "Vertex %x (degree %d) at %08llx:\n", src, deg, (long long) start);
 	while (deg--)
 	{
 		u32 x = bgetl(f);
@@ -1076,8 +1138,8 @@ dump_graph_index(u64 start, void *tmp)
 	struct fastbuf *f = tmp;
 	if (!f)
 		return;
-	sh_off_t ofs = bgeto(f);
-	bprintf(output, "%08x -> %010qx\n", (uns) start / BYTES_PER_O, (long long) ofs);
+	ucw_off_t ofs = bgeto(f);
+	bprintf(output, "%08x -> %010llx\n", (uns) start / BYTES_PER_O, (long long) ofs);
 }
 
 static void
@@ -1095,8 +1157,8 @@ dump_graph_vertex(struct fastbuf *b, int argc, char **argv)
 			if (vertex == 0xffffffff)
 				die("No such vertex in %s", fb_goes->name);
 		}
-		bsetpos(fb_index, BYTES_PER_O * (sh_off_t)vertex);
-		sh_off_t pos = bgeto(fb_index);
+		bsetpos(fb_index, BYTES_PER_O * (ucw_off_t)vertex);
+		ucw_off_t pos = bgeto(fb_index);
 		if (verbose)
 			bprintf(output, "# Translated vertex %x at position %llx\n", vertex, (long long)pos);
 		bsetpos(b, pos);
@@ -1138,7 +1200,7 @@ dump_merges(u64 id, void *tmp)
 
 static struct u64set zoek_u32_value;
 static struct zoek_def u32_zoeks[] = {
-	{ "Class",	&zoek_u32_value,	ZOEK_U64SET },
+	{ "Value",	&zoek_u32_value,	ZOEK_U64SET },
 	{ NULL }
 };
 
@@ -1193,7 +1255,7 @@ dump_refchain(struct fastbuf *f)
 	if (verbose)
 		bprintf(output, "OID:\t%08x\nLength:\t%d\n\t", oid, len);
 	else
-		bprintf(output, "%8qx %8x %8d ", (long long) pos, oid, len);
+		bprintf(output, "%8llx %8x %8d ", (long long) pos, oid, len);
 	uns last_pos = 0;
 	while (len)
 	{
@@ -1400,7 +1462,7 @@ dump_string_index(u64 start, void *tmp)
 	breadb(f, &size, sizeof(u32));
 	if (verbose)
 	{
-		bprintf(output, "String index entry at %08qx:\n", (long long) start);
+		bprintf(output, "String index entry at %08llx:\n", (long long) start);
 		bprintf(output, "Finger:\t");
 		for (i=0; i<12; i++)
 			bprintf(output, "%02x", fp.hash[i]);
@@ -1409,7 +1471,7 @@ dump_string_index(u64 start, void *tmp)
 	}
 	else
 	{
-		bprintf(output, "S%8qx ", (long long) start);
+		bprintf(output, "S%8llx ", (long long) start);
 		for (i=0; i<12; i++)
 			bprintf(output, "%02x", fp.hash[i]);
 		bprintf(output, " %8d\n", size);
@@ -1419,12 +1481,18 @@ dump_string_index(u64 start, void *tmp)
 	bprintf(output, "\n");
 }
 
+static struct strset zoek_string_map_fp;
+static struct zoek_def string_map_zoeks[] = {
+	{ "FP",		&zoek_string_map_fp,	ZOEK_STRSET },
+	{ NULL }
+};
+
 static void
 dump_string_map(u64 start, void *tmp)
 {
 	struct fastbuf *f = tmp;
 	struct fingerprint fp;
-	sh_off_t ref_pos;
+	ucw_off_t ref_pos;
 	uns i;
 
 	if (!f)
@@ -1435,22 +1503,20 @@ dump_string_map(u64 start, void *tmp)
 	}
 	breadb(f, &fp, sizeof(struct fingerprint));
 	ref_pos = bgeto(f);
+	byte fp_str[25];
+	for (i=0; i<12; i++)
+		sprintf(fp_str + i * 2, "%02x", fp.hash[i]);
+	fp_str[24] = 0;
+	if (!strset_find(&zoek_fingerprint_fp, fp_str))
+		return;
 	if (verbose)
 	{
-		bprintf(output, "String map entry at %08qx:\n", (long long) start);
-		bprintf(output, "Finger:\t");
-		for (i=0; i<12; i++)
-			bprintf(output, "%02x", fp.hash[i]);
-		bprintf(output, "\n");
-		bprintf(output, "RefPos:\t%08qx\n\n", (long long) ref_pos);
+		bprintf(output, "String map entry at %08llx:\n", (long long) start);
+		bprintf(output, "Finger:\t%s\n", fp_str);
+		bprintf(output, "RefPos:\t%08llx\n\n", (long long) ref_pos);
 	}
 	else
-	{
-		bprintf(output, "%8qx ", (long long) start);
-		for (i=0; i<12; i++)
-			bprintf(output, "%02x", fp.hash[i]);
-		bprintf(output, " %08qx\n", (long long) ref_pos);
-	}
+		bprintf(output, "%8llx %s %08llx\n", (long long) start, fp_str, (long long) ref_pos);
 }
 
 static struct u64set zoek_word_index_word_id;
@@ -1476,13 +1542,13 @@ dump_word_index(u64 start, void *tmp)
 	}
 	if (verbose)
 	{
-		bprintf(output, "Word index entry at %08qx:\n", (long long) start);
+		bprintf(output, "Word index entry at %08llx:\n", (long long) start);
 		bprintf(output, "Word:\t%x\n", wordid);
 		bprintf(output, "Size:\t%d\n", size);
 	}
 	else
 	{
-		bprintf(output, "S%8qx %8x %8d\n", (long long) start, wordid, size);
+		bprintf(output, "S%8llx %8x %8d\n", (long long) start, wordid, size);
 	}
 	while (size > 0)
 		size -= dump_refchain(f);
@@ -1574,6 +1640,93 @@ dump_feedback_gath(u64 id, void *tmp)
 	bprintf(output, " %08x %s %d\n", f->cardid, at, f->weight);
 }
 
+#ifdef CONFIG_IMAGES_SIM
+#define IMAGE_CLUSTER_SIZE sizeof(struct image_cluster)
+
+static u64
+dump_image_signatures_unsorted_header(struct fastbuf *f)
+{
+	uns count = bgetl(f);
+	if (verbose)
+		bprintf(output, "Count: %u\n\n", count);
+	return 4;
+}
+
+static void
+dump_image_signatures(u64 start, void *tmp)
+{
+	struct fastbuf *f = tmp;
+	uns oid;
+	struct image_signature sig;
+	int len;
+
+	if (!f)
+	{
+		if (!verbose && !bare)
+			bprintf(output, "Pos              OID      Len Fl  Cols  Rows  DF    DH   L   u   v  LH  HL  HH\n");
+		return;
+	}
+
+	if ((oid = bgetl(f)) == ~0U)
+		return;
+	if ((len = bpeekc(f)) < 0)
+		return;
+	breadb(f, &sig, image_signature_size(len));
+	if (!verbose)
+		bprintf(output, "%-16Lx %-8x %3u %02x %5u %5u %3u %5u %3u %3u %3u %3u %3u %3u\n",
+			(long long)start, oid, sig.len, sig.flags, sig.cols, sig.rows, sig.df, sig.dh,
+			sig.vec.f[0], sig.vec.f[1], sig.vec.f[2],
+			sig.vec.f[3], sig.vec.f[4], sig.vec.f[5]);
+	else
+	{
+		bprintf(output, "Pos=%-16Lx OID=%-8x Len=%-3u Fl=%02x  w=%-5u h=%-5u DF=%-3u DH=%-5u "
+			"L=%-3u u=%-3u v=%-3u LH=%-3u HL=%-3u HH=%-3u\n",
+			(long long)start, oid, sig.len, sig.flags, sig.cols, sig.rows, sig.df, sig.dh,
+			sig.vec.f[0], sig.vec.f[1], sig.vec.f[2],
+			sig.vec.f[3], sig.vec.f[4], sig.vec.f[5]);
+		for (uns i = 0; i < sig.len; i++)
+			bprintf(output, "                     region #%-3u  wa=%-3u wb=%-3u I1=%-3u I2=%-3u I3=%-3u X=%-3u Y=%-3u "
+				"L=%-3u u=%-3u v=%-3u LH=%-3u HL=%-3u HH=%-3u\n", i,
+				sig.reg[i].wa, sig.reg[i].wb,
+				sig.reg[i].h[0], sig.reg[i].h[1], sig.reg[i].h[2],
+				sig.reg[i].h[3], sig.reg[i].h[4],
+				sig.reg[i].f[0], sig.reg[i].f[1], sig.reg[i].f[2],
+				sig.reg[i].f[3], sig.reg[i].f[4], sig.reg[i].f[5]);
+		bputc(output, '\n');
+	}
+}
+
+static uns image_clusters_depth;
+
+static u64
+dump_image_clusters_header(struct fastbuf *f)
+{
+	image_clusters_depth = bgetl(f);
+	if (verbose)
+		bprintf(output, "Depth=%u\n\n", image_clusters_depth);
+	return 4;
+}
+
+static void
+dump_image_clusters(u64 id, void *tmp)
+{
+	struct image_cluster *clus = tmp;
+	if (!tmp)
+	{
+		if (!bare)
+			bprintf(output, "Id                Pos/Dot    L    u    v   LH   HL   HH\n");
+		return;
+	}
+	bprintf(output, "%-8x ", (uns)id);
+	if (id < (1U << (image_clusters_depth - 1)) - 1)
+		bprintf(output, "%16d %4d %4d %4d %4d %4d %4d\n", clus->dot,
+			clus->vec[0], clus->vec[1], clus->vec[2],
+			clus->vec[3], clus->vec[4], clus->vec[5]);
+	else
+		bprintf(output, "%16Lx\n", (long long)clus->pos);
+
+}
+#else
 #define IMAGE_CLUSTER_SIZE 0
 
 static u64
@@ -1599,6 +1752,7 @@ dump_image_clusters(u64 id UNUSED, void *tmp UNUSED)
 {
 	ASSERT(0);
 }
+#endif
 
 struct ule {
 	uns pos;
@@ -1610,7 +1764,7 @@ struct ule {
 #define HASH_KEY_ENDSTRING x
 #define HASH_WANT_FIND
 #define HASH_WANT_NEW
-#include "lib/hashtable.h"
+#include "ucw/hashtable.h"
 
 static void
 lookup_urls(struct fastbuf *b, int argc, char **argv)
@@ -1664,6 +1818,20 @@ lookup_urls(struct fastbuf *b, int argc, char **argv)
 	}
 }
 
+static void
+lookup_string_fp(int argc, char **argv)
+{
+	while (argc--)
+	{
+		struct fingerprint fp;
+		fingerprint(*argv, &fp);
+		for (uns i=0; i<12; i++)
+			bprintf(output, "%02x", fp.hash[i]);
+		bprintf(output, " %s\n", *argv);
+		argv++;
+	}
+}
+
 static void
 lookup_url_key(int argc, char **argv)
 {
@@ -1686,7 +1854,7 @@ lookup_url_key(int argc, char **argv)
 
 struct index_file {
 	int option;
-	byte **default_filename;
+	char **default_filename;
 	int record_size;
 		/*
 		 * record_size > 0:	uniform sequence with records of size record_size
@@ -1698,14 +1866,15 @@ struct index_file {
 	struct zoek_def *zoek;
 };
 
-static byte *fn_graph_obj_index, *fn_graph_skel_index;
-static byte *fn_graph_obj_real;
-static byte *fn_graph_obj_goes;
-static byte *fn_graph_obj_deg;
-static byte *fn_rank = "rank-obj";
+static char *fn_graph_obj_index, *fn_graph_skel_index;
+static char *fn_graph_obj_real;
+static char *fn_graph_obj_goes;
+static char *fn_graph_obj_deg;
+static char *fn_rank = "rank-obj";
 
 enum long_opt {
 	F_FIRST = 1000,
+	F_FP_SPLITS,
 	F_LINK_GRAPH_DEG,
 	F_LINK_GRAPH_GOES,
 	F_LINK_GRAPH_INDEX,
@@ -1713,6 +1882,7 @@ enum long_opt {
 	F_SKEL_GRAPH,
 	F_SKEL_GRAPH_INDEX,
 	F_SKEL_GRAPH_VERTEX,
+	F_STRING_FP,
 	F_URL_KEY,
 	F_URL_INDEX,
 	F_ADMIN_EXPORT,
@@ -1720,12 +1890,13 @@ enum long_opt {
 
 static struct index_file index_files[] = {
 	// option		file name		record size			record dumper		header	zoek
-	{ 'a',			&fn_attributes, 	sizeof(struct card_attr),	dump_card_attr,		NULL,	NULL },
+	{ 'a',			&fn_attributes,		sizeof(struct card_attr),	dump_card_attr,		NULL,	NULL },
 	{ 'c',			&fn_cards,		-(1 << CARD_POS_SHIFT),		dump_card,		NULL,	NULL },
 	{ 'd',			&fn_card_attrs,		sizeof(struct card_attr),	dump_card_attr,		NULL,	NULL },
 	{ 'I',			&fn_card_info,		sizeof(struct card_info),	dump_card_info,		NULL,	card_info_zoeks },
 	{ 'h',			&fn_checksums,		sizeof(struct csum),		dump_checksum,		NULL,	NULL },
 	{ 'F',			&fn_fingerprints,	sizeof(struct card_print),	dump_fingerprint,	NULL,	fingerprint_zoeks },
+	{ F_FP_SPLITS,		&fn_fp_splits,		4,				dump_u32,		NULL,	u32_zoeks },
 	{ 'l',			&fn_labels,		0,				dump_labels,		NULL,	labels_zoeks },
 	{ 'L',			&fn_labels_by_id,	0,				dump_labels_id,		NULL,	labels_id_zoeks },
 	{ 'x',			&fn_lexicon,		0,				dump_lexicon,		NULL,	lex_zoeks },
@@ -1743,7 +1914,7 @@ static struct index_file index_files[] = {
 	{ F_LINK_GRAPH_REAL,	&fn_graph_obj_real,	4,				dump_u32,		NULL,	NULL },
 	{ 'G',			&fn_graph_obj,		0,				NULL /* kludge */,	NULL,	NULL },
 	{ 'm',			&fn_merges,		sizeof(u32),			dump_merges,		NULL,	merges_zoeks },
- 	{ 'o',			&fn_rank,		sizeof(float),			dump_float,		NULL,	NULL },
+	{ 'o',			&fn_rank,		sizeof(float),			dump_float,		NULL,	NULL },
 	{ 'n',			&fn_notes,		sizeof(struct card_note),	dump_note,		NULL,	NULL },
 	{ 'N',			&fn_notes_skel,		sizeof(struct card_note),	dump_note,		NULL,	NULL },
 	{ 'p',			&fn_parameters,		sizeof(struct index_params),	dump_params,		NULL,	NULL },
@@ -1751,12 +1922,13 @@ static struct index_file index_files[] = {
 	{ 'r',			&fn_references,		0,				dump_refs,		NULL,	NULL },
 	{ 't',			&fn_ref_texts,		0,				dump_ref_texts,		NULL,	NULL },
 	{ 's',			&fn_sites,		sizeof(struct site_mapping),	dump_sites,		NULL,	NULL },
-	{ 'U',			&fn_signatures,		0, /* computed run-time */	dump_signatures,	NULL,	NULL },
+	{ 'U',			&fn_signatures,		0, /* computed run-time */	dump_signatures,	NULL,	signature_zoeks },
 	{ 'y',			&fn_stems,		0,				dump_stems,		NULL,	NULL },
 	{ 'Y',			&fn_stems_ordered,	0,				dump_stems_temp,	NULL,	NULL },
 	{ 'H',			&fn_string_hash,	sizeof(u32),			dump_u32,		NULL,	u32_zoeks },
 	{ 'X',			&fn_string_index,	0,				dump_string_index,	NULL,	NULL },
-	{ 'M',			&fn_string_map,		0,				dump_string_map,	NULL,	NULL },
+	{ 'M',			&fn_string_map,		0,				dump_string_map,	NULL,	string_map_zoeks },
+	{ F_STRING_FP,		NULL,			0,				NULL /* kludge*/,	NULL,	NULL },
 	{ 'u',			&fn_urls,		PASS_POSITION,			dump_urls,		NULL,	urls_zoeks },
 	{ 'q',			&fn_urls,		0,				NULL /* kludge */,	NULL,	NULL },
 	{ 'w',			&fn_word_index,		0,				dump_word_index,	NULL,	word_index_zoeks },
@@ -1769,8 +1941,8 @@ static struct index_file index_files[] = {
 	{ F_SKEL_GRAPH_VERTEX,	&fn_graph_skel,		0,				NULL /* kludge */,	NULL,	NULL },
 	{ F_URL_INDEX,		&fn_url_index,		BYTES_PER_O,			dump_url_index,		NULL,	NULL },
 	{ F_URL_KEY,		NULL,			0,				NULL /* kludge */,	NULL,	NULL },
-	{ F_ADMIN_EXPORT,	&fn_admin_export,	sizeof(struct admin_export),	dump_admin_export,	NULL,	NULL },
-	{ 0,  			NULL,			0,				NULL,			NULL,	NULL }
+	{ F_ADMIN_EXPORT,	&fn_admin_export,	sizeof(struct admin_export),	dump_admin_export,	dump_admin_export_header,	NULL },
+	{ 0,			NULL,			0,				NULL,			NULL,	NULL }
 };
 
 static char *shortopts = "0abcdef:ghi:klmnopqrstuvwxyz:ABDEFGHIKLMNOQPRTUWXY" CF_SHORT_OPTS;
@@ -1792,6 +1964,7 @@ static struct option longopts[] =
 	{ "card-info",		0, 0, 'I' },
 	{ "checksum",		0, 0, 'h' },
 	{ "finger",		0, 0, 'F' },
+	{ "fp-splits",		0, 0, F_FP_SPLITS },
 	{ "float",		0, 0, 'o' },
 	{ "label",		0, 0, 'l' },
 	{ "labels-id",		0, 0, 'L' },
@@ -1824,6 +1997,7 @@ static struct option longopts[] =
 	{ "string-hash",	0, 0, 'H' },
 	{ "string-index",	0, 0, 'X' },
 	{ "string-map",		0, 0, 'M' },
+	{ "string-fp",		0, 0, F_STRING_FP },
 	{ "url",		0, 0, 'u' },
 	{ "url-index",		0, 0, F_URL_INDEX },
 	{ "url-key",		0, 0, F_URL_KEY },
@@ -1864,6 +2038,7 @@ Index files:\n\
 -h, --checksum\t\tChecksums\n\
 -B, --feedback-gath\tFeedback to the gatherer\n\
 -F, --finger\t\tFingerprints (--annotate -z FP=<strset> -z CardID=<intset>)\n\
+    --fp-splits\t\tFP splitting tree (-z Value=<intset>)\n\
 -o, --float\t\tRank vectors\n\
 -D, --image-clusters\tClusters search tree\n\
 -e, --image-signatures\tImage signatures\n\
@@ -1893,13 +2068,14 @@ Index files:\n\
     --skel-graph\tSkeleton link graph\n\
     --skel-graph-index\t... vertex -> position\n\
     --skel-graph-vertex\t... find a given vertex\n\
--U, --signature\t\tSignatures\n\
+-U, --signature\t\tSignatures (-z CardID=<intset>)\n\
 -s, --site\t\tSites\n\
 -y, --stems\t\tStem mappings\n\
 -Y, --stems-ordered\tTemporary stem mappings from lexorder\n\
 -H, --string-hash\tString hash (-z Value=<intset>)\n\
 -X, --string-index\tString index\n\
--M, --string-map\tString map\n\
+-M, --string-map\tString map (-z FP=<strset>)\n\
+    --string-fp\tCalculate fingerprint for a given string\n\
 -u, --url\t\tUrl list (-z URL=<strset>)\n\
     --url-index\t\tUrl index (--annotate)\n\
     --url-key\t\tCalculate URL key and fingerprint for a given URL\n\
@@ -1924,7 +2100,7 @@ dump_index_interval(struct index_file *f, struct fastbuf *b, u64 start, u64 stop
 {
 	if (f->record_size == (int) PASS_POSITION)
 	{
-		sh_off_t size = bfilesize(b);
+		ucw_off_t size = bfilesize(b);
 		f->dump_single(PASS_POSITION, b);		// startup
 		for (u64 pos=start; pos<=stop; pos++)
 		{
@@ -1990,6 +2166,9 @@ dump_index_file(struct index_file *f, struct fastbuf *b, int argc, char **argv)
 		/* As have the graph lookup functions */
 		dump_graph_vertex(b, argc, argv);
 		return;
+	case F_STRING_FP:
+		lookup_string_fp(argc, argv);
+		return;
 	case F_URL_KEY:
 		lookup_url_key(argc, argv);
 		return;
