@@ -5,8 +5,9 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lfs.h"
-#include "lib/md5.h"
+#include "ucw/lfs.h"
+#include "ucw/md5.h"
+#include "ucw/string.h"
 #include "sherlock/index.h"
 #include "indexer/params.h"
 
@@ -17,7 +18,7 @@
 int
 main(int argc, char **argv)
 {
-  struct MD5Context mc;
+  md5_context mc;
   struct index_params para;
   int fd;
 
@@ -51,25 +52,24 @@ main(int argc, char **argv)
       return 1;
     }
   close(fd);
-  MD5Init(&mc);
-  MD5Update(&mc, (byte*)&para, sizeof(para));
+  md5_init(&mc);
+  md5_update(&mc, (byte*)&para, sizeof(para));
 
   for (int i=2; i<argc; i++)
     {
-      fd = sh_open(argv[i], O_RDONLY, 0);
+      fd = ucw_open(argv[i], O_RDONLY, 0);
       if (fd < 0)
 	{
 	  printf("<missing-%s>\n", argv[i]);
 	  return 1;
 	}
-      sh_off_t len = sh_seek(fd, 0, SEEK_END);
-      MD5Update(&mc, (byte*)&len, sizeof(len));
+      ucw_off_t len = ucw_seek(fd, 0, SEEK_END);
+      md5_update(&mc, (byte*)&len, sizeof(len));
       close(fd);
     }
 
-  byte digest[MD5_SIZE], hex[MD5_HEX_SIZE];
-  MD5Final(digest, &mc);
-  md5_to_hex(digest, hex);
+  byte hex[MD5_HEX_SIZE];
+  mem_to_hex(hex, md5_final(&mc), MD5_SIZE, MEM_TO_HEX_UPCASE);
   printf("%08x-%.16s\n", para.ref_time, hex);
 
   return 0;
