@@ -1,7 +1,7 @@
 #!/usr/bin/perl
 #
 #	Sherlock Query Sender
-#	(c) 2001--2005 Martin Mares <mj@ucw.cz>
+#	(c) 2001--2007 Martin Mares <mj@ucw.cz>
 #
 
 use strict;
@@ -9,7 +9,7 @@ use warnings;
 use Getopt::Long;
 
 use lib 'lib/perl5';
-use Sherlock::Query();
+use Sherlock::Query;
 
 sub do_query($);
 
@@ -22,6 +22,8 @@ my $footer = 1;
 my $stats = 0;
 my $control = 0;
 my $debug = 0;
+my $parsed = 0;
+my $indented = 0;
 
 Getopt::Long::Configure("bundling");
 GetOptions(
@@ -35,14 +37,18 @@ GetOptions(
 	"stats!" => \$stats,
 	"control!" => \$control,
 	"debug!" => \$debug,
+	"parsed!" => \$parsed,
+	"indented!" => \$indented,
 	# Need to repeat that since getopt doesn't support "long|short!" spec
 	"R" => \$requests,
 	"H" => \$headers,
 	"B" => \$body,
 	"F" => \$footer,
+	"P" => \$parsed,
 	"S" => \$stats,
 	"C" => \$control,
-	"d" => \$debug
+	"d" => \$debug,
+	"i" => \$indented,
 ) || die "Syntax: query [<options>] [<query>]
 
 Options:
@@ -56,6 +62,8 @@ Options:
 -S, --stats		Show statistics
 -C, --control		Send a control command instead of query
 -d, --debug		Show a debugging dump
+-P, --parsed		Show the reply as a parsed object instead of raw lines
+-i, --indented		Show sorted attributes and indented subobjects (implies -P)
 ";
 
 my $total_ok = 0;
@@ -82,6 +90,19 @@ if ($stats) {
 }
 exit $total_err ? 1 : 0;
 
+sub
+write_obj($)
+{
+	my ($obj) = @_;
+	if ($indented) {
+		$obj->write_indented(\*STDOUT);
+	} elsif ($parsed) {
+		$obj->write(\*STDOUT);
+	} else {
+		print join("\n", $obj->getarray("RAW")), "\n";
+	}
+}
+
 sub
 do_query($)
 {
@@ -93,20 +114,26 @@ do_query($)
 		$query = "control $query";
 	}
 	print "<<< $query\n" if $requests;
-	$stat = $q->query($query);
+	$stat = $q->query($query, raw => !$parsed);
 	$q->print if $debug;
-	local $, = "\n";
 	print "$stat\n" if $headers;
-	print @{$q->{HEADER}{RAW}}, "\n" if $headers;
+	if ($headers && $q->{HEADER}->get_attrs) {
+		write_obj($q->{HEADER});
+		print "\n";
+	}
 	if ($body) {
 		foreach my $c (@{$q->{CARDS}}) {
-			print @{$c->{RAW}}, "\n";
+			write_obj($c);
+			print "\n";
 		}
 	}
-	print @{$q->{FOOTER}{RAW}}, "\n" if $footer;
+	if ($footer && $q->{FOOTER}->get_attrs) {
+		write_obj($q->{FOOTER});
+		print "\n";
+	}
 	$total_ok++ if $stat =~ /^\+/;
 	$total_err++ if $stat =~ /^-/;
-	$total_cached++ if $q->{HEADER}{'C'};
-	if (my $rt = $q->{FOOTER}{'t'}) { $total_time += $rt; }
-	if (my $rT = $q->{FOOTER}{'T'}) { $rT =~ s/(\w+)=([0-9.]+)/$totals{$1}+=$2/ge; }
+	$total_cached++ if $q->{HEADER}->get('C');
+	if (my $rt = $q->{FOOTER}->get('t')) { $total_time += $rt; }
+	if (my $rT = $q->{FOOTER}->get('T')) { $rT =~ s/(\w+)=([0-9.]+)/$totals{$1}+=$2/ge; }
 }
