@@ -22,6 +22,7 @@ if (@ARGV && $ARGV[0] =~ /\//) {
 	$timetable = $ARGV[0];
 	shift @ARGV;
 }
+@exceptions = ();
 read_config();
 $now = time;
 $cycle = int(($now-$cycle_epoch)/$cycle_alignment);
@@ -30,7 +31,8 @@ $slot = int(($now-$cycle_start_time)/$slot_time);
 $slot_start_time = $cycle_start_time + $slot*$slot_time;
 $shutdown = 0;
 $shutdown_sends_alarm = 0;
-@exceptions = ();
+$shutdown_done = 0;
+$0 = $title if defined $title;
 
 $SIG{QUIT} = $SIG{TERM} = sub {
 	$shutdown = 1;
@@ -53,11 +55,12 @@ if (@exceptions) {
 	msg("I", "Requested out-of-order slots: " . join(' ', @exceptions));
 }
 
-while (!$shutdown) {
+while (!$shutdown_done) {
 	if ($slot == $cycle_slots) {
 		$slot = 0;
 		$cycle++;
 	}
+	$slot = "DOWN" if $shutdown;
 	slot_start();
 	if (@exceptions) {
 		$slot_name = shift @exceptions;
@@ -68,7 +71,7 @@ while (!$shutdown) {
 		print SF "$slot_name\n";
 		close SF;
 	}
-	$slot_special = ($slot_name =~ "^$special_slots\$");
+	$slot_special = $shutdown || ($slot_name =~ "^$special_slots\$");
 	msg("I", "Starting " . ($slot_special ? "special " : "") . "slot $slot_name of cycle $cycle.");
 	$slot_start_time += $slot_time;
 	if ($now >= $slot_start_time - $slot_min && !$slot_special) {
@@ -90,8 +93,9 @@ while (!$shutdown) {
 		}
 		$now = time;
 	}
-	if ($shutdown) {
+	if ($slot eq "DOWN" && $shutdown) {
 		msg("I", "Scheduler shut down.");
+		$shutdown_done = 1;
 	}
 	send_report($progress_mail, "Scheduler status ($cycle:$slot_name)");
 	if (!$shutdown && $now < $slot_start_time && !$slot_special) {
@@ -109,7 +113,7 @@ sub read_config {
 	while (<CF>) {
 		chomp;
 		/^\s*($|#)/ && next;
-		if (/^\$.*;\s*$/) {
+		if (/^(\$|@).*;\s*$/) {
 			eval;
 			$@ eq "" or die "$_: $@";
 		} elsif (/^(\S+)\s+(\d+\+)?(\d+)\s+(\S.*)$/) {
@@ -118,7 +122,9 @@ sub read_config {
 			$action_term[$i] = ($2 ne "") ? $2 : 0;
 			$action_timeout[$i] = $3;
 			$action_cmd[$i] = $4;
-			$action_shortname[$i] = ($action_cmd[$i] =~ /^(\S+)/) ? $1 : "???";
+			if ($action_cmd[$i] =~ m/^{.*}$/) { $action_shortname[$i] = "Perl expression"; }
+			elsif ($action_cmd[$i] =~ /^(\S+)/) { $action_shortname[$i] = $1; }
+			else { $action_shortname[$i] = "???"; }
 		} else {
 			die "Configuration parse error: $_";
 		}
@@ -145,6 +151,12 @@ sub send_report {
 	open(MAIL, "|bin/send-mail -s '$subject' $address") or msg("E", "Unable to send mail with report!");
 	print MAIL "Hello, Sherlock Scheduler speaking out its status report...\n\n";
 	print MAIL "$subject\n\n";
+	print MAIL "Host:                   " . `hostname -f`;
+	print MAIL "Title:                  " . $title if defined $title;
+	print MAIL "Current directory:      " . `pwd`;
+	print MAIL "Timetable:              $timetable\n\n";
+	print MAIL "Date and time           " . `date`;
+
 	if (defined $log_slot_start && open(L, "<$current_logfile")) {
 		seek L, $log_slot_start, 0;
 		while (<L>) { print MAIL $_; }
@@ -155,6 +167,12 @@ sub send_report {
 
 sub run_cmd {
 	my ($cmd, $term, $timeout, $shortname) = @_;
+	if ($cmd =~ m/^{.*}$/) {
+		msg("D", "Evaluating $cmd") if $trace;
+		$cmd = eval $cmd;
+		if ($cmd eq "") { return; }
+		elsif ($cmd =~ /^(\S+)/) { $shortname = $1; }
+	}
 	msg("D", "Running '$cmd' term=$term timeout=$timeout") if $trace;
 	my $pid = fork;
 	defined $pid or fatal "fork failed: $!";
