@@ -13,7 +13,6 @@ IN_SSH_AGENT=
 SSH_OPTIONS=
 SHCP_OPTIONS=
 if [ "$1" == "--have-agent" ] ; then
-	ssh-add ~/.ssh/send-index-key
 	IN_SSH_AGENT=1
 	SSH_OPTIONS="-A"
 	SHCP_OPTIONS="-a -A"
@@ -37,6 +36,7 @@ Options:
 --limit=...		Transfer speed limit (in MB/s)
 --swap-delay=...	Time delay between index swap (in sec)
 --agent			Start ssh-agent and use it for remote authentication
+--key=...		Add a custom key to the ssh-agent (default=~/.ssh/send-index-key)
 --append		Append files to partially uploaded index
 --files=...		Index files to transfer (default=standard index files)
 --extra-files=...	Additional files to transfer
@@ -45,7 +45,6 @@ Options:
 Note: * When uploading is requested, you can specify only one index to transfer
       * When \`--swaponly' is requested, you can specify more indices to swap at once
         with \`--remote-index-name="index1 index2 index3 ..."'
-      * If the --agent switch is used, it must precede all other switches.
 EOF
 	exit 1
 }
@@ -64,6 +63,7 @@ APPEND=
 INDEX_FILES="cards card-attrs card-prints sites references lexicon stems string-map string-hash parameters"
 EXTRA_FILES=
 KEEP_OLD=
+KEY=~/.ssh/send-index-key
 
 while [ "${1:0:2}" == "--" ] ; do
 	case "$1" in
@@ -89,6 +89,8 @@ while [ "${1:0:2}" == "--" ] ; do
 					;;
 		--agent)		[ -z "$IN_SSH_AGENT" ] && exec ssh-agent "$0" --have-agent "$@"
 					;;
+		--key=*)		KEY="${1:6}"
+					;;
 		--append)		APPEND=1
 					;;
 		--files=*)		INDEX_FILES="${1:8}"
@@ -104,6 +106,10 @@ while [ "${1:0:2}" == "--" ] ; do
 done
 [ -n "$1" ] || usage
 
+if [ -n "$IN_SSH_AGENT" ] ; then
+	ssh-add "$KEY"
+fi
+
 INDEX_FILES="$INDEX_FILES $EXTRA_FILES"
 
 function log
