@@ -39,15 +39,16 @@
 ||	MD5_SIZE b	md5 of random key and adler of file
 */
 #include "sherlock/sherlock.h"
-#include "lib/conf.h"
-#include "lib/getopt.h"
-#include "lib/lfs.h"
-#include "lib/md5.h"
-#include "lib/lizard.h"
-#include "lib/stkstring.h"
-#include "lib/clists.h"
-#include "lib/chartype.h"
-#include "lib/mempool.h"
+#include "ucw/conf.h"
+#include "ucw/getopt.h"
+#include "ucw/lfs.h"
+#include "ucw/md5.h"
+#include "ucw/lizard.h"
+#include "ucw/stkstring.h"
+#include "ucw/clists.h"
+#include "ucw/chartype.h"
+#include "ucw/mempool.h"
+#include "ucw/string.h"
 
 #include <string.h>
 #include <stdio.h>
@@ -121,9 +122,9 @@ struct file_info {
   byte md5[MD5_SIZE];
 };
 
-static byte *rundir = "run";
-static byte *ssh_program = "ssh";
-static byte *shcp_program = "bin/shcp";
+static char *rundir = "run";
+static char *ssh_program = "ssh";
+static char *shcp_program = "bin/shcp";
 
 static int buff_size = 65536;
 static int tcpb_size = 65536;
@@ -139,7 +140,7 @@ static unsigned int limit_usec;
 static unsigned int header_timeout = 60;
 static unsigned int timeout = 60;
 #define MAX_SSH_OPTIONS 20
-static byte *ssh_options[MAX_SSH_OPTIONS+1];
+static char *ssh_options[MAX_SSH_OPTIONS+1];
 static struct timeval limit_init_time;
 static s64 limit_init_size;
 static int file_count;
@@ -299,9 +300,9 @@ dsec_diff(struct timeval a, struct timeval b)
 static int
 is_regular_file(const byte *name)
 {
-  sh_stat_t st;
+  ucw_stat_t st;
 
-  if(sh_stat(name, &st) < 0)
+  if(ucw_stat(name, &st) < 0)
     return -1;
   else if(S_ISREG(st.st_mode))
     return 1;
@@ -312,9 +313,9 @@ is_regular_file(const byte *name)
 static int
 is_dir(const byte *name)
 {
-  sh_stat_t st;
+  ucw_stat_t st;
 
-  if(!sh_stat(name, &st)) {
+  if(!ucw_stat(name, &st)) {
     if(S_ISDIR(st.st_mode))
       return 1;
   }
@@ -459,7 +460,7 @@ shcp_read_write(struct host_info *rh, struct host_info **wh, void *buff, int len
       }
     }
   }
-  file_current->adler = update_adler32(file_current->adler, buff, len);
+  file_current->adler = adler32_update(file_current->adler, buff, len);
 }
 
 static void *
@@ -567,10 +568,10 @@ open_file(struct host_info *hi, struct file_info *fi)
   rw_flags = (hi->function == SHCP_SEND) ? O_WRONLY | O_CREAT | O_TRUNC : O_RDONLY;
 
   if(odirect)
-#ifdef O_DIRECT
+#ifdef CONFIG_DIRECT_IO
     rw_flags |= O_DIRECT;
 #else
-    die("Direct mode is not supported on this platform");
+    die("O_DIRECT not supported on this platform or direct I/O disabled by configure -CONFIG_DIRECT_IO");
 #endif
 
   if(hi->function == SHCP_SEND) {
@@ -584,7 +585,7 @@ open_file(struct host_info *hi, struct file_info *fi)
   else
     hi->file = fi->path;
 
-  if((hi->descriptor = sh_open(hi->file, rw_flags, 0666)) < 0)
+  if((hi->descriptor = ucw_open(hi->file, rw_flags, 0666)) < 0)
     DIE("Can't open file %s: %m", hi->file);
 }
 
@@ -655,17 +656,17 @@ read_file_header(struct host_info *hi, struct file_info *fi)
   ASSERT(fi);
   ASSERT(hi->function == SHCP_RECEIVE);
 
-  fi->adler = update_adler32(0, NULL, 0);
+  fi->adler = adler32_update(0, NULL, 0);
   hi->path = fi->path;
 
   switch(hi->type) {
     case SHCP_FILE:
       open_file(hi, fi);
-      if((fi->size = sh_seek(hi->descriptor, 0, SEEK_END)) < 0)
+      if((fi->size = ucw_seek(hi->descriptor, 0, SEEK_END)) < 0)
         DIE("Can't get stats for file %s: %m", hi->path);
-      sh_seek(hi->descriptor, 0, SEEK_SET);
+      ucw_seek(hi->descriptor, 0, SEEK_SET);
       randomkey(keybin, MD5_SIZE);
-      md5_to_hex(keybin, fi->key);
+      mem_to_hex(fi->key, keybin, MD5_SIZE, MEM_TO_HEX_UPCASE);
       TRACE("Creating file headers - size: %lld bytes, key: %s", (long long) fi->size, fi->key);
       break;
 
@@ -711,16 +712,16 @@ static void
 read_file_footer(struct host_info *hi, struct file_info *fi)
 {
   byte sig[MD5_SIZE];
-  struct MD5Context md5c;
+  md5_context md5c;
 
   ASSERT(hi);
   ASSERT(fi);
   ASSERT(hi->function == SHCP_RECEIVE);
 
-  MD5Init(&md5c);
-  MD5Update(&md5c, fi->key, strlen(fi->key));
-  MD5Update(&md5c, (byte *) &fi->adler, sizeof(fi->adler));
-  MD5Final(fi->md5, &md5c);
+  md5_init(&md5c);
+  md5_update(&md5c, fi->key, strlen(fi->key));
+  md5_update(&md5c, (byte *) &fi->adler, sizeof(fi->adler));
+  memcpy(fi->md5, md5_final(&md5c), MD5_SIZE);
 
   if(hi->type != SHCP_FILE) {
     sread(hi, sig, MD5_SIZE);
@@ -739,7 +740,7 @@ write_file_footer(struct host_info *hi, struct file_info *fi)
 
   if(hi->type == SHCP_FILE) {
     pthread_join(hi->thread, NULL);
-    sh_ftruncate(hi->descriptor, fi->size);
+    ucw_ftruncate(hi->descriptor, fi->size);
     close(hi->descriptor);
   }
   else
