@@ -5,7 +5,7 @@
  */
 
 #include "sherlock/sherlock.h"
-#include "lib/lfs.h"
+#include "ucw/lfs.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -14,24 +14,24 @@ int
 main(int argc, char **argv)
 {
   int i;
-  byte msg[256], *p;
+  byte buf[256], *p;
   int err=0;
 
   log_init("sizer");
   if (argc < 2)
     die("Usage: sizer <file> ...");
-  p = msg;
+  p = buf;
   for (i=1; i<argc; i++)
     {
       int fd;
-      if (p + strlen(argv[i]) + 32 > msg + sizeof(msg))
+      if (p + strlen(argv[i]) + 32 > buf + sizeof(buf))
 	{
 	  *p = 0;
-	  log(L_INFO, "%s", msg+1);
-	  p = msg;
+	  log(L_INFO, "%s", buf+1);
+	  p = buf;
 	}
       p += sprintf(p, " %s=", argv[i]);
-      fd = sh_open(argv[i], O_RDONLY, 0);
+      fd = ucw_open(argv[i], O_RDONLY, 0);
       if (fd < 0)
 	{
 	  *p++ = '?';
@@ -39,7 +39,7 @@ main(int argc, char **argv)
 	}
       else
 	{
-	  sh_off_t len = sh_seek(fd, 0, SEEK_END);
+	  ucw_off_t len = ucw_seek(fd, 0, SEEK_END);
 	  if (len < 0)
 	    {
 	      *p++ = '?';
@@ -50,10 +50,10 @@ main(int argc, char **argv)
 	  close(fd);
 	}
     }
-  if (p > msg)
+  if (p > buf)
     {
       *p = 0;
-      log(L_INFO, "%s", msg+1);
+      log(L_INFO, "%s", buf+1);
     }
   return err;
 }
