@@ -4,6 +4,7 @@
 #
 # (c) 2003-2005 Tomas Valla <tom@ucw.cz>
 # (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
+# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 #
 
 use lib 'lib/perl5';
@@ -17,8 +18,6 @@ analyze_options() or die
 # Recompute whole interfile, because we don't remember which index we are processing
 $force = 1;
 
-read_config("INDEXER_INDICES");
-my @indices = split /\s+/, read_config_value('INDEXER_INDICES');
 my $last_index = '';
 my $sending = 0;
 
@@ -57,6 +56,7 @@ sub chew_indexerlog {
 		lang_fr
 		lang_es
 		lang_it
+		lang_ru
 		indexing_start
 		indexing_end
 		idxcopy_start
@@ -81,12 +81,9 @@ sub chew_indexerlog {
 		size_refs
 		size_string
 		size_total );
-
-	foreach my $index (@indices) {
-		push @keys, "chewer_card_$index";
-		push @keys, "size_total_$index";
-		push @keys, "size_lexicon_$index";
-	}
+		# chewer_card_*
+		# size_total_*
+		# size_lexicon_*
 
 	my $hdr_string="#@\n";
 
@@ -108,7 +105,7 @@ sub chew_indexerlog {
 		defined $start_time or $start_time = $time - ($time % $quantum);
 
 		if ($time>$start_time+$quantum) {
-			for my $k (sort @keys) {
+			for my $k (sort keys %d) {
 				defined $d{$k} and print $out "$start_time\t$k\t$d{$k}\n";
 			}
 
@@ -130,7 +127,7 @@ sub chew_indexerlog {
 			$d{'scan_obj'} = $1;
 			$d{'scan_card'} = $2;
 		}
-		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) new\)/) {
+		elsif (/\[scanner] Scanned (\d+) objects \((\d+) ok, (\d+) err, (\d+) robots, (\d+) (new|skeletons)\)/) {
 			$d{'scan_obj'} = $1;
 			$d{'scan_doc'} = $2 + $3 + $4;
 		}
@@ -163,40 +160,23 @@ sub chew_indexerlog {
 			if (/string-map=(\d+)/) { $d{'size_string'} = $1; }
 			if (/total index size is (\d+)/) { $d{"size_total_$last_index"} = $1; }
 		}
-		elsif (/\[ireport] Filetypes in: unknown=(\d+) html=(\d+) pdf=(\d+) text=(\d+) msword=(\d+) excel=(\d+) RFU6=0 RFU7=0 jpeg=(\d+) png=(\d+) gif=(\d+)/) {
-			$d{'ft_in_unknown'} = $1;
-			$d{'ft_in_html'} = $2;
-			$d{'ft_in_pdf'} = $3;
-			$d{'ft_in_text'} = $4;
-			$d{'ft_in_msword'} = $5;
-			$d{'ft_in_excel'} = $6;
-			$d{'ft_in_jpeg'} = $7;
-			$d{'ft_in_png'} = $8;
-			$d{'ft_in_gif'} = $9;
+		elsif (/\[ireport] Filetypes in: (.+)$/) {
+			my %ft = map { split /=/, $_, 2 } split(/\s+/, $1);
+			for my $i (keys %ft) {
+				$d{"ft_in_" . $i} = $ft{$i} if $i !~ /^RFU\d+$/;
+			}
 		}
-		elsif (/\[ireport] Filetypes out: unknown=(\d+) html=(\d+) pdf=(\d+) text=(\d+) msword=(\d+) excel=(\d+) RFU6=0 RFU7=0 jpeg=(\d+) png=(\d+) gif=(\d+)/) {
-			$d{'ft_out_unknown'} = $1;
-			$d{'ft_out_html'} = $2;
-			$d{'ft_out_pdf'} = $3;
-			$d{'ft_out_text'} = $4;
-			$d{'ft_out_msword'} = $5;
-			$d{'ft_out_excel'} = $6;
-			$d{'ft_out_jpeg'} = $7;
-			$d{'ft_out_png'} = $8;
-			$d{'ft_out_gif'} = $9;
+		elsif (/\[ireport] Filetypes out: (.+)$/) {
+			my %ft = map { split /=/, $_, 2 } split(/\s+/, $1);
+			for my $i (keys %ft) {
+				$d{"ft_out_" . $i} = $ft{$i} if $i !~ /^RFU\d+$/;
+			}
 		}
-		elsif (/\[ireport] Languages: \?\?=(\d+) en=(\d+) cs=(\d+) sk=(\d+) pl=(\d+) hu=(\d+) de=(\d+) nl=(\d+) fr=(\d+) es=(\d+) it=(\d+)/) {
-			$d{'lang_unknown'} = $1;
-			$d{'lang_en'} = $2;
-			$d{'lang_cs'} = $3;
-			$d{'lang_sk'} = $4;
-			$d{'lang_pl'} = $5;
-			$d{'lang_hu'} = $6;
-			$d{'lang_de'} = $7;
-			$d{'lang_nl'} = $8;
-			$d{'lang_fr'} = $9;
-			$d{'lang_es'} = $10;
-			$d{'lang_it'} = $11;
+		elsif (/\[ireport] Languages: (.+)$/) {
+			my %lang = map { split /=/, $_, 2 } split(/\s+/, $1);
+			for my $i (keys %lang) {
+				$d{"lang_" . (($i eq '??') ? 'unknown' : $i)} = $lang{$i};
+			}
 		}
 
 		if (!$sending && /\[send-indices]/) {
@@ -205,10 +185,9 @@ sub chew_indexerlog {
 		}
 		elsif ($sending && /\[scheduler] All indices send/) {
 			$d{'idxcopy_end'} = $time;
-			$sending = 0;
 		}
 	}
-	for my $k (sort @keys) {
+	for my $k (sort keys %d) {
 		defined $d{$k} and print $out "$start_time\t$k\t$d{$k}\n";
 	}
 
