@@ -16,8 +16,8 @@
 # force		forces redrawing picture even if already in cache (optional)
 
 use lib 'lib/perl5';
-use Sherlock::CGI;
-use Sherlock::Ulimit;
+use UCW::CGI;
+use UCW::Ulimit;
 use strict;
 use warnings;
 use Time::Local;
@@ -32,7 +32,7 @@ my $error_pic = "error.png";
 
 $ENV{'PATH'} = "/opt/bin:/usr/local/bin:/usr/bin";
 
-Sherlock::Ulimit::setlimit($Sherlock::Ulimit::AS, 40000000,40000000);
+UCW::Ulimit::setlimit($UCW::Ulimit::AS, 40000000,40000000);
 
 get_stat();
 
@@ -55,7 +55,7 @@ sub get_stat {
 		force => { var => \$force, default => 0 },
 		options => { var => \$options, default => "" },
 	);
-	Sherlock::CGI::parse_args(\%pars);
+	UCW::CGI::parse_args(\%pars);
 	if ($prefix =~ /\.\./) {
 		response("Forbidden prefix \"".$prefix."\"");
 	}
