@@ -2,6 +2,7 @@
 # Watson statistics
 # (c) 2004 Tomas Valla <tom@ucw.cz>
 # (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
+# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 
 use lib "lib/perl5";
 use Sherlock::Watsonlib;
@@ -14,14 +15,8 @@ my $pic = new Sherlock::Watsonlib::picture $stat_picname, "Indexer: documents";
 $pic->{INIT_CMD}.="set yrange [0:]\nset data style linespoints\n";
 my $a = $pic->new_plot("Cards on scanner output");
 my $b = $pic->new_plot("Documents on scanner input");
-my %c = ();
 
-read_config('INDEXER_INDICES');
-my @indices = split /\s+/, read_config_value('INDEXER_INDICES');
-
-foreach my $index (@indices) {
-	$c{"$index"} = $pic->new_plot("Cards on chewer output (index $index)");
-}
+my %known_index = ();
 
 compute_stat($stat_begintime,$stat_endtime,$stat_prefix,\&proc_line);
 
@@ -31,8 +26,10 @@ sub proc_line {
 	my %r = @_;
 	$pic->plot_value($a, $r{'start_time'}, $r{'scan_card'}) if defined $r{'scan_card'};
 	$pic->plot_value($b, $r{'start_time'}, $r{'scan_doc'}) if defined $r{'scan_doc'};
-	foreach my $index (keys %c) {
-		$pic->plot_value($c{"$index"}, $r{'start_time'}, $r{"chewer_card_$index"}) if defined $r{"chewer_card_$index"};
+	for my $k (keys %r) {
+		if ($k =~ /chewer_card_(\w+)/) {
+			defined $known_index{$1} or $known_index{$1} = $pic->new_plot("Cards on chewer output (index $1)");
+			$pic->plot_value($known_index{$1}, $r{'start_time'}, $r{$k});
+		}
 	}
 }
-
