@@ -2,6 +2,7 @@
 # Watson statistics
 # (c) 2004 Tomas Valla <tom@ucw.cz>
 # (c) 2006 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
+# (c) 2007 Pavel Charvat <pchar@ucw.cz>
 
 use lib 'lib/perl5';
 use Sherlock::Watsonlib;
@@ -12,20 +13,19 @@ stat_options() or die stat_usage();
 
 my $pic = new Sherlock::Watsonlib::picture $stat_picname, "Indexer: Index size";
 $pic->{INIT_CMD}.="set yrange [0:]\nset data style linespoints\n";
-my %t = ();
 
-read_config('INDEXER_INDICES');
-my @indices = split /\s+/, read_config_value('INDEXER_INDICES');
+my %known_index = ();
 
-foreach my $index (@indices) {
-	$t{"$index"} = $pic->new_plot("Index size - $index (MB)");
-}
-
-compute_stat($stat_begintime,$stat_endtime,$stat_prefix, sub {
-	my %r = @_;
-	foreach my $index (keys %t) {
-		$pic->plot_value($t{"$index"}, $r{'start_time'}, $r{"size_total_$index"}/(1024*1024)) if defined $r{"size_total_$index"};
-	}
-});
+compute_stat($stat_begintime,$stat_endtime,$stat_prefix,\&proc_line);
 
 $pic->draw_picture;
+
+sub proc_line {
+	my %r = @_;
+	for my $k (keys %r) {
+		if ($k =~ /size_total_(\w+)/) {
+			defined $known_index{$1} or $known_index{$1} = $pic->new_plot("Index size - $1 (MB)");
+			$pic->plot_value($known_index{$1}, $r{'start_time'}, $r{$k}/(1024*1024));
+		}
+	}
+}
