@@ -3,7 +3,7 @@
 
 if [ -s log/watsonerr ]; then
 	echo "Watson encountered some errors during periodical runs"
-	echo "Please read log/watsonerr and delete it afterwards"
+	echo "Look to Watson's log for more information"
 	echo
-	tail log/watsonerr
-fi 1>&2
+	tail -n200 log/watsonerr
+fi
