@@ -3,7 +3,7 @@
 #
 # (c) 2003-2005 Tomas Valla <tom@ucw.cz>
 # (c) 2005 Vladimir Jelen <vladimir.jelen@netcentrum.cz>
-# (c) 2007 Pavel Charvat <pchar@ucw.cz>
+# (c) 2007-2009 Pavel Charvat <pchar@ucw.cz>
 
 if [ ! -f cf/watson ] ; then
 	echo "Missing config file cf/watson !" >&2
@@ -20,6 +20,8 @@ INDEXER_PREFIX=indexer-
 LEX_PREFIX=lex-
 LEX_LOG_SUBDIR=words/
 MUX_PREFIX=mux-
+SPRAY_PREFIX=spray-
+SPROXY_PREFIX=sproxy-
 ERROR_MAIL=
 
 . cf/watson
@@ -55,8 +57,8 @@ function download () {	# download <server_type>
 		elif [ "$host" != "$host$path$log_path$log_subdir$prefix*" ] ; then
 			echo "Fetching $host:$path$log_path$log_subdir$prefix*"
 			mkdir -p log/$server log/inter/$server && \
-				rsync -vuat "$host:$path$log_path$log_subdir$prefix*" log/$server/ && \
-				ssh "$host" "${path}bin/rotate-log" "$delete" "$delete" "$path$log_path$log_subdir$prefix*" && \
+				rsync -vuat -e "ssh -i $SSH_KEY" "$host:$path$log_path$log_subdir$prefix*" log/$server/ && \
+				ssh -i "$SSH_KEY" "$host" "${path}bin/rotate-log" "$delete" "$delete" "$path$log_path$log_subdir$prefix*" && \
 				bin/rotate-log "$compress" "$backup" log/$server/$prefix*
 		fi
 	done
@@ -81,26 +83,27 @@ function check () {	# check <server_type> <analyse_command> <result_prefix>
 	done
 }
 
-LOCK=lock/WATSON_LOCK
-
-if ! mkdir $LOCK 2>/dev/null ; then
-	bin/logger watson W "Watson locked: already running bin/watson"
-	exit 1
-fi
-
 # Rotate and download logs
 echo "Watson: Rotating and downloading logs"
 download SEARCH
-download GATHERD
 download INDEXER
 download LEX
+#ifdef CONFIG_SHEPHERD_PROTOCOL
+download REAP
+download SHEPHERD
+#endif
+download GATHERD
 
 # Analyse logs
 echo "Watson: Analyzing logs"
 check SEARCH bin/analyze-search
-check GATHERD bin/analyze-gatherd
 check INDEXER bin/analyze-indexer
 check INDEXER bin/analyze-indexer-status status-
+#ifdef CONFIG_SHEPHERD_PROTOCOL
+check SHEPHERD bin/analyze-shepherd
+check SHEPHERD_SERVERS bin/analyze-shepherd-status status-
+#endif
+check GATHERD bin/analyze-gatherd
 
 # Report errors
 if [ -n "$ERROR_MAIL" ] ; then
@@ -113,5 +116,3 @@ if [ -n "$ERROR_MAIL" ] ; then
 fi
 
 echo "Watson: Done."
-
-rmdir $LOCK
